import { Analysis } from "./analyzer"

export const serialize = (analysis: Analysis): Array<string | number> => [
  analysis.name,
  analysis.algorithm,
  analysis.dataSetName,
  analysis.dataSetSize,
  analysis.actualOperations,
  analysis.expectedOperationsBest,
  analysis.expectedOperationsAverage,
  analysis.expectedOperationsWorst,
]

export const deserialize = (array: Array<string | number>): Analysis =>
  ({
    name: array[0] as string,
    algorithm: array[1] as string,
    dataSetName: array[2] as string,
    dataSetSize: array[3] as number,
    actualOperations: array[4] as number,
    expectedOperationsBest: array[5] as number,
    expectedOperationsAverage: array[6] as number,
    expectedOperationsWorst: array[7] as number,
  } as Analysis)

export function persist(name: string, analyses: Analysis[]) {
  const serialized = analyses.map(serialize)
  const data = JSON.parse(localStorage.getItem("preanalyzed") || "{}")
  data[name] = serialized
  const json = JSON.stringify(data)
  localStorage.setItem("preanalyzed", json)
}

export const data = {
  "quick-sort-random": [
    ["Random", "Quick Sort", "Random", 10, 36, 33, 33, 100],
    ["Random", "Quick Sort", "Random", 15, 62, 59, 59, 225],
    ["Random", "Quick Sort", "Random", 20, 104, 86, 86, 400],
    ["Random", "Quick Sort", "Random", 30, 165, 147, 147, 900],
    ["Random", "Quick Sort", "Random", 40, 265, 213, 213, 1600],
    ["Random", "Quick Sort", "Random", 60, 387, 354, 354, 3600],
    ["Random", "Quick Sort", "Random", 80, 644, 506, 506, 6400],
    ["Random", "Quick Sort", "Random", 100, 718, 664, 664, 10000],
    ["Random", "Quick Sort", "Random", 150, 1344, 1084, 1084, 22500],
    ["Random", "Quick Sort", "Random", 200, 1697, 1529, 1529, 40000],
    ["Random", "Quick Sort", "Random", 300, 3269, 2469, 2469, 90000],
    ["Random", "Quick Sort", "Random", 400, 4047, 3458, 3458, 160000],
    ["Random", "Quick Sort", "Random", 600, 6372, 5537, 5537, 360000],
    ["Random", "Quick Sort", "Random", 800, 10055, 7715, 7715, 640000],
    ["Random", "Quick Sort", "Random", 1000, 13096, 9966, 9966, 1000000],
    ["Random", "Quick Sort", "Random", 1500, 19913, 15826, 15826, 2250000],
    ["Random", "Quick Sort", "Random", 2000, 29103, 21932, 21932, 4000000],
    ["Random", "Quick Sort", "Random", 3000, 44476, 34652, 34652, 9000000],
    ["Random", "Quick Sort", "Random", 4000, 58851, 47863, 47863, 16000000],
    ["Random", "Quick Sort", "Random", 6000, 93981, 75304, 75304, 36000000],
    ["Random", "Quick Sort", "Random", 8000, 133043, 103726, 103726, 64000000],
  ],
  "all-algorithms-random": [
    ["Bubble Sort", "Bubble Sort", "Random", 10, 44, 10, 100, 100],
    ["Counting Sort", "Counting Sort", "Random", 10, 48, 20, 20, 20],
    ["Heap Sort", "Heap Sort", "Random", 10, 36, 33, 33, 33],
    ["Insertion Sort", "Insertion Sort", "Random", 10, 20, 10, 100, 100],
    ["Merge Sort", "Merge Sort", "Random", 10, 68, 33, 33, 33],
    ["Quick Sort", "Quick Sort", "Random", 10, 38, 33, 33, 100],
    ["Selection Sort", "Selection Sort", "Random", 10, 55, 100, 100, 100],
    ["Tim Sort", "Tim Sort", "Random", 10, 27, 10, 33, 33],
    ["Bubble Sort", "Bubble Sort", "Random", 15, 192, 15, 225, 225],
    ["Counting Sort", "Counting Sort", "Random", 15, 75, 30, 30, 30],
    ["Heap Sort", "Heap Sort", "Random", 15, 58, 59, 59, 59],
    ["Insertion Sort", "Insertion Sort", "Random", 15, 79, 15, 225, 225],
    ["Merge Sort", "Merge Sort", "Random", 15, 118, 59, 59, 59],
    ["Quick Sort", "Quick Sort", "Random", 15, 62, 59, 59, 225],
    ["Selection Sort", "Selection Sort", "Random", 15, 120, 225, 225, 225],
    ["Tim Sort", "Tim Sort", "Random", 15, 98, 15, 59, 59],
    ["Bubble Sort", "Bubble Sort", "Random", 20, 336, 20, 400, 400],
    ["Counting Sort", "Counting Sort", "Random", 20, 100, 40, 40, 40],
    ["Heap Sort", "Heap Sort", "Random", 20, 95, 86, 86, 86],
    ["Insertion Sort", "Insertion Sort", "Random", 20, 82, 20, 400, 400],
    ["Merge Sort", "Merge Sort", "Random", 20, 176, 86, 86, 86],
    ["Quick Sort", "Quick Sort", "Random", 20, 112, 86, 86, 400],
    ["Selection Sort", "Selection Sort", "Random", 20, 210, 400, 400, 400],
    ["Tim Sort", "Tim Sort", "Random", 20, 126, 20, 86, 86],
    ["Bubble Sort", "Bubble Sort", "Random", 30, 775, 30, 900, 900],
    ["Counting Sort", "Counting Sort", "Random", 30, 150, 60, 60, 60],
    ["Heap Sort", "Heap Sort", "Random", 30, 152, 147, 147, 147],
    ["Insertion Sort", "Insertion Sort", "Random", 30, 262, 30, 900, 900],
    ["Merge Sort", "Merge Sort", "Random", 30, 296, 147, 147, 147],
    ["Quick Sort", "Quick Sort", "Random", 30, 162, 147, 147, 900],
    ["Selection Sort", "Selection Sort", "Random", 30, 465, 900, 900, 900],
    ["Tim Sort", "Tim Sort", "Random", 30, 309, 30, 147, 147],
    ["Bubble Sort", "Bubble Sort", "Random", 40, 1558, 40, 1600, 1600],
    ["Counting Sort", "Counting Sort", "Random", 40, 200, 80, 80, 80],
    ["Heap Sort", "Heap Sort", "Random", 40, 212, 213, 213, 213],
    ["Insertion Sort", "Insertion Sort", "Random", 40, 532, 40, 1600, 1600],
    ["Merge Sort", "Merge Sort", "Random", 40, 432, 213, 213, 213],
    ["Quick Sort", "Quick Sort", "Random", 40, 237, 213, 213, 1600],
    ["Selection Sort", "Selection Sort", "Random", 40, 820, 1600, 1600, 1600],
    ["Tim Sort", "Tim Sort", "Random", 40, 387, 40, 213, 213],
    ["Bubble Sort", "Bubble Sort", "Random", 60, 3355, 60, 3600, 3600],
    ["Counting Sort", "Counting Sort", "Random", 60, 300, 120, 120, 120],
    ["Heap Sort", "Heap Sort", "Random", 60, 358, 354, 354, 354],
    ["Insertion Sort", "Insertion Sort", "Random", 60, 1011, 60, 3600, 3600],
    ["Merge Sort", "Merge Sort", "Random", 60, 712, 354, 354, 354],
    ["Quick Sort", "Quick Sort", "Random", 60, 379, 354, 354, 3600],
    ["Selection Sort", "Selection Sort", "Random", 60, 1830, 3600, 3600, 3600],
    ["Tim Sort", "Tim Sort", "Random", 60, 829, 60, 354, 354],
    ["Bubble Sort", "Bubble Sort", "Random", 80, 5346, 80, 6400, 6400],
    ["Counting Sort", "Counting Sort", "Random", 80, 394, 160, 160, 160],
    ["Heap Sort", "Heap Sort", "Random", 80, 523, 506, 506, 506],
    ["Insertion Sort", "Insertion Sort", "Random", 80, 1781, 80, 6400, 6400],
    ["Merge Sort", "Merge Sort", "Random", 80, 1024, 506, 506, 506],
    ["Quick Sort", "Quick Sort", "Random", 80, 569, 506, 506, 6400],
    ["Selection Sort", "Selection Sort", "Random", 80, 3240, 6400, 6400, 6400],
    ["Tim Sort", "Tim Sort", "Random", 80, 882, 80, 506, 506],
    ["Bubble Sort", "Bubble Sort", "Random", 100, 9696, 100, 10000, 10000],
    ["Counting Sort", "Counting Sort", "Random", 100, 492, 200, 200, 200],
    ["Heap Sort", "Heap Sort", "Random", 100, 682, 664, 664, 664],
    ["Insertion Sort", "Insertion Sort", "Random", 100, 2578, 100, 10000, 10000],
    ["Merge Sort", "Merge Sort", "Random", 100, 1344, 664, 664, 664],
    ["Quick Sort", "Quick Sort", "Random", 100, 732, 664, 664, 10000],
    ["Selection Sort", "Selection Sort", "Random", 100, 5050, 10000, 10000, 10000],
    ["Tim Sort", "Tim Sort", "Random", 100, 1255, 100, 664, 664],
    ["Bubble Sort", "Bubble Sort", "Random", 150, 20989, 150, 22500, 22500],
    ["Counting Sort", "Counting Sort", "Random", 150, 746, 300, 300, 300],
    ["Heap Sort", "Heap Sort", "Random", 150, 1110, 1084, 1084, 1084],
    ["Insertion Sort", "Insertion Sort", "Random", 150, 5320, 150, 22500, 22500],
    ["Merge Sort", "Merge Sort", "Random", 150, 2188, 1084, 1084, 1084],
    ["Quick Sort", "Quick Sort", "Random", 150, 1146, 1084, 1084, 22500],
    ["Selection Sort", "Selection Sort", "Random", 150, 11325, 22500, 22500, 22500],
    ["Tim Sort", "Tim Sort", "Random", 150, 1828, 150, 1084, 1084],
    ["Bubble Sort", "Bubble Sort", "Random", 200, 37386, 200, 40000, 40000],
    ["Counting Sort", "Counting Sort", "Random", 200, 1000, 400, 400, 400],
    ["Heap Sort", "Heap Sort", "Random", 200, 1566, 1529, 1529, 1529],
    ["Insertion Sort", "Insertion Sort", "Random", 200, 10361, 200, 40000, 40000],
    ["Merge Sort", "Merge Sort", "Random", 200, 3088, 1529, 1529, 1529],
    ["Quick Sort", "Quick Sort", "Random", 200, 1741, 1529, 1529, 40000],
    ["Selection Sort", "Selection Sort", "Random", 200, 20100, 40000, 40000, 40000],
    ["Tim Sort", "Tim Sort", "Random", 200, 2804, 200, 1529, 1529],
    ["Bubble Sort", "Bubble Sort", "Random", 300, 81270, 300, 90000, 90000],
    ["Counting Sort", "Counting Sort", "Random", 300, 1500, 600, 600, 600],
    ["Heap Sort", "Heap Sort", "Random", 300, 2515, 2469, 2469, 2469],
    ["Insertion Sort", "Insertion Sort", "Random", 300, 22448, 300, 90000, 90000],
    ["Merge Sort", "Merge Sort", "Random", 300, 4976, 2469, 2469, 2469],
    ["Quick Sort", "Quick Sort", "Random", 300, 2882, 2469, 2469, 90000],
    ["Selection Sort", "Selection Sort", "Random", 300, 45150, 90000, 90000, 90000],
    ["Tim Sort", "Tim Sort", "Random", 300, 4149, 300, 2469, 2469],
    ["Bubble Sort", "Bubble Sort", "Random", 400, 152781, 400, 160000, 160000],
    ["Counting Sort", "Counting Sort", "Random", 400, 2000, 800, 800, 800],
    ["Heap Sort", "Heap Sort", "Random", 400, 3524, 3458, 3458, 3458],
    ["Insertion Sort", "Insertion Sort", "Random", 400, 38913, 400, 160000, 160000],
    ["Merge Sort", "Merge Sort", "Random", 400, 6976, 3458, 3458, 3458],
    ["Quick Sort", "Quick Sort", "Random", 400, 4272, 3458, 3458, 160000],
    ["Selection Sort", "Selection Sort", "Random", 400, 80200, 160000, 160000, 160000],
    ["Tim Sort", "Tim Sort", "Random", 400, 6268, 400, 3458, 3458],
    ["Bubble Sort", "Bubble Sort", "Random", 600, 353388, 600, 360000, 360000],
    ["Counting Sort", "Counting Sort", "Random", 600, 2990, 1200, 1200, 1200],
    ["Heap Sort", "Heap Sort", "Random", 600, 5660, 5537, 5537, 5537],
    ["Insertion Sort", "Insertion Sort", "Random", 600, 86643, 600, 360000, 360000],
    ["Merge Sort", "Merge Sort", "Random", 600, 11152, 5537, 5537, 5537],
    ["Quick Sort", "Quick Sort", "Random", 600, 6751, 5537, 5537, 360000],
    ["Selection Sort", "Selection Sort", "Random", 600, 180300, 360000, 360000, 360000],
    ["Tim Sort", "Tim Sort", "Random", 600, 9291, 600, 5537, 5537],
    ["Bubble Sort", "Bubble Sort", "Random", 800, 627183, 800, 640000, 640000],
    ["Counting Sort", "Counting Sort", "Random", 800, 3998, 1600, 1600, 1600],
    ["Heap Sort", "Heap Sort", "Random", 800, 7833, 7715, 7715, 7715],
    ["Insertion Sort", "Insertion Sort", "Random", 800, 159904, 800, 640000, 640000],
    ["Merge Sort", "Merge Sort", "Random", 800, 15552, 7715, 7715, 7715],
    ["Quick Sort", "Quick Sort", "Random", 800, 9016, 7715, 7715, 640000],
    ["Selection Sort", "Selection Sort", "Random", 800, 320400, 640000, 640000, 640000],
    ["Tim Sort", "Tim Sort", "Random", 800, 13814, 800, 7715, 7715],
    ["Bubble Sort", "Bubble Sort", "Random", 1000, 977977, 1000, 1000000, 1000000],
    ["Counting Sort", "Counting Sort", "Random", 1000, 4996, 2000, 2000, 2000],
    ["Heap Sort", "Heap Sort", "Random", 1000, 10106, 9966, 9966, 9966],
    ["Insertion Sort", "Insertion Sort", "Random", 1000, 239464, 1000, 1000000, 1000000],
    ["Merge Sort", "Merge Sort", "Random", 1000, 19952, 9966, 9966, 9966],
    ["Quick Sort", "Quick Sort", "Random", 1000, 12331, 9966, 9966, 1000000],
    ["Selection Sort", "Selection Sort", "Random", 1000, 500500, 1000000, 1000000, 1000000],
    ["Tim Sort", "Tim Sort", "Random", 1000, 19160, 1000, 9966, 9966],
    ["Bubble Sort", "Bubble Sort", "Random", 1500, 2221480, 1500, 2250000, 2250000],
    ["Counting Sort", "Counting Sort", "Random", 1500, 7494, 3000, 3000, 3000],
    ["Heap Sort", "Heap Sort", "Random", 1500, 16005, 15826, 15826, 15826],
    ["Insertion Sort", "Insertion Sort", "Random", 1500, 573260, 1500, 2250000, 2250000],
    ["Merge Sort", "Merge Sort", "Random", 1500, 31904, 15826, 15826, 15826],
    ["Quick Sort", "Quick Sort", "Random", 1500, 19393, 15826, 15826, 2250000],
    ["Selection Sort", "Selection Sort", "Random", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Tim Sort", "Tim Sort", "Random", 1500, 27536, 1500, 15826, 15826],
    ["Bubble Sort", "Bubble Sort", "Random", 2000, 3843921, 2000, 4000000, 4000000],
    ["Counting Sort", "Counting Sort", "Random", 2000, 10000, 4000, 4000, 4000],
    ["Heap Sort", "Heap Sort", "Random", 2000, 22216, 21932, 21932, 21932],
    ["Insertion Sort", "Insertion Sort", "Random", 2000, 994189, 2000, 4000000, 4000000],
    ["Merge Sort", "Merge Sort", "Random", 2000, 43904, 21932, 21932, 21932],
    ["Quick Sort", "Quick Sort", "Random", 2000, 26954, 21932, 21932, 4000000],
    ["Selection Sort", "Selection Sort", "Random", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Tim Sort", "Tim Sort", "Random", 2000, 41406, 2000, 21932, 21932],
    ["Bubble Sort", "Bubble Sort", "Random", 3000, 8930976, 3000, 9000000, 9000000],
    ["Counting Sort", "Counting Sort", "Random", 3000, 14996, 6000, 6000, 6000],
    ["Heap Sort", "Heap Sort", "Random", 3000, 35106, 34652, 34652, 34652],
    ["Insertion Sort", "Insertion Sort", "Random", 3000, 2239124, 3000, 9000000, 9000000],
    ["Merge Sort", "Merge Sort", "Random", 3000, 69808, 34652, 34652, 34652],
    ["Quick Sort", "Quick Sort", "Random", 3000, 46173, 34652, 34652, 9000000],
    ["Selection Sort", "Selection Sort", "Random", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Tim Sort", "Tim Sort", "Random", 3000, 59357, 3000, 34652, 34652],
    ["Bubble Sort", "Bubble Sort", "Random", 4000, 15791947, 4000, 16000000, 16000000],
    ["Counting Sort", "Counting Sort", "Random", 4000, 20000, 8000, 8000, 8000],
    ["Heap Sort", "Heap Sort", "Random", 4000, 48252, 47863, 47863, 47863],
    ["Insertion Sort", "Insertion Sort", "Random", 4000, 4062593, 4000, 16000000, 16000000],
    ["Merge Sort", "Merge Sort", "Random", 4000, 95808, 47863, 47863, 47863],
    ["Quick Sort", "Quick Sort", "Random", 4000, 59978, 47863, 47863, 16000000],
    ["Selection Sort", "Selection Sort", "Random", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Tim Sort", "Tim Sort", "Random", 4000, 89089, 4000, 47863, 47863],
    ["Bubble Sort", "Bubble Sort", "Random", 6000, 35057842, 6000, 36000000, 36000000],
    ["Counting Sort", "Counting Sort", "Random", 6000, 30000, 12000, 12000, 12000],
    ["Heap Sort", "Heap Sort", "Random", 6000, 76244, 75304, 75304, 75304],
    ["Insertion Sort", "Insertion Sort", "Random", 6000, 9028871, 6000, 36000000, 36000000],
    ["Merge Sort", "Merge Sort", "Random", 6000, 151616, 75304, 75304, 75304],
    ["Quick Sort", "Quick Sort", "Random", 6000, 98260, 75304, 75304, 36000000],
    ["Selection Sort", "Selection Sort", "Random", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Tim Sort", "Tim Sort", "Random", 6000, 128026, 6000, 75304, 75304],
    ["Bubble Sort", "Bubble Sort", "Random", 8000, 62711838, 8000, 64000000, 64000000],
    ["Counting Sort", "Counting Sort", "Random", 8000, 39998, 16000, 16000, 16000],
    ["Heap Sort", "Heap Sort", "Random", 8000, 104629, 103726, 103726, 103726],
    ["Insertion Sort", "Insertion Sort", "Random", 8000, 16049105, 8000, 64000000, 64000000],
    ["Merge Sort", "Merge Sort", "Random", 8000, 207616, 103726, 103726, 103726],
    ["Quick Sort", "Quick Sort", "Random", 8000, 140198, 103726, 103726, 64000000],
    ["Selection Sort", "Selection Sort", "Random", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Tim Sort", "Tim Sort", "Random", 8000, 190034, 8000, 103726, 103726],
  ],
  "all-algorithms-sorted": [
    ["Bubble Sort", "Bubble Sort", "Sorted", 10, 11, 10, 100, 100],
    ["Counting Sort", "Counting Sort", "Sorted", 10, 50, 20, 20, 20],
    ["Heap Sort", "Heap Sort", "Sorted", 10, 42, 33, 33, 33],
    ["Insertion Sort", "Insertion Sort", "Sorted", 10, 9, 10, 100, 100],
    ["Merge Sort", "Merge Sort", "Sorted", 10, 68, 33, 33, 33],
    ["Quick Sort", "Quick Sort", "Sorted", 10, 64, 33, 33, 100],
    ["Selection Sort", "Selection Sort", "Sorted", 10, 55, 100, 100, 100],
    ["Tim Sort", "Tim Sort", "Sorted", 10, 8, 10, 33, 33],
    ["Bubble Sort", "Bubble Sort", "Sorted", 15, 16, 15, 225, 225],
    ["Counting Sort", "Counting Sort", "Sorted", 15, 75, 30, 30, 30],
    ["Heap Sort", "Heap Sort", "Sorted", 15, 68, 59, 59, 59],
    ["Insertion Sort", "Insertion Sort", "Sorted", 15, 14, 15, 225, 225],
    ["Merge Sort", "Merge Sort", "Sorted", 15, 118, 59, 59, 59],
    ["Quick Sort", "Quick Sort", "Sorted", 15, 134, 59, 59, 225],
    ["Selection Sort", "Selection Sort", "Sorted", 15, 120, 225, 225, 225],
    ["Tim Sort", "Tim Sort", "Sorted", 15, 72, 15, 59, 59],
    ["Bubble Sort", "Bubble Sort", "Sorted", 20, 21, 20, 400, 400],
    ["Counting Sort", "Counting Sort", "Sorted", 20, 100, 40, 40, 40],
    ["Heap Sort", "Heap Sort", "Sorted", 20, 102, 86, 86, 86],
    ["Insertion Sort", "Insertion Sort", "Sorted", 20, 19, 20, 400, 400],
    ["Merge Sort", "Merge Sort", "Sorted", 20, 176, 86, 86, 86],
    ["Quick Sort", "Quick Sort", "Sorted", 20, 229, 86, 86, 400],
    ["Selection Sort", "Selection Sort", "Sorted", 20, 210, 400, 400, 400],
    ["Tim Sort", "Tim Sort", "Sorted", 20, 140, 20, 86, 86],
    ["Bubble Sort", "Bubble Sort", "Sorted", 30, 31, 30, 900, 900],
    ["Counting Sort", "Counting Sort", "Sorted", 30, 150, 60, 60, 60],
    ["Heap Sort", "Heap Sort", "Sorted", 30, 168, 147, 147, 147],
    ["Insertion Sort", "Insertion Sort", "Sorted", 30, 29, 30, 900, 900],
    ["Merge Sort", "Merge Sort", "Sorted", 30, 296, 147, 147, 147],
    ["Quick Sort", "Quick Sort", "Sorted", 30, 494, 147, 147, 900],
    ["Selection Sort", "Selection Sort", "Sorted", 30, 465, 900, 900, 900],
    ["Tim Sort", "Tim Sort", "Sorted", 30, 267, 30, 147, 147],
    ["Bubble Sort", "Bubble Sort", "Sorted", 40, 41, 40, 1600, 1600],
    ["Counting Sort", "Counting Sort", "Sorted", 40, 200, 80, 80, 80],
    ["Heap Sort", "Heap Sort", "Sorted", 40, 244, 213, 213, 213],
    ["Insertion Sort", "Insertion Sort", "Sorted", 40, 39, 40, 1600, 1600],
    ["Merge Sort", "Merge Sort", "Sorted", 40, 432, 213, 213, 213],
    ["Quick Sort", "Quick Sort", "Sorted", 40, 859, 213, 213, 1600],
    ["Selection Sort", "Selection Sort", "Sorted", 40, 820, 1600, 1600, 1600],
    ["Tim Sort", "Tim Sort", "Sorted", 40, 202, 40, 213, 213],
    ["Bubble Sort", "Bubble Sort", "Sorted", 60, 61, 60, 3600, 3600],
    ["Counting Sort", "Counting Sort", "Sorted", 60, 300, 120, 120, 120],
    ["Heap Sort", "Heap Sort", "Sorted", 60, 398, 354, 354, 354],
    ["Insertion Sort", "Insertion Sort", "Sorted", 60, 59, 60, 3600, 3600],
    ["Merge Sort", "Merge Sort", "Sorted", 60, 712, 354, 354, 354],
    ["Quick Sort", "Quick Sort", "Sorted", 60, 1889, 354, 354, 3600],
    ["Selection Sort", "Selection Sort", "Sorted", 60, 1830, 3600, 3600, 3600],
    ["Tim Sort", "Tim Sort", "Sorted", 60, 349, 60, 354, 354],
    ["Bubble Sort", "Bubble Sort", "Sorted", 80, 81, 80, 6400, 6400],
    ["Counting Sort", "Counting Sort", "Sorted", 80, 400, 160, 160, 160],
    ["Heap Sort", "Heap Sort", "Sorted", 80, 568, 506, 506, 506],
    ["Insertion Sort", "Insertion Sort", "Sorted", 80, 79, 80, 6400, 6400],
    ["Merge Sort", "Merge Sort", "Sorted", 80, 1024, 506, 506, 506],
    ["Quick Sort", "Quick Sort", "Sorted", 80, 3319, 506, 506, 6400],
    ["Selection Sort", "Selection Sort", "Sorted", 80, 3240, 6400, 6400, 6400],
    ["Tim Sort", "Tim Sort", "Sorted", 80, 292, 80, 506, 506],
    ["Bubble Sort", "Bubble Sort", "Sorted", 100, 101, 100, 10000, 10000],
    ["Counting Sort", "Counting Sort", "Sorted", 100, 500, 200, 200, 200],
    ["Heap Sort", "Heap Sort", "Sorted", 100, 742, 664, 664, 664],
    ["Insertion Sort", "Insertion Sort", "Sorted", 100, 99, 100, 10000, 10000],
    ["Merge Sort", "Merge Sort", "Sorted", 100, 1344, 664, 664, 664],
    ["Quick Sort", "Quick Sort", "Sorted", 100, 5149, 664, 664, 10000],
    ["Selection Sort", "Selection Sort", "Sorted", 100, 5050, 10000, 10000, 10000],
    ["Tim Sort", "Tim Sort", "Sorted", 100, 380, 100, 664, 664],
    ["Bubble Sort", "Bubble Sort", "Sorted", 150, 151, 150, 22500, 22500],
    ["Counting Sort", "Counting Sort", "Sorted", 150, 750, 300, 300, 300],
    ["Heap Sort", "Heap Sort", "Sorted", 150, 1206, 1084, 1084, 1084],
    ["Insertion Sort", "Insertion Sort", "Sorted", 150, 149, 150, 22500, 22500],
    ["Merge Sort", "Merge Sort", "Sorted", 150, 2188, 1084, 1084, 1084],
    ["Quick Sort", "Quick Sort", "Sorted", 150, 11474, 1084, 1084, 22500],
    ["Selection Sort", "Selection Sort", "Sorted", 150, 11325, 22500, 22500, 22500],
    ["Tim Sort", "Tim Sort", "Sorted", 150, 598, 150, 1084, 1084],
    ["Bubble Sort", "Bubble Sort", "Sorted", 200, 201, 200, 40000, 40000],
    ["Counting Sort", "Counting Sort", "Sorted", 200, 1000, 400, 400, 400],
    ["Heap Sort", "Heap Sort", "Sorted", 200, 1690, 1529, 1529, 1529],
    ["Insertion Sort", "Insertion Sort", "Sorted", 200, 199, 200, 40000, 40000],
    ["Merge Sort", "Merge Sort", "Sorted", 200, 3088, 1529, 1529, 1529],
    ["Quick Sort", "Quick Sort", "Sorted", 200, 20299, 1529, 1529, 40000],
    ["Selection Sort", "Selection Sort", "Sorted", 200, 20100, 40000, 40000, 40000],
    ["Tim Sort", "Tim Sort", "Sorted", 200, 816, 200, 1529, 1529],
    ["Bubble Sort", "Bubble Sort", "Sorted", 300, 301, 300, 90000, 90000],
    ["Counting Sort", "Counting Sort", "Sorted", 300, 1500, 600, 600, 600],
    ["Heap Sort", "Heap Sort", "Sorted", 300, 2712, 2469, 2469, 2469],
    ["Insertion Sort", "Insertion Sort", "Sorted", 300, 299, 300, 90000, 90000],
    ["Merge Sort", "Merge Sort", "Sorted", 300, 4976, 2469, 2469, 2469],
    ["Quick Sort", "Quick Sort", "Sorted", 300, 45449, 2469, 2469, 90000],
    ["Selection Sort", "Selection Sort", "Sorted", 300, 45150, 90000, 90000, 90000],
    ["Tim Sort", "Tim Sort", "Sorted", 300, 1012, 300, 2469, 2469],
    ["Bubble Sort", "Bubble Sort", "Sorted", 400, 401, 400, 160000, 160000],
    ["Counting Sort", "Counting Sort", "Sorted", 400, 2000, 800, 800, 800],
    ["Heap Sort", "Heap Sort", "Sorted", 400, 3774, 3458, 3458, 3458],
    ["Insertion Sort", "Insertion Sort", "Sorted", 400, 399, 400, 160000, 160000],
    ["Merge Sort", "Merge Sort", "Sorted", 400, 6976, 3458, 3458, 3458],
    ["Quick Sort", "Quick Sort", "Sorted", 400, 80599, 3458, 3458, 160000],
    ["Selection Sort", "Selection Sort", "Sorted", 400, 80200, 160000, 160000, 160000],
    ["Tim Sort", "Tim Sort", "Sorted", 400, 1317, 400, 3458, 3458],
    ["Bubble Sort", "Bubble Sort", "Sorted", 600, 601, 600, 360000, 360000],
    ["Counting Sort", "Counting Sort", "Sorted", 600, 3000, 1200, 1200, 1200],
    ["Heap Sort", "Heap Sort", "Sorted", 600, 6006, 5537, 5537, 5537],
    ["Insertion Sort", "Insertion Sort", "Sorted", 600, 599, 600, 360000, 360000],
    ["Merge Sort", "Merge Sort", "Sorted", 600, 11152, 5537, 5537, 5537],
    ["Quick Sort", "Quick Sort", "Sorted", 600, 180899, 5537, 5537, 360000],
    ["Selection Sort", "Selection Sort", "Sorted", 600, 180300, 360000, 360000, 360000],
    ["Tim Sort", "Tim Sort", "Sorted", 600, 1763, 600, 5537, 5537],
    ["Bubble Sort", "Bubble Sort", "Sorted", 800, 801, 800, 640000, 640000],
    ["Counting Sort", "Counting Sort", "Sorted", 800, 4000, 1600, 1600, 1600],
    ["Heap Sort", "Heap Sort", "Sorted", 800, 8356, 7715, 7715, 7715],
    ["Insertion Sort", "Insertion Sort", "Sorted", 800, 799, 800, 640000, 640000],
    ["Merge Sort", "Merge Sort", "Sorted", 800, 15552, 7715, 7715, 7715],
    ["Quick Sort", "Quick Sort", "Sorted", 800, 321199, 7715, 7715, 640000],
    ["Selection Sort", "Selection Sort", "Sorted", 800, 320400, 640000, 640000, 640000],
    ["Tim Sort", "Tim Sort", "Sorted", 800, 2318, 800, 7715, 7715],
    ["Bubble Sort", "Bubble Sort", "Sorted", 1000, 1001, 1000, 1000000, 1000000],
    ["Counting Sort", "Counting Sort", "Sorted", 1000, 5000, 2000, 2000, 2000],
    ["Heap Sort", "Heap Sort", "Sorted", 1000, 10710, 9966, 9966, 9966],
    ["Insertion Sort", "Insertion Sort", "Sorted", 1000, 999, 1000, 1000000, 1000000],
    ["Merge Sort", "Merge Sort", "Sorted", 1000, 19952, 9966, 9966, 9966],
    ["Quick Sort", "Quick Sort", "Sorted", 1000, 501499, 9966, 9966, 1000000],
    ["Selection Sort", "Selection Sort", "Sorted", 1000, 500500, 1000000, 1000000, 1000000],
    ["Tim Sort", "Tim Sort", "Sorted", 1000, 2876, 1000, 9966, 9966],
    ["Bubble Sort", "Bubble Sort", "Sorted", 1500, 1501, 1500, 2250000, 2250000],
    ["Counting Sort", "Counting Sort", "Sorted", 1500, 7500, 3000, 3000, 3000],
    ["Heap Sort", "Heap Sort", "Sorted", 1500, 17050, 15826, 15826, 15826],
    ["Insertion Sort", "Insertion Sort", "Sorted", 1500, 1499, 1500, 2250000, 2250000],
    ["Merge Sort", "Merge Sort", "Sorted", 1500, 31904, 15826, 15826, 15826],
    ["Quick Sort", "Quick Sort", "Sorted", 1500, 1127249, 15826, 15826, 2250000],
    ["Selection Sort", "Selection Sort", "Sorted", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Tim Sort", "Tim Sort", "Sorted", 1500, 5510, 1500, 15826, 15826],
    ["Bubble Sort", "Bubble Sort", "Sorted", 2000, 2001, 2000, 4000000, 4000000],
    ["Counting Sort", "Counting Sort", "Sorted", 2000, 10000, 4000, 4000, 4000],
    ["Heap Sort", "Heap Sort", "Sorted", 2000, 23302, 21932, 21932, 21932],
    ["Insertion Sort", "Insertion Sort", "Sorted", 2000, 1999, 2000, 4000000, 4000000],
    ["Merge Sort", "Merge Sort", "Sorted", 2000, 43904, 21932, 21932, 21932],
    ["Quick Sort", "Quick Sort", "Sorted", 2000, 2002999, 21932, 21932, 4000000],
    ["Selection Sort", "Selection Sort", "Sorted", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Tim Sort", "Tim Sort", "Sorted", 2000, 7370, 2000, 21932, 21932],
    ["Bubble Sort", "Bubble Sort", "Sorted", 3000, 3001, 3000, 9000000, 9000000],
    ["Counting Sort", "Counting Sort", "Sorted", 3000, 15000, 6000, 6000, 6000],
    ["Heap Sort", "Heap Sort", "Sorted", 3000, 37254, 34652, 34652, 34652],
    ["Insertion Sort", "Insertion Sort", "Sorted", 3000, 2999, 3000, 9000000, 9000000],
    ["Merge Sort", "Merge Sort", "Sorted", 3000, 69808, 34652, 34652, 34652],
    ["Quick Sort", "Quick Sort", "Sorted", 3000, 4504499, 34652, 34652, 9000000],
    ["Selection Sort", "Selection Sort", "Sorted", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Tim Sort", "Tim Sort", "Sorted", 3000, 9791, 3000, 34652, 34652],
    ["Bubble Sort", "Bubble Sort", "Sorted", 4000, 4001, 4000, 16000000, 16000000],
    ["Counting Sort", "Counting Sort", "Sorted", 4000, 20000, 8000, 8000, 8000],
    ["Heap Sort", "Heap Sort", "Sorted", 4000, 51144, 47863, 47863, 47863],
    ["Insertion Sort", "Insertion Sort", "Sorted", 4000, 3999, 4000, 16000000, 16000000],
    ["Merge Sort", "Merge Sort", "Sorted", 4000, 95808, 47863, 47863, 47863],
    ["Quick Sort", "Quick Sort", "Sorted", 4000, 8005999, 47863, 47863, 16000000],
    ["Selection Sort", "Selection Sort", "Sorted", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Tim Sort", "Tim Sort", "Sorted", 4000, 12375, 4000, 47863, 47863],
    ["Bubble Sort", "Bubble Sort", "Sorted", 6000, 6001, 6000, 36000000, 36000000],
    ["Counting Sort", "Counting Sort", "Sorted", 6000, 30000, 12000, 12000, 12000],
    ["Heap Sort", "Heap Sort", "Sorted", 6000, 80830, 75304, 75304, 75304],
    ["Insertion Sort", "Insertion Sort", "Sorted", 6000, 5999, 6000, 36000000, 36000000],
    ["Merge Sort", "Merge Sort", "Sorted", 6000, 151616, 75304, 75304, 75304],
    ["Quick Sort", "Quick Sort", "Sorted", 6000, 18008999, 75304, 75304, 36000000],
    ["Selection Sort", "Selection Sort", "Sorted", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Tim Sort", "Tim Sort", "Sorted", 6000, 17298, 6000, 75304, 75304],
    ["Bubble Sort", "Bubble Sort", "Sorted", 8000, 8001, 8000, 64000000, 64000000],
    ["Counting Sort", "Counting Sort", "Sorted", 8000, 40000, 16000, 16000, 16000],
    ["Heap Sort", "Heap Sort", "Sorted", 8000, 110640, 103726, 103726, 103726],
    ["Insertion Sort", "Insertion Sort", "Sorted", 8000, 7999, 8000, 64000000, 64000000],
    ["Merge Sort", "Merge Sort", "Sorted", 8000, 207616, 103726, 103726, 103726],
    ["Quick Sort", "Quick Sort", "Sorted", 8000, 30411780, 103726, 103726, 64000000],
    ["Selection Sort", "Selection Sort", "Sorted", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Tim Sort", "Tim Sort", "Sorted", 8000, 22384, 8000, 103726, 103726],
  ],
  "counting-sort-all": [
    ["Random", "Counting Sort", "Random", 10, 50, 20, 20, 20],
    ["Sorted", "Counting Sort", "Sorted", 10, 50, 20, 20, 20],
    ["Half Sorted", "Counting Sort", "Half Sorted", 10, 40, 20, 20, 20],
    ["Sorted Half", "Counting Sort", "Sorted Half", 10, 40, 20, 20, 20],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 10, 50, 20, 20, 20],
    ["Reversed", "Counting Sort", "Reversed", 10, 50, 20, 20, 20],
    ["Same", "Counting Sort", "Same", 10, 32, 20, 20, 20],
    ["Zig Zag", "Counting Sort", "Zig Zag", 10, 50, 20, 20, 20],
    ["Sawtooth", "Counting Sort", "Sawtooth", 10, 32, 20, 20, 20],
    ["Square", "Counting Sort", "Square", 10, 52, 20, 20, 20],
    ["Zeroes", "Counting Sort", "Zeroes", 10, 32, 20, 20, 20],
    ["Thousands", "Counting Sort", "Thousands", 10, 1632, 20, 20, 20],
    ["Millions", "Counting Sort", "Millions", 10, 1788698, 20, 20, 20],
    ["Blender", "Counting Sort", "Blender", 10, 50, 20, 20, 20],
    ["Random", "Counting Sort", "Random", 15, 75, 30, 30, 30],
    ["Sorted", "Counting Sort", "Sorted", 15, 75, 30, 30, 30],
    ["Half Sorted", "Counting Sort", "Half Sorted", 14, 58, 28, 28, 28],
    ["Sorted Half", "Counting Sort", "Sorted Half", 14, 56, 28, 28, 28],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 15, 68, 30, 30, 30],
    ["Reversed", "Counting Sort", "Reversed", 15, 75, 30, 30, 30],
    ["Same", "Counting Sort", "Same", 15, 47, 30, 30, 30],
    ["Zig Zag", "Counting Sort", "Zig Zag", 15, 75, 30, 30, 30],
    ["Sawtooth", "Counting Sort", "Sawtooth", 15, 42, 30, 30, 30],
    ["Square", "Counting Sort", "Square", 15, 77, 30, 30, 30],
    ["Zeroes", "Counting Sort", "Zeroes", 15, 47, 30, 30, 30],
    ["Thousands", "Counting Sort", "Thousands", 15, 1549, 30, 30, 30],
    ["Millions", "Counting Sort", "Millions", 15, 1821151, 30, 30, 30],
    ["Blender", "Counting Sort", "Blender", 15, 75, 30, 30, 30],
    ["Random", "Counting Sort", "Random", 20, 96, 40, 40, 40],
    ["Sorted", "Counting Sort", "Sorted", 20, 100, 40, 40, 40],
    ["Half Sorted", "Counting Sort", "Half Sorted", 20, 80, 40, 40, 40],
    ["Sorted Half", "Counting Sort", "Sorted Half", 20, 80, 40, 40, 40],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 20, 90, 40, 40, 40],
    ["Reversed", "Counting Sort", "Reversed", 20, 100, 40, 40, 40],
    ["Same", "Counting Sort", "Same", 20, 62, 40, 40, 40],
    ["Zig Zag", "Counting Sort", "Zig Zag", 20, 100, 40, 40, 40],
    ["Sawtooth", "Counting Sort", "Sawtooth", 20, 55, 40, 40, 40],
    ["Square", "Counting Sort", "Square", 20, 102, 40, 40, 40],
    ["Zeroes", "Counting Sort", "Zeroes", 20, 62, 40, 40, 40],
    ["Thousands", "Counting Sort", "Thousands", 20, 1934, 40, 40, 40],
    ["Millions", "Counting Sort", "Millions", 20, 1840930, 40, 40, 40],
    ["Blender", "Counting Sort", "Blender", 20, 94, 40, 40, 40],
    ["Random", "Counting Sort", "Random", 30, 150, 60, 60, 60],
    ["Sorted", "Counting Sort", "Sorted", 30, 150, 60, 60, 60],
    ["Half Sorted", "Counting Sort", "Half Sorted", 30, 120, 60, 60, 60],
    ["Sorted Half", "Counting Sort", "Sorted Half", 30, 120, 60, 60, 60],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 30, 140, 60, 60, 60],
    ["Reversed", "Counting Sort", "Reversed", 30, 150, 60, 60, 60],
    ["Same", "Counting Sort", "Same", 30, 92, 60, 60, 60],
    ["Zig Zag", "Counting Sort", "Zig Zag", 30, 150, 60, 60, 60],
    ["Sawtooth", "Counting Sort", "Sawtooth", 30, 78, 60, 60, 60],
    ["Square", "Counting Sort", "Square", 30, 152, 60, 60, 60],
    ["Zeroes", "Counting Sort", "Zeroes", 30, 92, 60, 60, 60],
    ["Thousands", "Counting Sort", "Thousands", 30, 2026, 60, 60, 60],
    ["Millions", "Counting Sort", "Millions", 30, 1887408, 60, 60, 60],
    ["Blender", "Counting Sort", "Blender", 30, 148, 60, 60, 60],
    ["Random", "Counting Sort", "Random", 40, 198, 80, 80, 80],
    ["Sorted", "Counting Sort", "Sorted", 40, 200, 80, 80, 80],
    ["Half Sorted", "Counting Sort", "Half Sorted", 40, 160, 80, 80, 80],
    ["Sorted Half", "Counting Sort", "Sorted Half", 40, 160, 80, 80, 80],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 40, 180, 80, 80, 80],
    ["Reversed", "Counting Sort", "Reversed", 40, 200, 80, 80, 80],
    ["Same", "Counting Sort", "Same", 40, 122, 80, 80, 80],
    ["Zig Zag", "Counting Sort", "Zig Zag", 40, 200, 80, 80, 80],
    ["Sawtooth", "Counting Sort", "Sawtooth", 40, 101, 80, 80, 80],
    ["Square", "Counting Sort", "Square", 40, 202, 80, 80, 80],
    ["Zeroes", "Counting Sort", "Zeroes", 40, 122, 80, 80, 80],
    ["Thousands", "Counting Sort", "Thousands", 40, 2044, 80, 80, 80],
    ["Millions", "Counting Sort", "Millions", 40, 1749012, 80, 80, 80],
    ["Blender", "Counting Sort", "Blender", 40, 196, 80, 80, 80],
    ["Random", "Counting Sort", "Random", 60, 300, 120, 120, 120],
    ["Sorted", "Counting Sort", "Sorted", 60, 300, 120, 120, 120],
    ["Half Sorted", "Counting Sort", "Half Sorted", 60, 240, 120, 120, 120],
    ["Sorted Half", "Counting Sort", "Sorted Half", 60, 240, 120, 120, 120],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 60, 266, 120, 120, 120],
    ["Reversed", "Counting Sort", "Reversed", 60, 300, 120, 120, 120],
    ["Same", "Counting Sort", "Same", 60, 182, 120, 120, 120],
    ["Zig Zag", "Counting Sort", "Zig Zag", 60, 300, 120, 120, 120],
    ["Sawtooth", "Counting Sort", "Sawtooth", 60, 144, 120, 120, 120],
    ["Square", "Counting Sort", "Square", 60, 302, 120, 120, 120],
    ["Zeroes", "Counting Sort", "Zeroes", 60, 182, 120, 120, 120],
    ["Thousands", "Counting Sort", "Thousands", 60, 2042, 120, 120, 120],
    ["Millions", "Counting Sort", "Millions", 60, 1879018, 120, 120, 120],
    ["Blender", "Counting Sort", "Blender", 60, 300, 120, 120, 120],
    ["Random", "Counting Sort", "Random", 80, 398, 160, 160, 160],
    ["Sorted", "Counting Sort", "Sorted", 80, 400, 160, 160, 160],
    ["Half Sorted", "Counting Sort", "Half Sorted", 80, 320, 160, 160, 160],
    ["Sorted Half", "Counting Sort", "Sorted Half", 80, 320, 160, 160, 160],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 80, 354, 160, 160, 160],
    ["Reversed", "Counting Sort", "Reversed", 80, 400, 160, 160, 160],
    ["Same", "Counting Sort", "Same", 80, 242, 160, 160, 160],
    ["Zig Zag", "Counting Sort", "Zig Zag", 80, 400, 160, 160, 160],
    ["Sawtooth", "Counting Sort", "Sawtooth", 80, 187, 160, 160, 160],
    ["Square", "Counting Sort", "Square", 80, 402, 160, 160, 160],
    ["Zeroes", "Counting Sort", "Zeroes", 80, 242, 160, 160, 160],
    ["Thousands", "Counting Sort", "Thousands", 80, 2192, 160, 160, 160],
    ["Millions", "Counting Sort", "Millions", 80, 1941908, 160, 160, 160],
    ["Blender", "Counting Sort", "Blender", 80, 400, 160, 160, 160],
    ["Random", "Counting Sort", "Random", 100, 500, 200, 200, 200],
    ["Sorted", "Counting Sort", "Sorted", 100, 500, 200, 200, 200],
    ["Half Sorted", "Counting Sort", "Half Sorted", 100, 400, 200, 200, 200],
    ["Sorted Half", "Counting Sort", "Sorted Half", 100, 400, 200, 200, 200],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 100, 448, 200, 200, 200],
    ["Reversed", "Counting Sort", "Reversed", 100, 500, 200, 200, 200],
    ["Same", "Counting Sort", "Same", 100, 302, 200, 200, 200],
    ["Zig Zag", "Counting Sort", "Zig Zag", 100, 500, 200, 200, 200],
    ["Sawtooth", "Counting Sort", "Sawtooth", 100, 320, 200, 200, 200],
    ["Square", "Counting Sort", "Square", 100, 502, 200, 200, 200],
    ["Zeroes", "Counting Sort", "Zeroes", 100, 302, 200, 200, 200],
    ["Thousands", "Counting Sort", "Thousands", 100, 2286, 200, 200, 200],
    ["Millions", "Counting Sort", "Millions", 100, 1984532, 200, 200, 200],
    ["Blender", "Counting Sort", "Blender", 100, 1813723, 200, 200, 200],
    ["Random", "Counting Sort", "Random", 150, 750, 300, 300, 300],
    ["Sorted", "Counting Sort", "Sorted", 150, 750, 300, 300, 300],
    ["Half Sorted", "Counting Sort", "Half Sorted", 150, 600, 300, 300, 300],
    ["Sorted Half", "Counting Sort", "Sorted Half", 150, 600, 300, 300, 300],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 150, 680, 300, 300, 300],
    ["Reversed", "Counting Sort", "Reversed", 150, 750, 300, 300, 300],
    ["Same", "Counting Sort", "Same", 150, 452, 300, 300, 300],
    ["Zig Zag", "Counting Sort", "Zig Zag", 150, 750, 300, 300, 300],
    ["Sawtooth", "Counting Sort", "Sawtooth", 150, 339, 300, 300, 300],
    ["Square", "Counting Sort", "Square", 150, 752, 300, 300, 300],
    ["Zeroes", "Counting Sort", "Zeroes", 150, 452, 300, 300, 300],
    ["Thousands", "Counting Sort", "Thousands", 150, 2426, 300, 300, 300],
    ["Millions", "Counting Sort", "Millions", 150, 1947122, 300, 300, 300],
    ["Blender", "Counting Sort", "Blender", 150, 1999404, 300, 300, 300],
    ["Random", "Counting Sort", "Random", 200, 998, 400, 400, 400],
    ["Sorted", "Counting Sort", "Sorted", 200, 1000, 400, 400, 400],
    ["Half Sorted", "Counting Sort", "Half Sorted", 200, 800, 400, 400, 400],
    ["Sorted Half", "Counting Sort", "Sorted Half", 200, 800, 400, 400, 400],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 200, 896, 400, 400, 400],
    ["Reversed", "Counting Sort", "Reversed", 200, 1000, 400, 400, 400],
    ["Same", "Counting Sort", "Same", 200, 602, 400, 400, 400],
    ["Zig Zag", "Counting Sort", "Zig Zag", 200, 1000, 400, 400, 400],
    ["Sawtooth", "Counting Sort", "Sawtooth", 200, 445, 400, 400, 400],
    ["Square", "Counting Sort", "Square", 200, 1002, 400, 400, 400],
    ["Zeroes", "Counting Sort", "Zeroes", 200, 602, 400, 400, 400],
    ["Thousands", "Counting Sort", "Thousands", 200, 2594, 400, 400, 400],
    ["Millions", "Counting Sort", "Millions", 200, 1992578, 400, 400, 400],
    ["Blender", "Counting Sort", "Blender", 200, 1196328, 400, 400, 400],
    ["Random", "Counting Sort", "Random", 300, 1498, 600, 600, 600],
    ["Sorted", "Counting Sort", "Sorted", 300, 1500, 600, 600, 600],
    ["Half Sorted", "Counting Sort", "Half Sorted", 300, 1200, 600, 600, 600],
    ["Sorted Half", "Counting Sort", "Sorted Half", 300, 1200, 600, 600, 600],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 300, 1346, 600, 600, 600],
    ["Reversed", "Counting Sort", "Reversed", 300, 1500, 600, 600, 600],
    ["Same", "Counting Sort", "Same", 300, 902, 600, 600, 600],
    ["Zig Zag", "Counting Sort", "Zig Zag", 300, 1500, 600, 600, 600],
    ["Sawtooth", "Counting Sort", "Sawtooth", 300, 654, 600, 600, 600],
    ["Square", "Counting Sort", "Square", 300, 1502, 600, 600, 600],
    ["Zeroes", "Counting Sort", "Zeroes", 300, 902, 600, 600, 600],
    ["Thousands", "Counting Sort", "Thousands", 300, 2892, 600, 600, 600],
    ["Millions", "Counting Sort", "Millions", 300, 1992340, 600, 600, 600],
    ["Blender", "Counting Sort", "Blender", 300, 1999547, 600, 600, 600],
    ["Random", "Counting Sort", "Random", 400, 1998, 800, 800, 800],
    ["Sorted", "Counting Sort", "Sorted", 400, 2000, 800, 800, 800],
    ["Half Sorted", "Counting Sort", "Half Sorted", 400, 1600, 800, 800, 800],
    ["Sorted Half", "Counting Sort", "Sorted Half", 400, 1600, 800, 800, 800],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 400, 1800, 800, 800, 800],
    ["Reversed", "Counting Sort", "Reversed", 400, 2000, 800, 800, 800],
    ["Same", "Counting Sort", "Same", 400, 1202, 800, 800, 800],
    ["Zig Zag", "Counting Sort", "Zig Zag", 400, 2000, 800, 800, 800],
    ["Sawtooth", "Counting Sort", "Sawtooth", 400, 1240, 800, 800, 800],
    ["Square", "Counting Sort", "Square", 400, 2002, 800, 800, 800],
    ["Zeroes", "Counting Sort", "Zeroes", 400, 1202, 800, 800, 800],
    ["Thousands", "Counting Sort", "Thousands", 400, 3190, 800, 800, 800],
    ["Millions", "Counting Sort", "Millions", 400, 1989746, 800, 800, 800],
    ["Blender", "Counting Sort", "Blender", 400, 1930374, 800, 800, 800],
    ["Random", "Counting Sort", "Random", 600, 3000, 1200, 1200, 1200],
    ["Sorted", "Counting Sort", "Sorted", 600, 3000, 1200, 1200, 1200],
    ["Half Sorted", "Counting Sort", "Half Sorted", 600, 2400, 1200, 1200, 1200],
    ["Sorted Half", "Counting Sort", "Sorted Half", 600, 2400, 1200, 1200, 1200],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 600, 2698, 1200, 1200, 1200],
    ["Reversed", "Counting Sort", "Reversed", 600, 3000, 1200, 1200, 1200],
    ["Same", "Counting Sort", "Same", 600, 1802, 1200, 1200, 1200],
    ["Zig Zag", "Counting Sort", "Zig Zag", 600, 3000, 1200, 1200, 1200],
    ["Sawtooth", "Counting Sort", "Sawtooth", 600, 1275, 1200, 1200, 1200],
    ["Square", "Counting Sort", "Square", 600, 3002, 1200, 1200, 1200],
    ["Zeroes", "Counting Sort", "Zeroes", 600, 1802, 1200, 1200, 1200],
    ["Thousands", "Counting Sort", "Thousands", 600, 3796, 1200, 1200, 1200],
    ["Millions", "Counting Sort", "Millions", 600, 2000272, 1200, 1200, 1200],
    ["Blender", "Counting Sort", "Blender", 600, 1955821, 1200, 1200, 1200],
    ["Random", "Counting Sort", "Random", 800, 3994, 1600, 1600, 1600],
    ["Sorted", "Counting Sort", "Sorted", 800, 4000, 1600, 1600, 1600],
    ["Half Sorted", "Counting Sort", "Half Sorted", 800, 3200, 1600, 1600, 1600],
    ["Sorted Half", "Counting Sort", "Sorted Half", 800, 3200, 1600, 1600, 1600],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 800, 3590, 1600, 1600, 1600],
    ["Reversed", "Counting Sort", "Reversed", 800, 4000, 1600, 1600, 1600],
    ["Same", "Counting Sort", "Same", 800, 2402, 1600, 1600, 1600],
    ["Zig Zag", "Counting Sort", "Zig Zag", 800, 4000, 1600, 1600, 1600],
    ["Sawtooth", "Counting Sort", "Sawtooth", 800, 1687, 1600, 1600, 1600],
    ["Square", "Counting Sort", "Square", 800, 4002, 1600, 1600, 1600],
    ["Zeroes", "Counting Sort", "Zeroes", 800, 2402, 1600, 1600, 1600],
    ["Thousands", "Counting Sort", "Thousands", 800, 4400, 1600, 1600, 1600],
    ["Millions", "Counting Sort", "Millions", 800, 2001810, 1600, 1600, 1600],
    ["Blender", "Counting Sort", "Blender", 800, 1924939, 1600, 1600, 1600],
    ["Random", "Counting Sort", "Random", 1000, 5000, 2000, 2000, 2000],
    ["Sorted", "Counting Sort", "Sorted", 1000, 5000, 2000, 2000, 2000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 1000, 4000, 2000, 2000, 2000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 1000, 4000, 2000, 2000, 2000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 1000, 4498, 2000, 2000, 2000],
    ["Reversed", "Counting Sort", "Reversed", 1000, 5000, 2000, 2000, 2000],
    ["Same", "Counting Sort", "Same", 1000, 3002, 2000, 2000, 2000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 1000, 5000, 2000, 2000, 2000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 1000, 2096, 2000, 2000, 2000],
    ["Square", "Counting Sort", "Square", 1000, 5002, 2000, 2000, 2000],
    ["Zeroes", "Counting Sort", "Zeroes", 1000, 3002, 2000, 2000, 2000],
    ["Thousands", "Counting Sort", "Thousands", 1000, 5000, 2000, 2000, 2000],
    ["Millions", "Counting Sort", "Millions", 1000, 1999926, 2000, 2000, 2000],
    ["Blender", "Counting Sort", "Blender", 1000, 1994836, 2000, 2000, 2000],
    ["Random", "Counting Sort", "Random", 1500, 7494, 3000, 3000, 3000],
    ["Sorted", "Counting Sort", "Sorted", 1500, 7500, 3000, 3000, 3000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 1500, 6000, 3000, 3000, 3000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 1500, 6000, 3000, 3000, 3000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 1500, 6746, 3000, 3000, 3000],
    ["Reversed", "Counting Sort", "Reversed", 1500, 7500, 3000, 3000, 3000],
    ["Same", "Counting Sort", "Same", 1500, 4502, 3000, 3000, 3000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 1500, 7500, 3000, 3000, 3000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 1500, 3117, 3000, 3000, 3000],
    ["Square", "Counting Sort", "Square", 1500, 7502, 3000, 3000, 3000],
    ["Zeroes", "Counting Sort", "Zeroes", 1500, 4502, 3000, 3000, 3000],
    ["Thousands", "Counting Sort", "Thousands", 1500, 6498, 3000, 3000, 3000],
    ["Millions", "Counting Sort", "Millions", 1500, 2002636, 3000, 3000, 3000],
    ["Blender", "Counting Sort", "Blender", 1500, 1996427, 3000, 3000, 3000],
    ["Random", "Counting Sort", "Random", 2000, 10000, 4000, 4000, 4000],
    ["Sorted", "Counting Sort", "Sorted", 2000, 10000, 4000, 4000, 4000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 2000, 8000, 4000, 4000, 4000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 2000, 8000, 4000, 4000, 4000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 2000, 9000, 4000, 4000, 4000],
    ["Reversed", "Counting Sort", "Reversed", 2000, 10000, 4000, 4000, 4000],
    ["Same", "Counting Sort", "Same", 2000, 6002, 4000, 4000, 4000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 2000, 10000, 4000, 4000, 4000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 2000, 4135, 4000, 4000, 4000],
    ["Square", "Counting Sort", "Square", 2000, 10002, 4000, 4000, 4000],
    ["Zeroes", "Counting Sort", "Zeroes", 2000, 6002, 4000, 4000, 4000],
    ["Thousands", "Counting Sort", "Thousands", 2000, 8000, 4000, 4000, 4000],
    ["Millions", "Counting Sort", "Millions", 2000, 2002954, 4000, 4000, 4000],
    ["Blender", "Counting Sort", "Blender", 2000, 1998969, 4000, 4000, 4000],
    ["Random", "Counting Sort", "Random", 3000, 15000, 6000, 6000, 6000],
    ["Sorted", "Counting Sort", "Sorted", 3000, 15000, 6000, 6000, 6000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 3000, 12000, 6000, 6000, 6000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 3000, 12000, 6000, 6000, 6000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 3000, 13498, 6000, 6000, 6000],
    ["Reversed", "Counting Sort", "Reversed", 3000, 15000, 6000, 6000, 6000],
    ["Same", "Counting Sort", "Same", 3000, 9002, 6000, 6000, 6000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 3000, 15000, 6000, 6000, 6000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 3000, 6165, 6000, 6000, 6000],
    ["Square", "Counting Sort", "Square", 3000, 15002, 6000, 6000, 6000],
    ["Zeroes", "Counting Sort", "Zeroes", 3000, 9002, 6000, 6000, 6000],
    ["Thousands", "Counting Sort", "Thousands", 3000, 11000, 6000, 6000, 6000],
    ["Millions", "Counting Sort", "Millions", 3000, 2008244, 6000, 6000, 6000],
    ["Blender", "Counting Sort", "Blender", 3000, 2004487, 6000, 6000, 6000],
    ["Random", "Counting Sort", "Random", 4000, 19998, 8000, 8000, 8000],
    ["Sorted", "Counting Sort", "Sorted", 4000, 20000, 8000, 8000, 8000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 4000, 16000, 8000, 8000, 8000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 4000, 16000, 8000, 8000, 8000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 4000, 18000, 8000, 8000, 8000],
    ["Reversed", "Counting Sort", "Reversed", 4000, 20000, 8000, 8000, 8000],
    ["Same", "Counting Sort", "Same", 4000, 12002, 8000, 8000, 8000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 4000, 20000, 8000, 8000, 8000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 4000, 8192, 8000, 8000, 8000],
    ["Square", "Counting Sort", "Square", 4000, 20002, 8000, 8000, 8000],
    ["Zeroes", "Counting Sort", "Zeroes", 4000, 12002, 8000, 8000, 8000],
    ["Thousands", "Counting Sort", "Thousands", 4000, 14000, 8000, 8000, 8000],
    ["Millions", "Counting Sort", "Millions", 4000, 2010782, 8000, 8000, 8000],
    ["Blender", "Counting Sort", "Blender", 4000, 2011336, 8000, 8000, 8000],
    ["Random", "Counting Sort", "Random", 6000, 30000, 12000, 12000, 12000],
    ["Sorted", "Counting Sort", "Sorted", 6000, 30000, 12000, 12000, 12000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 6000, 24000, 12000, 12000, 12000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 6000, 24000, 12000, 12000, 12000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 6000, 27000, 12000, 12000, 12000],
    ["Reversed", "Counting Sort", "Reversed", 6000, 30000, 12000, 12000, 12000],
    ["Same", "Counting Sort", "Same", 6000, 18002, 12000, 12000, 12000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 6000, 30000, 12000, 12000, 12000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 6000, 12234, 12000, 12000, 12000],
    ["Square", "Counting Sort", "Square", 6000, 30002, 12000, 12000, 12000],
    ["Zeroes", "Counting Sort", "Zeroes", 6000, 18002, 12000, 12000, 12000],
    ["Thousands", "Counting Sort", "Thousands", 6000, 20000, 12000, 12000, 12000],
    ["Millions", "Counting Sort", "Millions", 6000, 2017260, 12000, 12000, 12000],
    ["Blender", "Counting Sort", "Blender", 6000, 2012105, 12000, 12000, 12000],
    ["Random", "Counting Sort", "Random", 8000, 40000, 16000, 16000, 16000],
    ["Sorted", "Counting Sort", "Sorted", 8000, 40000, 16000, 16000, 16000],
    ["Half Sorted", "Counting Sort", "Half Sorted", 8000, 32000, 16000, 16000, 16000],
    ["Sorted Half", "Counting Sort", "Sorted Half", 8000, 32000, 16000, 16000, 16000],
    ["Semi Sorted", "Counting Sort", "Semi Sorted", 8000, 35998, 16000, 16000, 16000],
    ["Reversed", "Counting Sort", "Reversed", 8000, 40000, 16000, 16000, 16000],
    ["Same", "Counting Sort", "Same", 8000, 24002, 16000, 16000, 16000],
    ["Zig Zag", "Counting Sort", "Zig Zag", 8000, 40000, 16000, 16000, 16000],
    ["Sawtooth", "Counting Sort", "Sawtooth", 8000, 16270, 16000, 16000, 16000],
    ["Square", "Counting Sort", "Square", 8000, 40002, 16000, 16000, 16000],
    ["Zeroes", "Counting Sort", "Zeroes", 8000, 24002, 16000, 16000, 16000],
    ["Thousands", "Counting Sort", "Thousands", 8000, 26000, 16000, 16000, 16000],
    ["Millions", "Counting Sort", "Millions", 8000, 2023640, 16000, 16000, 16000],
    ["Blender", "Counting Sort", "Blender", 8000, 2021488, 16000, 16000, 16000],
  ],
  "linearithmics-race": [
    ["Merge Sort", "Merge Sort", "Random", 11, 78, 38, 38, 38],
    ["Tim Sort", "Tim Sort", "Random", 11, 44, 11, 38, 38],
    ["Heap Sort", "Heap Sort", "Random", 11, 45, 38, 38, 38],
    ["Merge Sort", "Merge Sort", "Sorted", 9, 58, 29, 29, 29],
    ["Tim Sort", "Tim Sort", "Sorted", 9, 7, 9, 29, 29],
    ["Heap Sort", "Heap Sort", "Sorted", 9, 36, 29, 29, 29],
    ["Merge Sort", "Merge Sort", "Half Sorted", 6, 32, 16, 16, 16],
    ["Tim Sort", "Tim Sort", "Half Sorted", 6, 15, 6, 16, 16],
    ["Heap Sort", "Heap Sort", "Half Sorted", 6, 18, 16, 16, 16],
    ["Merge Sort", "Merge Sort", "Sorted Half", 10, 68, 33, 33, 33],
    ["Tim Sort", "Tim Sort", "Sorted Half", 10, 29, 10, 33, 33],
    ["Heap Sort", "Heap Sort", "Sorted Half", 10, 37, 33, 33, 33],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 9, 58, 29, 29, 29],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 9, 7, 9, 29, 29],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 9, 36, 29, 29, 29],
    ["Merge Sort", "Merge Sort", "Reversed", 12, 88, 43, 43, 43],
    ["Tim Sort", "Tim Sort", "Reversed", 12, 81, 12, 43, 43],
    ["Heap Sort", "Heap Sort", "Reversed", 12, 42, 43, 43, 43],
    ["Merge Sort", "Merge Sort", "Same", 11, 78, 38, 38, 38],
    ["Tim Sort", "Tim Sort", "Same", 11, 9, 11, 38, 38],
    ["Heap Sort", "Heap Sort", "Same", 11, 22, 38, 38, 38],
    ["Merge Sort", "Merge Sort", "Zig Zag", 8, 48, 24, 24, 24],
    ["Tim Sort", "Tim Sort", "Zig Zag", 8, 26, 8, 24, 24],
    ["Heap Sort", "Heap Sort", "Zig Zag", 8, 30, 24, 24, 24],
    ["Merge Sort", "Merge Sort", "Sawtooth", 11, 78, 38, 38, 38],
    ["Tim Sort", "Tim Sort", "Sawtooth", 11, 40, 11, 38, 38],
    ["Heap Sort", "Heap Sort", "Sawtooth", 11, 46, 38, 38, 38],
    ["Merge Sort", "Merge Sort", "Square", 12, 88, 43, 43, 43],
    ["Tim Sort", "Tim Sort", "Square", 12, 38, 12, 43, 43],
    ["Heap Sort", "Heap Sort", "Square", 12, 36, 43, 43, 43],
    ["Merge Sort", "Merge Sort", "Zeroes", 10, 68, 33, 33, 33],
    ["Tim Sort", "Tim Sort", "Zeroes", 10, 8, 10, 33, 33],
    ["Heap Sort", "Heap Sort", "Zeroes", 10, 21, 33, 33, 33],
    ["Merge Sort", "Merge Sort", "Thousands", 12, 88, 43, 43, 43],
    ["Tim Sort", "Tim Sort", "Thousands", 12, 56, 12, 43, 43],
    ["Heap Sort", "Heap Sort", "Thousands", 12, 50, 43, 43, 43],
    ["Merge Sort", "Merge Sort", "Millions", 10, 68, 33, 33, 33],
    ["Tim Sort", "Tim Sort", "Millions", 10, 43, 10, 33, 33],
    ["Heap Sort", "Heap Sort", "Millions", 10, 39, 33, 33, 33],
    ["Merge Sort", "Merge Sort", "Blender", 8, 48, 24, 24, 24],
    ["Tim Sort", "Tim Sort", "Blender", 8, 17, 8, 24, 24],
    ["Heap Sort", "Heap Sort", "Blender", 8, 28, 24, 24, 24],
    ["Merge Sort", "Merge Sort", "Random", 13, 98, 48, 48, 48],
    ["Tim Sort", "Tim Sort", "Random", 13, 58, 13, 48, 48],
    ["Heap Sort", "Heap Sort", "Random", 13, 47, 48, 48, 48],
    ["Merge Sort", "Merge Sort", "Sorted", 17, 140, 69, 69, 69],
    ["Tim Sort", "Tim Sort", "Sorted", 17, 98, 17, 69, 69],
    ["Heap Sort", "Heap Sort", "Sorted", 17, 82, 69, 69, 69],
    ["Merge Sort", "Merge Sort", "Half Sorted", 16, 128, 64, 64, 64],
    ["Tim Sort", "Tim Sort", "Half Sorted", 16, 77, 16, 64, 64],
    ["Heap Sort", "Heap Sort", "Half Sorted", 16, 68, 64, 64, 64],
    ["Merge Sort", "Merge Sort", "Sorted Half", 14, 108, 53, 53, 53],
    ["Tim Sort", "Tim Sort", "Sorted Half", 14, 64, 14, 53, 53],
    ["Heap Sort", "Heap Sort", "Sorted Half", 14, 54, 53, 53, 53],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 12, 88, 43, 43, 43],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 12, 35, 12, 43, 43],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 12, 49, 43, 43, 43],
    ["Merge Sort", "Merge Sort", "Reversed", 18, 152, 75, 75, 75],
    ["Tim Sort", "Tim Sort", "Reversed", 18, 113, 18, 75, 75],
    ["Heap Sort", "Heap Sort", "Reversed", 18, 71, 75, 75, 75],
    ["Merge Sort", "Merge Sort", "Same", 15, 118, 59, 59, 59],
    ["Tim Sort", "Tim Sort", "Same", 15, 13, 15, 59, 59],
    ["Heap Sort", "Heap Sort", "Same", 15, 30, 59, 59, 59],
    ["Merge Sort", "Merge Sort", "Zig Zag", 15, 118, 59, 59, 59],
    ["Tim Sort", "Tim Sort", "Zig Zag", 15, 92, 15, 59, 59],
    ["Heap Sort", "Heap Sort", "Zig Zag", 15, 64, 59, 59, 59],
    ["Merge Sort", "Merge Sort", "Sawtooth", 14, 108, 53, 53, 53],
    ["Tim Sort", "Tim Sort", "Sawtooth", 14, 70, 14, 53, 53],
    ["Heap Sort", "Heap Sort", "Sawtooth", 14, 61, 53, 53, 53],
    ["Merge Sort", "Merge Sort", "Square", 15, 118, 59, 59, 59],
    ["Tim Sort", "Tim Sort", "Square", 15, 66, 15, 59, 59],
    ["Heap Sort", "Heap Sort", "Square", 15, 44, 59, 59, 59],
    ["Merge Sort", "Merge Sort", "Zeroes", 17, 140, 69, 69, 69],
    ["Tim Sort", "Tim Sort", "Zeroes", 17, 15, 17, 69, 69],
    ["Heap Sort", "Heap Sort", "Zeroes", 17, 34, 69, 69, 69],
    ["Merge Sort", "Merge Sort", "Thousands", 12, 88, 43, 43, 43],
    ["Tim Sort", "Tim Sort", "Thousands", 12, 60, 12, 43, 43],
    ["Heap Sort", "Heap Sort", "Thousands", 12, 51, 43, 43, 43],
    ["Merge Sort", "Merge Sort", "Millions", 13, 98, 48, 48, 48],
    ["Tim Sort", "Tim Sort", "Millions", 13, 69, 13, 48, 48],
    ["Heap Sort", "Heap Sort", "Millions", 13, 54, 48, 48, 48],
    ["Merge Sort", "Merge Sort", "Blender", 16, 128, 64, 64, 64],
    ["Tim Sort", "Tim Sort", "Blender", 16, 115, 16, 64, 64],
    ["Heap Sort", "Heap Sort", "Blender", 16, 69, 64, 64, 64],
    ["Merge Sort", "Merge Sort", "Random", 20, 176, 86, 86, 86],
    ["Tim Sort", "Tim Sort", "Random", 20, 161, 20, 86, 86],
    ["Heap Sort", "Heap Sort", "Random", 20, 89, 86, 86, 86],
    ["Merge Sort", "Merge Sort", "Sorted", 20, 176, 86, 86, 86],
    ["Tim Sort", "Tim Sort", "Sorted", 20, 140, 20, 86, 86],
    ["Heap Sort", "Heap Sort", "Sorted", 20, 102, 86, 86, 86],
    ["Merge Sort", "Merge Sort", "Half Sorted", 22, 200, 98, 98, 98],
    ["Tim Sort", "Tim Sort", "Half Sorted", 22, 162, 22, 98, 98],
    ["Heap Sort", "Heap Sort", "Half Sorted", 22, 103, 98, 98, 98],
    ["Merge Sort", "Merge Sort", "Sorted Half", 24, 224, 110, 110, 110],
    ["Tim Sort", "Tim Sort", "Sorted Half", 24, 173, 24, 110, 110],
    ["Heap Sort", "Heap Sort", "Sorted Half", 24, 119, 110, 110, 110],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 24, 224, 110, 110, 110],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 24, 207, 24, 110, 110],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 24, 122, 110, 110, 110],
    ["Merge Sort", "Merge Sort", "Reversed", 23, 212, 104, 104, 104],
    ["Tim Sort", "Tim Sort", "Reversed", 23, 136, 23, 104, 104],
    ["Heap Sort", "Heap Sort", "Reversed", 23, 97, 104, 104, 104],
    ["Merge Sort", "Merge Sort", "Same", 20, 176, 86, 86, 86],
    ["Tim Sort", "Tim Sort", "Same", 20, 18, 20, 86, 86],
    ["Heap Sort", "Heap Sort", "Same", 20, 41, 86, 86, 86],
    ["Merge Sort", "Merge Sort", "Zig Zag", 15, 118, 59, 59, 59],
    ["Tim Sort", "Tim Sort", "Zig Zag", 15, 92, 15, 59, 59],
    ["Heap Sort", "Heap Sort", "Zig Zag", 15, 64, 59, 59, 59],
    ["Merge Sort", "Merge Sort", "Sawtooth", 16, 128, 64, 64, 64],
    ["Tim Sort", "Tim Sort", "Sawtooth", 16, 103, 16, 64, 64],
    ["Heap Sort", "Heap Sort", "Sawtooth", 16, 69, 64, 64, 64],
    ["Merge Sort", "Merge Sort", "Square", 22, 200, 98, 98, 98],
    ["Tim Sort", "Tim Sort", "Square", 22, 132, 22, 98, 98],
    ["Heap Sort", "Heap Sort", "Square", 22, 72, 98, 98, 98],
    ["Merge Sort", "Merge Sort", "Zeroes", 17, 140, 69, 69, 69],
    ["Tim Sort", "Tim Sort", "Zeroes", 17, 15, 17, 69, 69],
    ["Heap Sort", "Heap Sort", "Zeroes", 17, 34, 69, 69, 69],
    ["Merge Sort", "Merge Sort", "Thousands", 24, 224, 110, 110, 110],
    ["Tim Sort", "Tim Sort", "Thousands", 24, 281, 24, 110, 110],
    ["Heap Sort", "Heap Sort", "Thousands", 24, 115, 110, 110, 110],
    ["Merge Sort", "Merge Sort", "Millions", 22, 200, 98, 98, 98],
    ["Tim Sort", "Tim Sort", "Millions", 22, 164, 22, 98, 98],
    ["Heap Sort", "Heap Sort", "Millions", 22, 105, 98, 98, 98],
    ["Merge Sort", "Merge Sort", "Blender", 16, 128, 64, 64, 64],
    ["Tim Sort", "Tim Sort", "Blender", 16, 100, 16, 64, 64],
    ["Heap Sort", "Heap Sort", "Blender", 16, 67, 64, 64, 64],
    ["Merge Sort", "Merge Sort", "Random", 28, 272, 135, 135, 135],
    ["Tim Sort", "Tim Sort", "Random", 28, 301, 28, 135, 135],
    ["Heap Sort", "Heap Sort", "Random", 28, 136, 135, 135, 135],
    ["Merge Sort", "Merge Sort", "Sorted", 27, 260, 128, 128, 128],
    ["Tim Sort", "Tim Sort", "Sorted", 27, 230, 27, 128, 128],
    ["Heap Sort", "Heap Sort", "Sorted", 27, 146, 128, 128, 128],
    ["Merge Sort", "Merge Sort", "Half Sorted", 30, 296, 147, 147, 147],
    ["Tim Sort", "Tim Sort", "Half Sorted", 30, 290, 30, 147, 147],
    ["Heap Sort", "Heap Sort", "Half Sorted", 30, 160, 147, 147, 147],
    ["Merge Sort", "Merge Sort", "Sorted Half", 36, 376, 186, 186, 186],
    ["Tim Sort", "Tim Sort", "Sorted Half", 36, 304, 36, 186, 186],
    ["Heap Sort", "Heap Sort", "Sorted Half", 36, 197, 186, 186, 186],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 28, 272, 135, 135, 135],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 28, 289, 28, 135, 135],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 28, 148, 135, 135, 135],
    ["Merge Sort", "Merge Sort", "Reversed", 33, 334, 166, 166, 166],
    ["Tim Sort", "Tim Sort", "Reversed", 33, 86, 33, 166, 166],
    ["Heap Sort", "Heap Sort", "Reversed", 33, 152, 166, 166, 166],
    ["Merge Sort", "Merge Sort", "Same", 32, 320, 160, 160, 160],
    ["Tim Sort", "Tim Sort", "Same", 32, 32, 32, 160, 160],
    ["Heap Sort", "Heap Sort", "Same", 32, 65, 160, 160, 160],
    ["Merge Sort", "Merge Sort", "Zig Zag", 23, 212, 104, 104, 104],
    ["Tim Sort", "Tim Sort", "Zig Zag", 23, 209, 23, 104, 104],
    ["Heap Sort", "Heap Sort", "Zig Zag", 23, 114, 104, 104, 104],
    ["Merge Sort", "Merge Sort", "Sawtooth", 37, 390, 193, 193, 193],
    ["Tim Sort", "Tim Sort", "Sawtooth", 37, 292, 37, 193, 193],
    ["Heap Sort", "Heap Sort", "Sawtooth", 37, 197, 193, 193, 193],
    ["Merge Sort", "Merge Sort", "Square", 30, 296, 147, 147, 147],
    ["Tim Sort", "Tim Sort", "Square", 30, 211, 30, 147, 147],
    ["Heap Sort", "Heap Sort", "Square", 30, 107, 147, 147, 147],
    ["Merge Sort", "Merge Sort", "Zeroes", 37, 390, 193, 193, 193],
    ["Tim Sort", "Tim Sort", "Zeroes", 37, 37, 37, 193, 193],
    ["Heap Sort", "Heap Sort", "Zeroes", 37, 74, 193, 193, 193],
    ["Merge Sort", "Merge Sort", "Thousands", 25, 236, 116, 116, 116],
    ["Tim Sort", "Tim Sort", "Thousands", 25, 255, 25, 116, 116],
    ["Heap Sort", "Heap Sort", "Thousands", 25, 117, 116, 116, 116],
    ["Merge Sort", "Merge Sort", "Millions", 25, 236, 116, 116, 116],
    ["Tim Sort", "Tim Sort", "Millions", 25, 278, 25, 116, 116],
    ["Heap Sort", "Heap Sort", "Millions", 25, 121, 116, 116, 116],
    ["Merge Sort", "Merge Sort", "Blender", 33, 334, 166, 166, 166],
    ["Tim Sort", "Tim Sort", "Blender", 33, 290, 33, 166, 166],
    ["Heap Sort", "Heap Sort", "Blender", 33, 173, 166, 166, 166],
    ["Merge Sort", "Merge Sort", "Random", 45, 502, 247, 247, 247],
    ["Tim Sort", "Tim Sort", "Random", 45, 473, 45, 247, 247],
    ["Heap Sort", "Heap Sort", "Random", 45, 247, 247, 247, 247],
    ["Merge Sort", "Merge Sort", "Sorted", 32, 320, 160, 160, 160],
    ["Tim Sort", "Tim Sort", "Sorted", 32, 143, 32, 160, 160],
    ["Heap Sort", "Heap Sort", "Sorted", 32, 180, 160, 160, 160],
    ["Merge Sort", "Merge Sort", "Half Sorted", 46, 516, 254, 254, 254],
    ["Tim Sort", "Tim Sort", "Half Sorted", 46, 435, 46, 254, 254],
    ["Heap Sort", "Heap Sort", "Half Sorted", 46, 270, 254, 254, 254],
    ["Merge Sort", "Merge Sort", "Sorted Half", 34, 348, 173, 173, 173],
    ["Tim Sort", "Tim Sort", "Sorted Half", 34, 257, 34, 173, 173],
    ["Heap Sort", "Heap Sort", "Sorted Half", 34, 181, 173, 173, 173],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 34, 348, 173, 173, 173],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 34, 251, 34, 173, 173],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 34, 187, 173, 173, 173],
    ["Merge Sort", "Merge Sort", "Reversed", 49, 558, 275, 275, 275],
    ["Tim Sort", "Tim Sort", "Reversed", 49, 124, 49, 275, 275],
    ["Heap Sort", "Heap Sort", "Reversed", 49, 250, 275, 275, 275],
    ["Merge Sort", "Merge Sort", "Same", 30, 296, 147, 147, 147],
    ["Tim Sort", "Tim Sort", "Same", 30, 28, 30, 147, 147],
    ["Heap Sort", "Heap Sort", "Same", 30, 61, 147, 147, 147],
    ["Merge Sort", "Merge Sort", "Zig Zag", 40, 432, 213, 213, 213],
    ["Tim Sort", "Tim Sort", "Zig Zag", 40, 381, 40, 213, 213],
    ["Heap Sort", "Heap Sort", "Zig Zag", 40, 228, 213, 213, 213],
    ["Merge Sort", "Merge Sort", "Sawtooth", 47, 530, 261, 261, 261],
    ["Tim Sort", "Tim Sort", "Sawtooth", 47, 459, 47, 261, 261],
    ["Heap Sort", "Heap Sort", "Sawtooth", 47, 273, 261, 261, 261],
    ["Merge Sort", "Merge Sort", "Square", 38, 404, 199, 199, 199],
    ["Tim Sort", "Tim Sort", "Square", 38, 208, 38, 199, 199],
    ["Heap Sort", "Heap Sort", "Square", 38, 134, 199, 199, 199],
    ["Merge Sort", "Merge Sort", "Zeroes", 41, 446, 220, 220, 220],
    ["Tim Sort", "Tim Sort", "Zeroes", 41, 41, 41, 220, 220],
    ["Heap Sort", "Heap Sort", "Zeroes", 41, 82, 220, 220, 220],
    ["Merge Sort", "Merge Sort", "Thousands", 49, 558, 275, 275, 275],
    ["Tim Sort", "Tim Sort", "Thousands", 49, 579, 49, 275, 275],
    ["Heap Sort", "Heap Sort", "Thousands", 49, 291, 275, 275, 275],
    ["Merge Sort", "Merge Sort", "Millions", 45, 502, 247, 247, 247],
    ["Tim Sort", "Tim Sort", "Millions", 45, 435, 45, 247, 247],
    ["Heap Sort", "Heap Sort", "Millions", 45, 260, 247, 247, 247],
    ["Merge Sort", "Merge Sort", "Blender", 30, 296, 147, 147, 147],
    ["Tim Sort", "Tim Sort", "Blender", 30, 325, 30, 147, 147],
    ["Heap Sort", "Heap Sort", "Blender", 30, 156, 147, 147, 147],
    ["Merge Sort", "Merge Sort", "Random", 46, 516, 254, 254, 254],
    ["Tim Sort", "Tim Sort", "Random", 46, 478, 46, 254, 254],
    ["Heap Sort", "Heap Sort", "Random", 46, 262, 254, 254, 254],
    ["Merge Sort", "Merge Sort", "Sorted", 68, 832, 414, 414, 414],
    ["Tim Sort", "Tim Sort", "Sorted", 68, 231, 68, 414, 414],
    ["Heap Sort", "Heap Sort", "Sorted", 68, 462, 414, 414, 414],
    ["Merge Sort", "Merge Sort", "Half Sorted", 62, 740, 369, 369, 369],
    ["Tim Sort", "Tim Sort", "Half Sorted", 62, 738, 62, 369, 369],
    ["Heap Sort", "Heap Sort", "Half Sorted", 62, 383, 369, 369, 369],
    ["Merge Sort", "Merge Sort", "Sorted Half", 70, 864, 429, 429, 429],
    ["Tim Sort", "Tim Sort", "Sorted Half", 70, 636, 70, 429, 429],
    ["Heap Sort", "Heap Sort", "Sorted Half", 70, 420, 429, 429, 429],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 45, 502, 247, 247, 247],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 45, 380, 45, 247, 247],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 45, 274, 247, 247, 247],
    ["Merge Sort", "Merge Sort", "Reversed", 71, 880, 437, 437, 437],
    ["Tim Sort", "Tim Sort", "Reversed", 71, 185, 71, 437, 437],
    ["Heap Sort", "Heap Sort", "Reversed", 71, 399, 437, 437, 437],
    ["Merge Sort", "Merge Sort", "Same", 51, 586, 289, 289, 289],
    ["Tim Sort", "Tim Sort", "Same", 51, 51, 51, 289, 289],
    ["Heap Sort", "Heap Sort", "Same", 51, 102, 289, 289, 289],
    ["Merge Sort", "Merge Sort", "Zig Zag", 68, 832, 414, 414, 414],
    ["Tim Sort", "Tim Sort", "Zig Zag", 68, 678, 68, 414, 414],
    ["Heap Sort", "Heap Sort", "Zig Zag", 68, 433, 414, 414, 414],
    ["Merge Sort", "Merge Sort", "Sawtooth", 58, 684, 340, 340, 340],
    ["Tim Sort", "Tim Sort", "Sawtooth", 58, 684, 58, 340, 340],
    ["Heap Sort", "Heap Sort", "Sawtooth", 58, 349, 340, 340, 340],
    ["Merge Sort", "Merge Sort", "Square", 45, 502, 247, 247, 247],
    ["Tim Sort", "Tim Sort", "Square", 45, 333, 45, 247, 247],
    ["Heap Sort", "Heap Sort", "Square", 45, 162, 247, 247, 247],
    ["Merge Sort", "Merge Sort", "Zeroes", 56, 656, 325, 325, 325],
    ["Tim Sort", "Tim Sort", "Zeroes", 56, 56, 56, 325, 325],
    ["Heap Sort", "Heap Sort", "Zeroes", 56, 113, 325, 325, 325],
    ["Merge Sort", "Merge Sort", "Thousands", 74, 928, 459, 459, 459],
    ["Tim Sort", "Tim Sort", "Thousands", 74, 848, 74, 459, 459],
    ["Heap Sort", "Heap Sort", "Thousands", 74, 450, 459, 459, 459],
    ["Merge Sort", "Merge Sort", "Millions", 63, 754, 377, 377, 377],
    ["Tim Sort", "Tim Sort", "Millions", 63, 905, 63, 377, 377],
    ["Heap Sort", "Heap Sort", "Millions", 63, 377, 377, 377, 377],
    ["Merge Sort", "Merge Sort", "Blender", 53, 614, 304, 304, 304],
    ["Tim Sort", "Tim Sort", "Blender", 53, 584, 53, 304, 304],
    ["Heap Sort", "Heap Sort", "Blender", 53, 314, 304, 304, 304],
    ["Merge Sort", "Merge Sort", "Random", 79, 1008, 498, 498, 498],
    ["Tim Sort", "Tim Sort", "Random", 79, 888, 79, 498, 498],
    ["Heap Sort", "Heap Sort", "Random", 79, 506, 498, 498, 498],
    ["Merge Sort", "Merge Sort", "Sorted", 77, 976, 483, 483, 483],
    ["Tim Sort", "Tim Sort", "Sorted", 77, 285, 77, 483, 483],
    ["Heap Sort", "Heap Sort", "Sorted", 77, 540, 483, 483, 483],
    ["Merge Sort", "Merge Sort", "Half Sorted", 70, 864, 429, 429, 429],
    ["Tim Sort", "Tim Sort", "Half Sorted", 70, 569, 70, 429, 429],
    ["Heap Sort", "Heap Sort", "Half Sorted", 70, 452, 429, 429, 429],
    ["Merge Sort", "Merge Sort", "Sorted Half", 62, 740, 369, 369, 369],
    ["Tim Sort", "Tim Sort", "Sorted Half", 62, 742, 62, 369, 369],
    ["Heap Sort", "Heap Sort", "Sorted Half", 62, 354, 369, 369, 369],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 88, 1152, 568, 568, 568],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 88, 788, 88, 568, 568],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 88, 601, 568, 568, 568],
    ["Merge Sort", "Merge Sort", "Reversed", 65, 784, 391, 391, 391],
    ["Tim Sort", "Tim Sort", "Reversed", 65, 169, 65, 391, 391],
    ["Heap Sort", "Heap Sort", "Reversed", 65, 364, 391, 391, 391],
    ["Merge Sort", "Merge Sort", "Same", 69, 848, 421, 421, 421],
    ["Tim Sort", "Tim Sort", "Same", 69, 70, 69, 421, 421],
    ["Heap Sort", "Heap Sort", "Same", 69, 138, 421, 421, 421],
    ["Merge Sort", "Merge Sort", "Zig Zag", 70, 864, 429, 429, 429],
    ["Tim Sort", "Tim Sort", "Zig Zag", 70, 714, 70, 429, 429],
    ["Heap Sort", "Heap Sort", "Zig Zag", 70, 449, 429, 429, 429],
    ["Merge Sort", "Merge Sort", "Sawtooth", 76, 960, 475, 475, 475],
    ["Tim Sort", "Tim Sort", "Sawtooth", 76, 738, 76, 475, 475],
    ["Heap Sort", "Heap Sort", "Sawtooth", 76, 496, 475, 475, 475],
    ["Merge Sort", "Merge Sort", "Square", 98, 1312, 648, 648, 648],
    ["Tim Sort", "Tim Sort", "Square", 98, 838, 98, 648, 648],
    ["Heap Sort", "Heap Sort", "Square", 98, 417, 648, 648, 648],
    ["Merge Sort", "Merge Sort", "Zeroes", 63, 754, 377, 377, 377],
    ["Tim Sort", "Tim Sort", "Zeroes", 63, 63, 63, 377, 377],
    ["Heap Sort", "Heap Sort", "Zeroes", 63, 126, 377, 377, 377],
    ["Merge Sort", "Merge Sort", "Thousands", 76, 960, 475, 475, 475],
    ["Tim Sort", "Tim Sort", "Thousands", 76, 829, 76, 475, 475],
    ["Heap Sort", "Heap Sort", "Thousands", 76, 487, 475, 475, 475],
    ["Merge Sort", "Merge Sort", "Millions", 84, 1088, 537, 537, 537],
    ["Tim Sort", "Tim Sort", "Millions", 84, 971, 84, 537, 537],
    ["Heap Sort", "Heap Sort", "Millions", 84, 550, 537, 537, 537],
    ["Merge Sort", "Merge Sort", "Blender", 95, 1264, 624, 624, 624],
    ["Tim Sort", "Tim Sort", "Blender", 95, 1180, 95, 624, 624],
    ["Heap Sort", "Heap Sort", "Blender", 95, 646, 624, 624, 624],
    ["Merge Sort", "Merge Sort", "Random", 120, 1664, 829, 829, 829],
    ["Tim Sort", "Tim Sort", "Random", 120, 1691, 120, 829, 829],
    ["Heap Sort", "Heap Sort", "Random", 120, 845, 829, 829, 829],
    ["Merge Sort", "Merge Sort", "Sorted", 104, 1408, 697, 697, 697],
    ["Tim Sort", "Tim Sort", "Sorted", 104, 515, 104, 697, 697],
    ["Heap Sort", "Heap Sort", "Sorted", 104, 774, 697, 697, 697],
    ["Merge Sort", "Merge Sort", "Half Sorted", 114, 1568, 779, 779, 779],
    ["Tim Sort", "Tim Sort", "Half Sorted", 114, 1304, 114, 779, 779],
    ["Heap Sort", "Heap Sort", "Half Sorted", 114, 804, 779, 779, 779],
    ["Merge Sort", "Merge Sort", "Sorted Half", 110, 1504, 746, 746, 746],
    ["Tim Sort", "Tim Sort", "Sorted Half", 110, 1240, 110, 746, 746],
    ["Heap Sort", "Heap Sort", "Sorted Half", 110, 737, 746, 746, 746],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 120, 1664, 829, 829, 829],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 120, 1472, 120, 829, 829],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 120, 888, 829, 829, 829],
    ["Merge Sort", "Merge Sort", "Reversed", 105, 1424, 705, 705, 705],
    ["Tim Sort", "Tim Sort", "Reversed", 105, 705, 105, 705, 705],
    ["Heap Sort", "Heap Sort", "Reversed", 105, 654, 705, 705, 705],
    ["Merge Sort", "Merge Sort", "Same", 79, 1008, 498, 498, 498],
    ["Tim Sort", "Tim Sort", "Same", 79, 80, 79, 498, 498],
    ["Heap Sort", "Heap Sort", "Same", 79, 158, 498, 498, 498],
    ["Merge Sort", "Merge Sort", "Zig Zag", 104, 1408, 697, 697, 697],
    ["Tim Sort", "Tim Sort", "Zig Zag", 104, 1337, 104, 697, 697],
    ["Heap Sort", "Heap Sort", "Zig Zag", 104, 726, 697, 697, 697],
    ["Merge Sort", "Merge Sort", "Sawtooth", 106, 1440, 713, 713, 713],
    ["Tim Sort", "Tim Sort", "Sawtooth", 106, 1322, 106, 713, 713],
    ["Heap Sort", "Heap Sort", "Sawtooth", 106, 731, 713, 713, 713],
    ["Merge Sort", "Merge Sort", "Square", 103, 1392, 689, 689, 689],
    ["Tim Sort", "Tim Sort", "Square", 103, 903, 103, 689, 689],
    ["Heap Sort", "Heap Sort", "Square", 103, 432, 689, 689, 689],
    ["Merge Sort", "Merge Sort", "Zeroes", 119, 1648, 820, 820, 820],
    ["Tim Sort", "Tim Sort", "Zeroes", 119, 120, 119, 820, 820],
    ["Heap Sort", "Heap Sort", "Zeroes", 119, 238, 820, 820, 820],
    ["Merge Sort", "Merge Sort", "Thousands", 97, 1296, 640, 640, 640],
    ["Tim Sort", "Tim Sort", "Thousands", 97, 1204, 97, 640, 640],
    ["Heap Sort", "Heap Sort", "Thousands", 97, 655, 640, 640, 640],
    ["Merge Sort", "Merge Sort", "Millions", 80, 1024, 506, 506, 506],
    ["Tim Sort", "Tim Sort", "Millions", 80, 921, 80, 506, 506],
    ["Heap Sort", "Heap Sort", "Millions", 80, 518, 506, 506, 506],
    ["Merge Sort", "Merge Sort", "Blender", 101, 1360, 672, 672, 672],
    ["Tim Sort", "Tim Sort", "Blender", 101, 1240, 101, 672, 672],
    ["Heap Sort", "Heap Sort", "Blender", 101, 652, 672, 672, 672],
    ["Merge Sort", "Merge Sort", "Random", 183, 2782, 1375, 1375, 1375],
    ["Tim Sort", "Tim Sort", "Random", 183, 2515, 183, 1375, 1375],
    ["Heap Sort", "Heap Sort", "Random", 183, 1402, 1375, 1375, 1375],
    ["Merge Sort", "Merge Sort", "Sorted", 181, 2746, 1357, 1357, 1357],
    ["Tim Sort", "Tim Sort", "Sorted", 181, 738, 181, 1357, 1357],
    ["Heap Sort", "Heap Sort", "Sorted", 181, 1504, 1357, 1357, 1357],
    ["Merge Sort", "Merge Sort", "Half Sorted", 138, 1972, 981, 981, 981],
    ["Tim Sort", "Tim Sort", "Half Sorted", 138, 1118, 138, 981, 981],
    ["Heap Sort", "Heap Sort", "Half Sorted", 138, 1025, 981, 981, 981],
    ["Merge Sort", "Merge Sort", "Sorted Half", 116, 1600, 796, 796, 796],
    ["Tim Sort", "Tim Sort", "Sorted Half", 116, 1180, 116, 796, 796],
    ["Heap Sort", "Heap Sort", "Sorted Half", 116, 786, 796, 796, 796],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 175, 2638, 1304, 1304, 1304],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 175, 2113, 175, 1304, 1304],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 175, 1359, 1304, 1304, 1304],
    ["Merge Sort", "Merge Sort", "Reversed", 135, 1918, 955, 955, 955],
    ["Tim Sort", "Tim Sort", "Reversed", 135, 407, 135, 955, 955],
    ["Heap Sort", "Heap Sort", "Reversed", 135, 889, 955, 955, 955],
    ["Merge Sort", "Merge Sort", "Same", 175, 2638, 1304, 1304, 1304],
    ["Tim Sort", "Tim Sort", "Same", 175, 177, 175, 1304, 1304],
    ["Heap Sort", "Heap Sort", "Same", 175, 350, 1304, 1304, 1304],
    ["Merge Sort", "Merge Sort", "Zig Zag", 134, 1900, 947, 947, 947],
    ["Tim Sort", "Tim Sort", "Zig Zag", 134, 1523, 134, 947, 947],
    ["Heap Sort", "Heap Sort", "Zig Zag", 134, 981, 947, 947, 947],
    ["Merge Sort", "Merge Sort", "Sawtooth", 146, 2116, 1050, 1050, 1050],
    ["Tim Sort", "Tim Sort", "Sawtooth", 146, 1664, 146, 1050, 1050],
    ["Heap Sort", "Heap Sort", "Sawtooth", 146, 1062, 1050, 1050, 1050],
    ["Merge Sort", "Merge Sort", "Square", 143, 2062, 1024, 1024, 1024],
    ["Tim Sort", "Tim Sort", "Square", 143, 1089, 143, 1024, 1024],
    ["Heap Sort", "Heap Sort", "Square", 143, 613, 1024, 1024, 1024],
    ["Merge Sort", "Merge Sort", "Zeroes", 154, 2260, 1119, 1119, 1119],
    ["Tim Sort", "Tim Sort", "Zeroes", 154, 156, 154, 1119, 1119],
    ["Heap Sort", "Heap Sort", "Zeroes", 154, 309, 1119, 1119, 1119],
    ["Merge Sort", "Merge Sort", "Thousands", 144, 2080, 1032, 1032, 1032],
    ["Tim Sort", "Tim Sort", "Thousands", 144, 1742, 144, 1032, 1032],
    ["Heap Sort", "Heap Sort", "Thousands", 144, 1049, 1032, 1032, 1032],
    ["Merge Sort", "Merge Sort", "Millions", 170, 2548, 1260, 1260, 1260],
    ["Tim Sort", "Tim Sort", "Millions", 170, 2206, 170, 1260, 1260],
    ["Heap Sort", "Heap Sort", "Millions", 170, 1280, 1260, 1260, 1260],
    ["Merge Sort", "Merge Sort", "Blender", 168, 2512, 1242, 1242, 1242],
    ["Tim Sort", "Tim Sort", "Blender", 168, 2102, 168, 1242, 1242],
    ["Heap Sort", "Heap Sort", "Blender", 168, 1208, 1242, 1242, 1242],
    ["Merge Sort", "Merge Sort", "Random", 159, 2350, 1163, 1163, 1163],
    ["Tim Sort", "Tim Sort", "Random", 159, 1971, 159, 1163, 1163],
    ["Heap Sort", "Heap Sort", "Random", 159, 1187, 1163, 1163, 1163],
    ["Merge Sort", "Merge Sort", "Sorted", 225, 3538, 1758, 1758, 1758],
    ["Tim Sort", "Tim Sort", "Sorted", 225, 921, 225, 1758, 1758],
    ["Heap Sort", "Heap Sort", "Sorted", 225, 1928, 1758, 1758, 1758],
    ["Merge Sort", "Merge Sort", "Half Sorted", 162, 2404, 1189, 1189, 1189],
    ["Tim Sort", "Tim Sort", "Half Sorted", 162, 1437, 162, 1189, 1189],
    ["Heap Sort", "Heap Sort", "Half Sorted", 162, 1227, 1189, 1189, 1189],
    ["Merge Sort", "Merge Sort", "Sorted Half", 196, 3016, 1492, 1492, 1492],
    ["Tim Sort", "Tim Sort", "Sorted Half", 196, 1926, 196, 1492, 1492],
    ["Heap Sort", "Heap Sort", "Sorted Half", 196, 1462, 1492, 1492, 1492],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 156, 2296, 1137, 1137, 1137],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 156, 1627, 156, 1137, 1137],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 156, 1212, 1137, 1137, 1137],
    ["Merge Sort", "Merge Sort", "Reversed", 158, 2332, 1154, 1154, 1154],
    ["Tim Sort", "Tim Sort", "Reversed", 158, 492, 158, 1154, 1154],
    ["Heap Sort", "Heap Sort", "Reversed", 158, 1075, 1154, 1154, 1154],
    ["Merge Sort", "Merge Sort", "Same", 213, 3322, 1647, 1647, 1647],
    ["Tim Sort", "Tim Sort", "Same", 213, 215, 213, 1647, 1647],
    ["Heap Sort", "Heap Sort", "Same", 213, 426, 1647, 1647, 1647],
    ["Merge Sort", "Merge Sort", "Zig Zag", 150, 2188, 1084, 1084, 1084],
    ["Tim Sort", "Tim Sort", "Zig Zag", 150, 1843, 150, 1084, 1084],
    ["Heap Sort", "Heap Sort", "Zig Zag", 150, 1121, 1084, 1084, 1084],
    ["Merge Sort", "Merge Sort", "Sawtooth", 166, 2476, 1224, 1224, 1224],
    ["Tim Sort", "Tim Sort", "Sawtooth", 166, 1978, 166, 1224, 1224],
    ["Heap Sort", "Heap Sort", "Sawtooth", 166, 1255, 1224, 1224, 1224],
    ["Merge Sort", "Merge Sort", "Square", 241, 3826, 1907, 1907, 1907],
    ["Tim Sort", "Tim Sort", "Square", 241, 2243, 241, 1907, 1907],
    ["Heap Sort", "Heap Sort", "Square", 241, 1100, 1907, 1907, 1907],
    ["Merge Sort", "Merge Sort", "Zeroes", 238, 3772, 1879, 1879, 1879],
    ["Tim Sort", "Tim Sort", "Zeroes", 238, 240, 238, 1879, 1879],
    ["Heap Sort", "Heap Sort", "Zeroes", 238, 477, 1879, 1879, 1879],
    ["Merge Sort", "Merge Sort", "Thousands", 185, 2818, 1393, 1393, 1393],
    ["Tim Sort", "Tim Sort", "Thousands", 185, 2470, 185, 1393, 1393],
    ["Heap Sort", "Heap Sort", "Thousands", 185, 1404, 1393, 1393, 1393],
    ["Merge Sort", "Merge Sort", "Millions", 214, 3340, 1657, 1657, 1657],
    ["Tim Sort", "Tim Sort", "Millions", 214, 3151, 214, 1657, 1657],
    ["Heap Sort", "Heap Sort", "Millions", 214, 1702, 1657, 1657, 1657],
    ["Merge Sort", "Merge Sort", "Blender", 216, 3376, 1675, 1675, 1675],
    ["Tim Sort", "Tim Sort", "Blender", 216, 3306, 216, 1675, 1675],
    ["Heap Sort", "Heap Sort", "Blender", 216, 1615, 1675, 1675, 1675],
    ["Merge Sort", "Merge Sort", "Random", 298, 4936, 2449, 2449, 2449],
    ["Tim Sort", "Tim Sort", "Random", 298, 4079, 298, 2449, 2449],
    ["Heap Sort", "Heap Sort", "Random", 298, 2487, 2449, 2449, 2449],
    ["Merge Sort", "Merge Sort", "Sorted", 294, 4856, 2411, 2411, 2411],
    ["Tim Sort", "Tim Sort", "Sorted", 294, 998, 294, 2411, 2411],
    ["Heap Sort", "Heap Sort", "Sorted", 294, 2644, 2411, 2411, 2411],
    ["Merge Sort", "Merge Sort", "Half Sorted", 292, 4816, 2391, 2391, 2391],
    ["Tim Sort", "Tim Sort", "Half Sorted", 292, 2700, 292, 2391, 2391],
    ["Heap Sort", "Heap Sort", "Half Sorted", 292, 2512, 2391, 2391, 2391],
    ["Merge Sort", "Merge Sort", "Sorted Half", 326, 5496, 2722, 2722, 2722],
    ["Tim Sort", "Tim Sort", "Sorted Half", 326, 3338, 326, 2722, 2722],
    ["Heap Sort", "Heap Sort", "Sorted Half", 326, 2697, 2722, 2722, 2722],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 315, 5276, 2614, 2614, 2614],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 315, 3776, 315, 2614, 2614],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 315, 2755, 2614, 2614, 2614],
    ["Merge Sort", "Merge Sort", "Reversed", 283, 4636, 2305, 2305, 2305],
    ["Tim Sort", "Tim Sort", "Reversed", 283, 775, 283, 2305, 2305],
    ["Heap Sort", "Heap Sort", "Reversed", 283, 2147, 2305, 2305, 2305],
    ["Merge Sort", "Merge Sort", "Same", 297, 4916, 2440, 2440, 2440],
    ["Tim Sort", "Tim Sort", "Same", 297, 300, 297, 2440, 2440],
    ["Heap Sort", "Heap Sort", "Same", 297, 594, 2440, 2440, 2440],
    ["Merge Sort", "Merge Sort", "Zig Zag", 312, 5216, 2585, 2585, 2585],
    ["Tim Sort", "Tim Sort", "Zig Zag", 312, 4318, 312, 2585, 2585],
    ["Heap Sort", "Heap Sort", "Zig Zag", 312, 2646, 2585, 2585, 2585],
    ["Merge Sort", "Merge Sort", "Sawtooth", 280, 4576, 2276, 2276, 2276],
    ["Tim Sort", "Tim Sort", "Sawtooth", 280, 3474, 280, 2276, 2276],
    ["Heap Sort", "Heap Sort", "Sawtooth", 280, 2304, 2276, 2276, 2276],
    ["Merge Sort", "Merge Sort", "Square", 274, 4456, 2219, 2219, 2219],
    ["Tim Sort", "Tim Sort", "Square", 274, 2281, 274, 2219, 2219],
    ["Heap Sort", "Heap Sort", "Square", 274, 1243, 2219, 2219, 2219],
    ["Merge Sort", "Merge Sort", "Zeroes", 237, 3754, 1870, 1870, 1870],
    ["Tim Sort", "Tim Sort", "Zeroes", 237, 239, 237, 1870, 1870],
    ["Heap Sort", "Heap Sort", "Zeroes", 237, 474, 1870, 1870, 1870],
    ["Merge Sort", "Merge Sort", "Thousands", 275, 4476, 2228, 2228, 2228],
    ["Tim Sort", "Tim Sort", "Thousands", 275, 3579, 275, 2228, 2228],
    ["Heap Sort", "Heap Sort", "Thousands", 275, 2263, 2228, 2228, 2228],
    ["Merge Sort", "Merge Sort", "Millions", 322, 5416, 2683, 2683, 2683],
    ["Tim Sort", "Tim Sort", "Millions", 322, 4500, 322, 2683, 2683],
    ["Heap Sort", "Heap Sort", "Millions", 322, 2730, 2683, 2683, 2683],
    ["Merge Sort", "Merge Sort", "Blender", 353, 6036, 2988, 2988, 2988],
    ["Tim Sort", "Tim Sort", "Blender", 353, 4656, 353, 2988, 2988],
    ["Heap Sort", "Heap Sort", "Blender", 353, 2808, 2988, 2988, 2988],
    ["Merge Sort", "Merge Sort", "Random", 463, 8236, 4100, 4100, 4100],
    ["Tim Sort", "Tim Sort", "Random", 463, 7847, 463, 4100, 4100],
    ["Heap Sort", "Heap Sort", "Random", 463, 4137, 4100, 4100, 4100],
    ["Merge Sort", "Merge Sort", "Sorted", 415, 7276, 3609, 3609, 3609],
    ["Tim Sort", "Tim Sort", "Sorted", 415, 1367, 415, 3609, 3609],
    ["Heap Sort", "Heap Sort", "Sorted", 415, 3944, 3609, 3609, 3609],
    ["Merge Sort", "Merge Sort", "Half Sorted", 332, 5616, 2781, 2781, 2781],
    ["Tim Sort", "Tim Sort", "Half Sorted", 332, 3276, 332, 2781, 2781],
    ["Heap Sort", "Heap Sort", "Half Sorted", 332, 2895, 2781, 2781, 2781],
    ["Merge Sort", "Merge Sort", "Sorted Half", 316, 5296, 2624, 2624, 2624],
    ["Tim Sort", "Tim Sort", "Sorted Half", 316, 3141, 316, 2624, 2624],
    ["Heap Sort", "Heap Sort", "Sorted Half", 316, 2590, 2624, 2624, 2624],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 312, 5216, 2585, 2585, 2585],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 312, 3700, 312, 2585, 2585],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 312, 2684, 2585, 2585, 2585],
    ["Merge Sort", "Merge Sort", "Reversed", 492, 8816, 4400, 4400, 4400],
    ["Tim Sort", "Tim Sort", "Reversed", 492, 1343, 492, 4400, 4400],
    ["Heap Sort", "Heap Sort", "Reversed", 492, 4106, 4400, 4400, 4400],
    ["Merge Sort", "Merge Sort", "Same", 366, 6296, 3117, 3117, 3117],
    ["Tim Sort", "Tim Sort", "Same", 366, 369, 366, 3117, 3117],
    ["Heap Sort", "Heap Sort", "Same", 366, 733, 3117, 3117, 3117],
    ["Merge Sort", "Merge Sort", "Zig Zag", 317, 5316, 2634, 2634, 2634],
    ["Tim Sort", "Tim Sort", "Zig Zag", 317, 4460, 317, 2634, 2634],
    ["Heap Sort", "Heap Sort", "Zig Zag", 317, 2691, 2634, 2634, 2634],
    ["Merge Sort", "Merge Sort", "Sawtooth", 374, 6456, 3197, 3197, 3197],
    ["Tim Sort", "Tim Sort", "Sawtooth", 374, 5313, 374, 3197, 3197],
    ["Heap Sort", "Heap Sort", "Sawtooth", 374, 3260, 3197, 3197, 3197],
    ["Merge Sort", "Merge Sort", "Square", 335, 5676, 2810, 2810, 2810],
    ["Tim Sort", "Tim Sort", "Square", 335, 2949, 335, 2810, 2810],
    ["Heap Sort", "Heap Sort", "Square", 335, 1575, 2810, 2810, 2810],
    ["Merge Sort", "Merge Sort", "Zeroes", 488, 8736, 4358, 4358, 4358],
    ["Tim Sort", "Tim Sort", "Zeroes", 488, 491, 488, 4358, 4358],
    ["Heap Sort", "Heap Sort", "Zeroes", 488, 977, 4358, 4358, 4358],
    ["Merge Sort", "Merge Sort", "Thousands", 382, 6616, 3277, 3277, 3277],
    ["Tim Sort", "Tim Sort", "Thousands", 382, 5748, 382, 3277, 3277],
    ["Heap Sort", "Heap Sort", "Thousands", 382, 3340, 3277, 3277, 3277],
    ["Merge Sort", "Merge Sort", "Millions", 341, 5796, 2869, 2869, 2869],
    ["Tim Sort", "Tim Sort", "Millions", 341, 4955, 341, 2869, 2869],
    ["Heap Sort", "Heap Sort", "Millions", 341, 2926, 2869, 2869, 2869],
    ["Merge Sort", "Merge Sort", "Blender", 381, 6596, 3267, 3267, 3267],
    ["Tim Sort", "Tim Sort", "Blender", 381, 4928, 381, 3267, 3267],
    ["Heap Sort", "Heap Sort", "Blender", 381, 3050, 3267, 3267, 3267],
    ["Merge Sort", "Merge Sort", "Random", 566, 10404, 5176, 5176, 5176],
    ["Tim Sort", "Tim Sort", "Random", 566, 8442, 566, 5176, 5176],
    ["Heap Sort", "Heap Sort", "Random", 566, 5248, 5176, 5176, 5176],
    ["Merge Sort", "Merge Sort", "Sorted", 504, 9056, 4525, 4525, 4525],
    ["Tim Sort", "Tim Sort", "Sorted", 504, 1650, 504, 4525, 4525],
    ["Heap Sort", "Heap Sort", "Sorted", 504, 4894, 4525, 4525, 4525],
    ["Merge Sort", "Merge Sort", "Half Sorted", 574, 10580, 5261, 5261, 5261],
    ["Tim Sort", "Tim Sort", "Half Sorted", 574, 5626, 574, 5261, 5261],
    ["Heap Sort", "Heap Sort", "Half Sorted", 574, 5423, 5261, 5261, 5261],
    ["Merge Sort", "Merge Sort", "Sorted Half", 588, 10888, 5409, 5409, 5409],
    ["Tim Sort", "Tim Sort", "Sorted Half", 588, 5679, 588, 5409, 5409],
    ["Heap Sort", "Heap Sort", "Sorted Half", 588, 5311, 5409, 5409, 5409],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 563, 10338, 5144, 5144, 5144],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 563, 7660, 563, 5144, 5144],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 563, 5410, 5144, 5144, 5144],
    ["Merge Sort", "Merge Sort", "Reversed", 559, 10250, 5102, 5102, 5102],
    ["Tim Sort", "Tim Sort", "Reversed", 559, 1525, 559, 5102, 5102],
    ["Heap Sort", "Heap Sort", "Reversed", 559, 4791, 5102, 5102, 5102],
    ["Merge Sort", "Merge Sort", "Same", 645, 12142, 6020, 6020, 6020],
    ["Tim Sort", "Tim Sort", "Same", 645, 649, 645, 6020, 6020],
    ["Heap Sort", "Heap Sort", "Same", 645, 1290, 6020, 6020, 6020],
    ["Merge Sort", "Merge Sort", "Zig Zag", 593, 10998, 5463, 5463, 5463],
    ["Tim Sort", "Tim Sort", "Zig Zag", 593, 8566, 593, 5463, 5463],
    ["Heap Sort", "Heap Sort", "Zig Zag", 593, 5582, 5463, 5463, 5463],
    ["Merge Sort", "Merge Sort", "Sawtooth", 552, 10096, 5028, 5028, 5028],
    ["Tim Sort", "Tim Sort", "Sawtooth", 552, 7231, 552, 5028, 5028],
    ["Heap Sort", "Heap Sort", "Sawtooth", 552, 5071, 5028, 5028, 5028],
    ["Merge Sort", "Merge Sort", "Square", 503, 9036, 4514, 4514, 4514],
    ["Tim Sort", "Tim Sort", "Square", 503, 5557, 503, 4514, 4514],
    ["Heap Sort", "Heap Sort", "Square", 503, 2518, 4514, 4514, 4514],
    ["Merge Sort", "Merge Sort", "Zeroes", 548, 10008, 4986, 4986, 4986],
    ["Tim Sort", "Tim Sort", "Zeroes", 548, 552, 548, 4986, 4986],
    ["Heap Sort", "Heap Sort", "Zeroes", 548, 1097, 4986, 4986, 4986],
    ["Merge Sort", "Merge Sort", "Thousands", 598, 11108, 5516, 5516, 5516],
    ["Tim Sort", "Tim Sort", "Thousands", 598, 9066, 598, 5516, 5516],
    ["Heap Sort", "Heap Sort", "Thousands", 598, 5585, 5516, 5516, 5516],
    ["Merge Sort", "Merge Sort", "Millions", 509, 9156, 4577, 4577, 4577],
    ["Tim Sort", "Tim Sort", "Millions", 509, 9048, 509, 4577, 4577],
    ["Heap Sort", "Heap Sort", "Millions", 509, 4647, 4577, 4577, 4577],
    ["Merge Sort", "Merge Sort", "Blender", 652, 12296, 6095, 6095, 6095],
    ["Tim Sort", "Tim Sort", "Blender", 652, 9004, 652, 6095, 6095],
    ["Heap Sort", "Heap Sort", "Blender", 652, 5990, 6095, 6095, 6095],
    ["Merge Sort", "Merge Sort", "Random", 765, 14782, 7328, 7328, 7328],
    ["Tim Sort", "Tim Sort", "Random", 765, 13066, 765, 7328, 7328],
    ["Heap Sort", "Heap Sort", "Random", 765, 7489, 7328, 7328, 7328],
    ["Merge Sort", "Merge Sort", "Sorted", 787, 15266, 7571, 7571, 7571],
    ["Tim Sort", "Tim Sort", "Sorted", 787, 2286, 787, 7571, 7571],
    ["Heap Sort", "Heap Sort", "Sorted", 787, 8218, 7571, 7571, 7571],
    ["Merge Sort", "Merge Sort", "Half Sorted", 814, 15860, 7870, 7870, 7870],
    ["Tim Sort", "Tim Sort", "Half Sorted", 814, 8985, 814, 7870, 7870],
    ["Heap Sort", "Heap Sort", "Half Sorted", 814, 8004, 7870, 7870, 7870],
    ["Merge Sort", "Merge Sort", "Sorted Half", 854, 16740, 8316, 8316, 8316],
    ["Tim Sort", "Tim Sort", "Sorted Half", 854, 9703, 854, 8316, 8316],
    ["Heap Sort", "Heap Sort", "Sorted Half", 854, 8171, 8316, 8316, 8316],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 615, 11482, 5698, 5698, 5698],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 615, 8371, 615, 5698, 5698],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 615, 5822, 5698, 5698, 5698],
    ["Merge Sort", "Merge Sort", "Reversed", 641, 12054, 5977, 5977, 5977],
    ["Tim Sort", "Tim Sort", "Reversed", 641, 1750, 641, 5977, 5977],
    ["Heap Sort", "Heap Sort", "Reversed", 641, 5608, 5977, 5977, 5977],
    ["Merge Sort", "Merge Sort", "Same", 797, 15486, 7682, 7682, 7682],
    ["Tim Sort", "Tim Sort", "Same", 797, 801, 797, 7682, 7682],
    ["Heap Sort", "Heap Sort", "Same", 797, 1594, 7682, 7682, 7682],
    ["Merge Sort", "Merge Sort", "Zig Zag", 764, 14760, 7317, 7317, 7317],
    ["Tim Sort", "Tim Sort", "Zig Zag", 764, 12411, 764, 7317, 7317],
    ["Heap Sort", "Heap Sort", "Zig Zag", 764, 7481, 7317, 7317, 7317],
    ["Merge Sort", "Merge Sort", "Sawtooth", 925, 18302, 9114, 9114, 9114],
    ["Tim Sort", "Tim Sort", "Sawtooth", 925, 15333, 925, 9114, 9114],
    ["Heap Sort", "Heap Sort", "Sawtooth", 925, 9161, 9114, 9114, 9114],
    ["Merge Sort", "Merge Sort", "Square", 737, 14166, 7020, 7020, 7020],
    ["Tim Sort", "Tim Sort", "Square", 737, 3982, 737, 7020, 7020],
    ["Heap Sort", "Heap Sort", "Square", 737, 3860, 7020, 7020, 7020],
    ["Merge Sort", "Merge Sort", "Zeroes", 620, 11592, 5751, 5751, 5751],
    ["Tim Sort", "Tim Sort", "Zeroes", 620, 624, 620, 5751, 5751],
    ["Heap Sort", "Heap Sort", "Zeroes", 620, 1241, 5751, 5751, 5751],
    ["Merge Sort", "Merge Sort", "Thousands", 857, 16806, 8350, 8350, 8350],
    ["Tim Sort", "Tim Sort", "Thousands", 857, 15505, 857, 8350, 8350],
    ["Heap Sort", "Heap Sort", "Thousands", 857, 8490, 8350, 8350, 8350],
    ["Merge Sort", "Merge Sort", "Millions", 826, 16124, 8004, 8004, 8004],
    ["Tim Sort", "Tim Sort", "Millions", 826, 14375, 826, 8004, 8004],
    ["Heap Sort", "Heap Sort", "Millions", 826, 8142, 8004, 8004, 8004],
    ["Merge Sort", "Merge Sort", "Blender", 694, 13220, 6551, 6551, 6551],
    ["Tim Sort", "Tim Sort", "Blender", 694, 10390, 694, 6551, 6551],
    ["Heap Sort", "Heap Sort", "Blender", 694, 6155, 6551, 6551, 6551],
    ["Merge Sort", "Merge Sort", "Random", 1231, 25448, 12637, 12637, 12637],
    ["Tim Sort", "Tim Sort", "Random", 1231, 20847, 1231, 12637, 12637],
    ["Heap Sort", "Heap Sort", "Random", 1231, 12803, 12637, 12637, 12637],
    ["Merge Sort", "Merge Sort", "Sorted", 1111, 22568, 11241, 11241, 11241],
    ["Tim Sort", "Tim Sort", "Sorted", 1111, 4128, 1111, 11241, 11241],
    ["Heap Sort", "Heap Sort", "Sorted", 1111, 12096, 11241, 11241, 11241],
    ["Merge Sort", "Merge Sort", "Half Sorted", 822, 16036, 7959, 7959, 7959],
    ["Tim Sort", "Tim Sort", "Half Sorted", 822, 9173, 822, 7959, 7959],
    ["Heap Sort", "Heap Sort", "Half Sorted", 822, 8317, 7959, 7959, 7959],
    ["Merge Sort", "Merge Sort", "Sorted Half", 982, 19556, 9761, 9761, 9761],
    ["Tim Sort", "Tim Sort", "Sorted Half", 982, 11426, 982, 9761, 9761],
    ["Heap Sort", "Heap Sort", "Sorted Half", 982, 9537, 9761, 9761, 9761],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 856, 16784, 8339, 8339, 8339],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 856, 13623, 856, 8339, 8339],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 856, 8717, 8339, 8339, 8339],
    ["Merge Sort", "Merge Sort", "Reversed", 1088, 22016, 10975, 10975, 10975],
    ["Tim Sort", "Tim Sort", "Reversed", 1088, 3066, 1088, 10975, 10975],
    ["Heap Sort", "Heap Sort", "Reversed", 1088, 10282, 10975, 10975, 10975],
    ["Merge Sort", "Merge Sort", "Same", 826, 16124, 8004, 8004, 8004],
    ["Tim Sort", "Tim Sort", "Same", 826, 830, 826, 8004, 8004],
    ["Heap Sort", "Heap Sort", "Same", 826, 1653, 8004, 8004, 8004],
    ["Merge Sort", "Merge Sort", "Zig Zag", 1026, 20528, 10263, 10263, 10263],
    ["Tim Sort", "Tim Sort", "Zig Zag", 1026, 15503, 1026, 10263, 10263],
    ["Heap Sort", "Heap Sort", "Zig Zag", 1026, 10456, 10263, 10263, 10263],
    ["Merge Sort", "Merge Sort", "Sawtooth", 849, 16630, 8260, 8260, 8260],
    ["Tim Sort", "Tim Sort", "Sawtooth", 849, 13495, 849, 8260, 8260],
    ["Heap Sort", "Heap Sort", "Sawtooth", 849, 8306, 8260, 8260, 8260],
    ["Merge Sort", "Merge Sort", "Square", 1187, 24392, 12123, 12123, 12123],
    ["Tim Sort", "Tim Sort", "Square", 1187, 7015, 1187, 12123, 12123],
    ["Heap Sort", "Heap Sort", "Square", 1187, 6463, 12123, 12123, 12123],
    ["Merge Sort", "Merge Sort", "Zeroes", 985, 19622, 9795, 9795, 9795],
    ["Tim Sort", "Tim Sort", "Zeroes", 985, 989, 985, 9795, 9795],
    ["Heap Sort", "Heap Sort", "Zeroes", 985, 1970, 9795, 9795, 9795],
    ["Merge Sort", "Merge Sort", "Thousands", 901, 17774, 8844, 8844, 8844],
    ["Tim Sort", "Tim Sort", "Thousands", 901, 16488, 901, 8844, 8844],
    ["Heap Sort", "Heap Sort", "Thousands", 901, 8967, 8844, 8844, 8844],
    ["Merge Sort", "Merge Sort", "Millions", 901, 17774, 8844, 8844, 8844],
    ["Tim Sort", "Tim Sort", "Millions", 901, 16695, 901, 8844, 8844],
    ["Heap Sort", "Heap Sort", "Millions", 901, 8977, 8844, 8844, 8844],
    ["Merge Sort", "Merge Sort", "Blender", 815, 15882, 7882, 7882, 7882],
    ["Tim Sort", "Tim Sort", "Blender", 815, 11300, 815, 7882, 7882],
    ["Heap Sort", "Heap Sort", "Blender", 815, 7298, 7882, 7882, 7882],
    ["Merge Sort", "Merge Sort", "Random", 1218, 25136, 12485, 12485, 12485],
    ["Tim Sort", "Tim Sort", "Random", 1218, 20885, 1218, 12485, 12485],
    ["Heap Sort", "Heap Sort", "Random", 1218, 12640, 12485, 12485, 12485],
    ["Merge Sort", "Merge Sort", "Sorted", 1178, 24176, 12018, 12018, 12018],
    ["Tim Sort", "Tim Sort", "Sorted", 1178, 4369, 1178, 12018, 12018],
    ["Heap Sort", "Heap Sort", "Sorted", 1178, 12952, 12018, 12018, 12018],
    ["Merge Sort", "Merge Sort", "Half Sorted", 1768, 38336, 19073, 19073, 19073],
    ["Tim Sort", "Tim Sort", "Half Sorted", 1768, 21117, 1768, 19073, 19073],
    ["Heap Sort", "Heap Sort", "Half Sorted", 1768, 19837, 19073, 19073, 19073],
    ["Merge Sort", "Merge Sort", "Sorted Half", 1374, 28880, 14323, 14323, 14323],
    ["Tim Sort", "Tim Sort", "Sorted Half", 1374, 15251, 1374, 14323, 14323],
    ["Heap Sort", "Heap Sort", "Sorted Half", 1374, 14093, 14323, 14323, 14323],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 1832, 39872, 19857, 19857, 19857],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 1832, 33168, 1832, 19857, 19857],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 1832, 20527, 19857, 19857, 19857],
    ["Merge Sort", "Merge Sort", "Reversed", 1179, 24200, 12030, 12030, 12030],
    ["Tim Sort", "Tim Sort", "Reversed", 1179, 3413, 1179, 12030, 12030],
    ["Heap Sort", "Heap Sort", "Reversed", 1179, 11327, 12030, 12030, 12030],
    ["Merge Sort", "Merge Sort", "Same", 1167, 23912, 11890, 11890, 11890],
    ["Tim Sort", "Tim Sort", "Same", 1167, 1172, 1167, 11890, 11890],
    ["Heap Sort", "Heap Sort", "Same", 1167, 2334, 11890, 11890, 11890],
    ["Merge Sort", "Merge Sort", "Zig Zag", 1657, 35672, 17721, 17721, 17721],
    ["Tim Sort", "Tim Sort", "Zig Zag", 1657, 30888, 1657, 17721, 17721],
    ["Heap Sort", "Heap Sort", "Zig Zag", 1657, 18138, 17721, 17721, 17721],
    ["Merge Sort", "Merge Sort", "Sawtooth", 1368, 28736, 14252, 14252, 14252],
    ["Tim Sort", "Tim Sort", "Sawtooth", 1368, 19121, 1368, 14252, 14252],
    ["Heap Sort", "Heap Sort", "Sawtooth", 1368, 14432, 14252, 14252, 14252],
    ["Merge Sort", "Merge Sort", "Square", 1432, 30272, 15013, 15013, 15013],
    ["Tim Sort", "Tim Sort", "Square", 1432, 8045, 1432, 15013, 15013],
    ["Heap Sort", "Heap Sort", "Square", 1432, 7998, 15013, 15013, 15013],
    ["Merge Sort", "Merge Sort", "Zeroes", 1350, 28304, 14038, 14038, 14038],
    ["Tim Sort", "Tim Sort", "Zeroes", 1350, 1355, 1350, 14038, 14038],
    ["Heap Sort", "Heap Sort", "Zeroes", 1350, 2701, 14038, 14038, 14038],
    ["Merge Sort", "Merge Sort", "Thousands", 1228, 25376, 12602, 12602, 12602],
    ["Tim Sort", "Tim Sort", "Thousands", 1228, 20749, 1228, 12602, 12602],
    ["Heap Sort", "Heap Sort", "Thousands", 1228, 12734, 12602, 12602, 12602],
    ["Merge Sort", "Merge Sort", "Millions", 1234, 25520, 12672, 12672, 12672],
    ["Tim Sort", "Tim Sort", "Millions", 1234, 21478, 1234, 12672, 12672],
    ["Heap Sort", "Heap Sort", "Millions", 1234, 12835, 12672, 12672, 12672],
    ["Merge Sort", "Merge Sort", "Blender", 1265, 26264, 13036, 13036, 13036],
    ["Tim Sort", "Tim Sort", "Blender", 1265, 16920, 1265, 13036, 13036],
    ["Heap Sort", "Heap Sort", "Blender", 1265, 12020, 13036, 13036, 13036],
    ["Merge Sort", "Merge Sort", "Random", 1598, 34256, 17006, 17006, 17006],
    ["Tim Sort", "Tim Sort", "Random", 1598, 30016, 1598, 17006, 17006],
    ["Heap Sort", "Heap Sort", "Random", 1598, 17221, 17006, 17006, 17006],
    ["Merge Sort", "Merge Sort", "Sorted", 1727, 37352, 18572, 18572, 18572],
    ["Tim Sort", "Tim Sort", "Sorted", 1727, 6314, 1727, 18572, 18572],
    ["Heap Sort", "Heap Sort", "Sorted", 1727, 19952, 18572, 18572, 18572],
    ["Merge Sort", "Merge Sort", "Half Sorted", 1614, 34640, 17199, 17199, 17199],
    ["Tim Sort", "Tim Sort", "Half Sorted", 1614, 19081, 1614, 17199, 17199],
    ["Heap Sort", "Heap Sort", "Half Sorted", 1614, 17733, 17199, 17199, 17199],
    ["Merge Sort", "Merge Sort", "Sorted Half", 2206, 49164, 24503, 24503, 24503],
    ["Tim Sort", "Tim Sort", "Sorted Half", 2206, 24329, 2206, 24503, 24503],
    ["Heap Sort", "Heap Sort", "Sorted Half", 2206, 24084, 24503, 24503, 24503],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 2151, 47734, 23813, 23813, 23813],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 2151, 35001, 2151, 23813, 23813],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 2151, 24666, 23813, 23813, 23813],
    ["Merge Sort", "Merge Sort", "Reversed", 1927, 42152, 21028, 21028, 21028],
    ["Tim Sort", "Tim Sort", "Reversed", 1927, 7156, 1927, 21028, 21028],
    ["Heap Sort", "Heap Sort", "Reversed", 1927, 19777, 21028, 21028, 21028],
    ["Merge Sort", "Merge Sort", "Same", 1982, 43472, 21708, 21708, 21708],
    ["Tim Sort", "Tim Sort", "Same", 1982, 1987, 1982, 21708, 21708],
    ["Heap Sort", "Heap Sort", "Same", 1982, 3965, 21708, 21708, 21708],
    ["Merge Sort", "Merge Sort", "Zig Zag", 1683, 36296, 18036, 18036, 18036],
    ["Tim Sort", "Tim Sort", "Zig Zag", 1683, 31546, 1683, 18036, 18036],
    ["Heap Sort", "Heap Sort", "Zig Zag", 1683, 18445, 18036, 18036, 18036],
    ["Merge Sort", "Merge Sort", "Sawtooth", 2351, 52934, 26329, 26329, 26329],
    ["Tim Sort", "Tim Sort", "Sawtooth", 2351, 33014, 2351, 26329, 26329],
    ["Heap Sort", "Heap Sort", "Sawtooth", 2351, 26342, 26329, 26329, 26329],
    ["Merge Sort", "Merge Sort", "Square", 1596, 34208, 16982, 16982, 16982],
    ["Tim Sort", "Tim Sort", "Square", 1596, 9150, 1596, 16982, 16982],
    ["Heap Sort", "Heap Sort", "Square", 1596, 8879, 16982, 16982, 16982],
    ["Merge Sort", "Merge Sort", "Zeroes", 1749, 37880, 18841, 18841, 18841],
    ["Tim Sort", "Tim Sort", "Zeroes", 1749, 1754, 1749, 18841, 18841],
    ["Heap Sort", "Heap Sort", "Zeroes", 1749, 3498, 18841, 18841, 18841],
    ["Merge Sort", "Merge Sort", "Thousands", 2486, 56444, 28041, 28041, 28041],
    ["Tim Sort", "Tim Sort", "Thousands", 2486, 45740, 2486, 28041, 28041],
    ["Heap Sort", "Heap Sort", "Thousands", 2486, 28394, 28041, 28041, 28041],
    ["Merge Sort", "Merge Sort", "Millions", 2497, 56730, 28181, 28181, 28181],
    ["Tim Sort", "Tim Sort", "Millions", 2497, 46021, 2497, 28181, 28181],
    ["Heap Sort", "Heap Sort", "Millions", 2497, 28540, 28181, 28181, 28181],
    ["Merge Sort", "Merge Sort", "Blender", 1845, 40184, 20017, 20017, 20017],
    ["Tim Sort", "Tim Sort", "Blender", 1845, 29982, 1845, 20017, 20017],
    ["Heap Sort", "Heap Sort", "Blender", 1845, 19076, 20017, 20017, 20017],
    ["Merge Sort", "Merge Sort", "Random", 3671, 87254, 43472, 43472, 43472],
    ["Tim Sort", "Tim Sort", "Random", 3671, 78326, 3671, 43472, 43472],
    ["Heap Sort", "Heap Sort", "Random", 3671, 43887, 43472, 43472, 43472],
    ["Merge Sort", "Merge Sort", "Sorted", 2522, 57380, 28499, 28499, 28499],
    ["Tim Sort", "Tim Sort", "Sorted", 2522, 8558, 2522, 28499, 28499],
    ["Heap Sort", "Heap Sort", "Sorted", 2522, 30616, 28499, 28499, 28499],
    ["Merge Sort", "Merge Sort", "Half Sorted", 2734, 62892, 31214, 31214, 31214],
    ["Tim Sort", "Tim Sort", "Half Sorted", 2734, 33638, 2734, 31214, 31214],
    ["Heap Sort", "Heap Sort", "Half Sorted", 2734, 31767, 31214, 31214, 31214],
    ["Merge Sort", "Merge Sort", "Sorted Half", 3068, 71576, 35537, 35537, 35537],
    ["Tim Sort", "Tim Sort", "Sorted Half", 3068, 38326, 3068, 35537, 35537],
    ["Heap Sort", "Heap Sort", "Sorted Half", 3068, 34931, 35537, 35537, 35537],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 2841, 65674, 32592, 32592, 32592],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 2841, 50778, 2841, 32592, 32592],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 2841, 33682, 32592, 32592, 32592],
    ["Merge Sort", "Merge Sort", "Reversed", 3284, 77192, 38361, 38361, 38361],
    ["Tim Sort", "Tim Sort", "Reversed", 3284, 8957, 3284, 38361, 38361],
    ["Heap Sort", "Heap Sort", "Reversed", 3284, 36326, 38361, 38361, 38361],
    ["Merge Sort", "Merge Sort", "Same", 2735, 62918, 31226, 31226, 31226],
    ["Tim Sort", "Tim Sort", "Same", 2735, 2741, 2735, 31226, 31226],
    ["Heap Sort", "Heap Sort", "Same", 2735, 5470, 31226, 31226, 31226],
    ["Merge Sort", "Merge Sort", "Zig Zag", 2437, 55170, 27418, 27418, 27418],
    ["Tim Sort", "Tim Sort", "Zig Zag", 2437, 43310, 2437, 27418, 27418],
    ["Heap Sort", "Heap Sort", "Zig Zag", 2437, 27991, 27418, 27418, 27418],
    ["Merge Sort", "Merge Sort", "Sawtooth", 3728, 88736, 44230, 44230, 44230],
    ["Tim Sort", "Tim Sort", "Sawtooth", 3728, 54832, 3728, 44230, 44230],
    ["Heap Sort", "Heap Sort", "Sawtooth", 3728, 44184, 44230, 44230, 44230],
    ["Merge Sort", "Merge Sort", "Square", 2600, 59408, 29495, 29495, 29495],
    ["Tim Sort", "Tim Sort", "Square", 2600, 15049, 2600, 29495, 29495],
    ["Heap Sort", "Heap Sort", "Square", 2600, 15270, 29495, 29495, 29495],
    ["Merge Sort", "Merge Sort", "Zeroes", 3418, 80676, 40124, 40124, 40124],
    ["Tim Sort", "Tim Sort", "Zeroes", 3418, 3424, 3418, 40124, 40124],
    ["Heap Sort", "Heap Sort", "Zeroes", 3418, 6837, 40124, 40124, 40124],
    ["Merge Sort", "Merge Sort", "Thousands", 3746, 89204, 44469, 44469, 44469],
    ["Tim Sort", "Tim Sort", "Thousands", 3746, 81675, 3746, 44469, 44469],
    ["Heap Sort", "Heap Sort", "Thousands", 3746, 44915, 44469, 44469, 44469],
    ["Merge Sort", "Merge Sort", "Millions", 2771, 63854, 31690, 31690, 31690],
    ["Tim Sort", "Tim Sort", "Millions", 2771, 52968, 2771, 31690, 31690],
    ["Heap Sort", "Heap Sort", "Millions", 2771, 32067, 31690, 31690, 31690],
    ["Merge Sort", "Merge Sort", "Blender", 3339, 78622, 39084, 39084, 39084],
    ["Tim Sort", "Tim Sort", "Blender", 3339, 53078, 3339, 39084, 39084],
    ["Heap Sort", "Heap Sort", "Blender", 3339, 36331, 39084, 39084, 39084],
    ["Merge Sort", "Merge Sort", "Random", 3696, 87904, 43804, 43804, 43804],
    ["Tim Sort", "Tim Sort", "Random", 3696, 79619, 3696, 43804, 43804],
    ["Heap Sort", "Heap Sort", "Random", 3696, 44300, 43804, 43804, 43804],
    ["Merge Sort", "Merge Sort", "Sorted", 4262, 102952, 51388, 51388, 51388],
    ["Tim Sort", "Tim Sort", "Sorted", 4262, 12881, 4262, 51388, 51388],
    ["Heap Sort", "Heap Sort", "Sorted", 4262, 54936, 51388, 51388, 51388],
    ["Merge Sort", "Merge Sort", "Half Sorted", 4790, 117736, 58562, 58562, 58562],
    ["Tim Sort", "Tim Sort", "Half Sorted", 4790, 58810, 4790, 58562, 58562],
    ["Heap Sort", "Heap Sort", "Half Sorted", 4790, 59899, 58562, 58562, 58562],
    ["Merge Sort", "Merge Sort", "Sorted Half", 3948, 94456, 47166, 47166, 47166],
    ["Tim Sort", "Tim Sort", "Sorted Half", 3948, 53096, 3948, 47166, 47166],
    ["Heap Sort", "Heap Sort", "Sorted Half", 3948, 46058, 47166, 47166, 47166],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 3073, 71706, 35602, 35602, 35602],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 3073, 56744, 3073, 35602, 35602],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 3073, 36328, 35602, 35602, 35602],
    ["Merge Sort", "Merge Sort", "Reversed", 4052, 97160, 48561, 48561, 48561],
    ["Tim Sort", "Tim Sort", "Reversed", 4052, 11049, 4052, 48561, 48561],
    ["Heap Sort", "Heap Sort", "Reversed", 4052, 45986, 48561, 48561, 48561],
    ["Merge Sort", "Merge Sort", "Same", 4521, 110204, 54896, 54896, 54896],
    ["Tim Sort", "Tim Sort", "Same", 4521, 4528, 4521, 54896, 54896],
    ["Heap Sort", "Heap Sort", "Same", 4521, 9042, 54896, 54896, 54896],
    ["Merge Sort", "Merge Sort", "Zig Zag", 4111, 98724, 49354, 49354, 49354],
    ["Tim Sort", "Tim Sort", "Zig Zag", 4111, 73317, 4111, 49354, 49354],
    ["Heap Sort", "Heap Sort", "Zig Zag", 4111, 50202, 49354, 49354, 49354],
    ["Merge Sort", "Merge Sort", "Sawtooth", 4239, 102308, 51078, 51078, 51078],
    ["Tim Sort", "Tim Sort", "Sawtooth", 4239, 53321, 4239, 51078, 51078],
    ["Heap Sort", "Heap Sort", "Sawtooth", 4239, 50944, 51078, 51078, 51078],
    ["Merge Sort", "Merge Sort", "Square", 3864, 92272, 46043, 46043, 46043],
    ["Tim Sort", "Tim Sort", "Square", 3864, 22928, 3864, 46043, 46043],
    ["Heap Sort", "Heap Sort", "Square", 3864, 23810, 46043, 46043, 46043],
    ["Merge Sort", "Merge Sort", "Zeroes", 3590, 85148, 42397, 42397, 42397],
    ["Tim Sort", "Tim Sort", "Zeroes", 3590, 3596, 3590, 42397, 42397],
    ["Heap Sort", "Heap Sort", "Zeroes", 3590, 7181, 42397, 42397, 42397],
    ["Merge Sort", "Merge Sort", "Thousands", 3953, 94586, 47233, 47233, 47233],
    ["Tim Sort", "Tim Sort", "Thousands", 3953, 86997, 3953, 47233, 47233],
    ["Heap Sort", "Heap Sort", "Thousands", 3953, 47634, 47233, 47233, 47233],
    ["Merge Sort", "Merge Sort", "Millions", 3671, 87254, 43472, 43472, 43472],
    ["Tim Sort", "Tim Sort", "Millions", 3671, 78567, 3671, 43472, 43472],
    ["Heap Sort", "Heap Sort", "Millions", 3671, 43906, 43472, 43472, 43472],
    ["Merge Sort", "Merge Sort", "Blender", 3889, 92922, 46377, 46377, 46377],
    ["Tim Sort", "Tim Sort", "Blender", 3889, 70342, 3889, 46377, 46377],
    ["Heap Sort", "Heap Sort", "Blender", 3889, 43069, 46377, 46377, 46377],
    ["Merge Sort", "Merge Sort", "Random", 6326, 160744, 79879, 79879, 79879],
    ["Tim Sort", "Tim Sort", "Random", 6326, 137181, 6326, 79879, 79879],
    ["Heap Sort", "Heap Sort", "Random", 6326, 80794, 79879, 79879, 79879],
    ["Merge Sort", "Merge Sort", "Sorted", 4809, 118268, 58821, 58821, 58821],
    ["Tim Sort", "Tim Sort", "Sorted", 4809, 14273, 4809, 58821, 58821],
    ["Heap Sort", "Heap Sort", "Sorted", 4809, 63076, 58821, 58821, 58821],
    ["Merge Sort", "Merge Sort", "Half Sorted", 7254, 186728, 93029, 93029, 93029],
    ["Tim Sort", "Tim Sort", "Half Sorted", 7254, 99747, 7254, 93029, 93029],
    ["Heap Sort", "Heap Sort", "Half Sorted", 7254, 93838, 93029, 93029, 93029],
    ["Merge Sort", "Merge Sort", "Sorted Half", 6258, 158840, 78923, 78923, 78923],
    ["Tim Sort", "Tim Sort", "Sorted Half", 6258, 80704, 6258, 78923, 78923],
    ["Heap Sort", "Heap Sort", "Sorted Half", 6258, 77858, 78923, 78923, 78923],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 4994, 123448, 61356, 61356, 61356],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 4994, 92561, 4994, 61356, 61356],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 4994, 63224, 61356, 61356, 61356],
    ["Merge Sort", "Merge Sort", "Reversed", 5804, 146128, 72566, 72566, 72566],
    ["Tim Sort", "Tim Sort", "Reversed", 5804, 15816, 5804, 72566, 72566],
    ["Heap Sort", "Heap Sort", "Reversed", 5804, 68990, 72566, 72566, 72566],
    ["Merge Sort", "Merge Sort", "Same", 7209, 185468, 92388, 92388, 92388],
    ["Tim Sort", "Tim Sort", "Same", 7209, 7216, 7209, 92388, 92388],
    ["Heap Sort", "Heap Sort", "Same", 7209, 14418, 92388, 92388, 92388],
    ["Merge Sort", "Merge Sort", "Zig Zag", 7041, 180764, 89995, 89995, 89995],
    ["Tim Sort", "Tim Sort", "Zig Zag", 7041, 147056, 7041, 89995, 89995],
    ["Heap Sort", "Heap Sort", "Zig Zag", 7041, 91732, 89995, 89995, 89995],
    ["Merge Sort", "Merge Sort", "Sawtooth", 5646, 141704, 70366, 70366, 70366],
    ["Tim Sort", "Tim Sort", "Sawtooth", 5646, 75800, 5646, 70366, 70366],
    ["Heap Sort", "Heap Sort", "Sawtooth", 5646, 70412, 70366, 70366, 70366],
    ["Merge Sort", "Merge Sort", "Square", 7340, 189136, 94257, 94257, 94257],
    ["Tim Sort", "Tim Sort", "Square", 7340, 45484, 7340, 94257, 94257],
    ["Heap Sort", "Heap Sort", "Square", 7340, 48206, 94257, 94257, 94257],
    ["Merge Sort", "Merge Sort", "Zeroes", 5064, 125408, 62318, 62318, 62318],
    ["Tim Sort", "Tim Sort", "Zeroes", 5064, 5071, 5064, 62318, 62318],
    ["Heap Sort", "Heap Sort", "Zeroes", 5064, 10129, 62318, 62318, 62318],
    ["Merge Sort", "Merge Sort", "Thousands", 7049, 180988, 90109, 90109, 90109],
    ["Tim Sort", "Tim Sort", "Thousands", 7049, 158281, 7049, 90109, 90109],
    ["Heap Sort", "Heap Sort", "Thousands", 7049, 90975, 90109, 90109, 90109],
    ["Merge Sort", "Merge Sort", "Millions", 4590, 112136, 55834, 55834, 55834],
    ["Tim Sort", "Tim Sort", "Millions", 4590, 89043, 4590, 55834, 55834],
    ["Heap Sort", "Heap Sort", "Millions", 4590, 56410, 55834, 55834, 55834],
    ["Merge Sort", "Merge Sort", "Blender", 5145, 127676, 63432, 63432, 63432],
    ["Tim Sort", "Tim Sort", "Blender", 5145, 79826, 5145, 63432, 63432],
    ["Heap Sort", "Heap Sort", "Blender", 5145, 58635, 63432, 63432, 63432],
    ["Merge Sort", "Merge Sort", "Random", 6878, 176200, 87679, 87679, 87679],
    ["Tim Sort", "Tim Sort", "Random", 6878, 154257, 6878, 87679, 87679],
    ["Heap Sort", "Heap Sort", "Random", 6878, 88538, 87679, 87679, 87679],
    ["Merge Sort", "Merge Sort", "Sorted", 6616, 168864, 83969, 83969, 83969],
    ["Tim Sort", "Tim Sort", "Sorted", 6616, 18863, 6616, 83969, 83969],
    ["Heap Sort", "Heap Sort", "Sorted", 6616, 90038, 83969, 83969, 83969],
    ["Merge Sort", "Merge Sort", "Half Sorted", 8502, 222292, 110982, 110982, 110982],
    ["Tim Sort", "Tim Sort", "Half Sorted", 8502, 106150, 8502, 110982, 110982],
    ["Heap Sort", "Heap Sort", "Half Sorted", 8502, 113655, 110982, 110982, 110982],
    ["Merge Sort", "Merge Sort", "Sorted Half", 7652, 197872, 98723, 98723, 98723],
    ["Tim Sort", "Tim Sort", "Sorted Half", 7652, 106647, 7652, 98723, 98723],
    ["Heap Sort", "Heap Sort", "Sorted Half", 7652, 96730, 98723, 98723, 98723],
    ["Merge Sort", "Merge Sort", "Semi Sorted", 8593, 225022, 112301, 112301, 112301],
    ["Tim Sort", "Tim Sort", "Semi Sorted", 8593, 169562, 8593, 112301, 112301],
    ["Heap Sort", "Heap Sort", "Semi Sorted", 8593, 115782, 112301, 112301, 112301],
    ["Merge Sort", "Merge Sort", "Reversed", 9149, 241702, 120395, 120395, 120395],
    ["Tim Sort", "Tim Sort", "Reversed", 9149, 24917, 9149, 120395, 120395],
    ["Heap Sort", "Heap Sort", "Reversed", 9149, 114766, 120395, 120395, 120395],
    ["Merge Sort", "Merge Sort", "Same", 6172, 156432, 77715, 77715, 77715],
    ["Tim Sort", "Tim Sort", "Same", 6172, 6179, 6172, 77715, 77715],
    ["Heap Sort", "Heap Sort", "Same", 6172, 12345, 77715, 77715, 77715],
    ["Merge Sort", "Merge Sort", "Zig Zag", 7404, 190928, 95172, 95172, 95172],
    ["Tim Sort", "Tim Sort", "Zig Zag", 7404, 155400, 7404, 95172, 95172],
    ["Heap Sort", "Heap Sort", "Zig Zag", 7404, 96871, 95172, 95172, 95172],
    ["Merge Sort", "Merge Sort", "Sawtooth", 8372, 218392, 109099, 109099, 109099],
    ["Tim Sort", "Tim Sort", "Sawtooth", 8372, 114632, 8372, 109099, 109099],
    ["Heap Sort", "Heap Sort", "Sawtooth", 8372, 108784, 109099, 109099, 109099],
    ["Merge Sort", "Merge Sort", "Square", 9266, 245212, 122105, 122105, 122105],
    ["Tim Sort", "Tim Sort", "Square", 9266, 58884, 9266, 122105, 122105],
    ["Heap Sort", "Heap Sort", "Square", 9266, 61611, 122105, 122105, 122105],
    ["Merge Sort", "Merge Sort", "Zeroes", 6773, 173260, 86190, 86190, 86190],
    ["Tim Sort", "Tim Sort", "Zeroes", 6773, 6780, 6773, 86190, 86190],
    ["Heap Sort", "Heap Sort", "Zeroes", 6773, 13546, 86190, 86190, 86190],
    ["Merge Sort", "Merge Sort", "Thousands", 9993, 267022, 132774, 132774, 132774],
    ["Tim Sort", "Tim Sort", "Thousands", 9993, 214768, 9993, 132774, 132774],
    ["Heap Sort", "Heap Sort", "Thousands", 9993, 133992, 132774, 132774, 132774],
    ["Merge Sort", "Merge Sort", "Millions", 8051, 209044, 104461, 104461, 104461],
    ["Tim Sort", "Tim Sort", "Millions", 8051, 192136, 8051, 104461, 104461],
    ["Heap Sort", "Heap Sort", "Millions", 8051, 105370, 104461, 104461, 104461],
    ["Merge Sort", "Merge Sort", "Blender", 9867, 263242, 130919, 130919, 130919],
    ["Tim Sort", "Tim Sort", "Blender", 9867, 174346, 9867, 130919, 130919],
    ["Heap Sort", "Heap Sort", "Blender", 9867, 122266, 130919, 130919, 130919],
  ],
  "bubble-sort-sorted": [
    ["Sorted", "Bubble Sort", "Sorted", 10, 11, 10, 100, 100],
    ["Sorted", "Bubble Sort", "Sorted", 15, 16, 15, 225, 225],
    ["Sorted", "Bubble Sort", "Sorted", 20, 21, 20, 400, 400],
    ["Sorted", "Bubble Sort", "Sorted", 30, 31, 30, 900, 900],
    ["Sorted", "Bubble Sort", "Sorted", 40, 41, 40, 1600, 1600],
    ["Sorted", "Bubble Sort", "Sorted", 60, 61, 60, 3600, 3600],
    ["Sorted", "Bubble Sort", "Sorted", 80, 81, 80, 6400, 6400],
    ["Sorted", "Bubble Sort", "Sorted", 100, 101, 100, 10000, 10000],
    ["Sorted", "Bubble Sort", "Sorted", 150, 151, 150, 22500, 22500],
    ["Sorted", "Bubble Sort", "Sorted", 200, 201, 200, 40000, 40000],
    ["Sorted", "Bubble Sort", "Sorted", 300, 301, 300, 90000, 90000],
    ["Sorted", "Bubble Sort", "Sorted", 400, 401, 400, 160000, 160000],
    ["Sorted", "Bubble Sort", "Sorted", 600, 601, 600, 360000, 360000],
    ["Sorted", "Bubble Sort", "Sorted", 800, 801, 800, 640000, 640000],
    ["Sorted", "Bubble Sort", "Sorted", 1000, 1001, 1000, 1000000, 1000000],
    ["Sorted", "Bubble Sort", "Sorted", 1500, 1501, 1500, 2250000, 2250000],
    ["Sorted", "Bubble Sort", "Sorted", 2000, 2001, 2000, 4000000, 4000000],
    ["Sorted", "Bubble Sort", "Sorted", 3000, 3001, 3000, 9000000, 9000000],
    ["Sorted", "Bubble Sort", "Sorted", 4000, 4001, 4000, 16000000, 16000000],
    ["Sorted", "Bubble Sort", "Sorted", 6000, 6001, 6000, 36000000, 36000000],
    ["Sorted", "Bubble Sort", "Sorted", 8000, 8001, 8000, 64000000, 64000000],
  ],
  "bubble-sort-random": [
    ["Random", "Bubble Sort", "Random", 10, 99, 10, 100, 100],
    ["Random", "Bubble Sort", "Random", 15, 160, 15, 225, 225],
    ["Random", "Bubble Sort", "Random", 20, 336, 20, 400, 400],
    ["Random", "Bubble Sort", "Random", 30, 806, 30, 900, 900],
    ["Random", "Bubble Sort", "Random", 40, 1148, 40, 1600, 1600],
    ["Random", "Bubble Sort", "Random", 60, 3172, 60, 3600, 3600],
    ["Random", "Bubble Sort", "Random", 80, 5589, 80, 6400, 6400],
    ["Random", "Bubble Sort", "Random", 100, 9191, 100, 10000, 10000],
    ["Random", "Bubble Sort", "Random", 150, 21744, 150, 22500, 22500],
    ["Random", "Bubble Sort", "Random", 200, 35376, 200, 40000, 40000],
    ["Random", "Bubble Sort", "Random", 300, 83076, 300, 90000, 90000],
    ["Random", "Bubble Sort", "Random", 400, 153182, 400, 160000, 160000],
    ["Random", "Bubble Sort", "Random", 600, 340767, 600, 360000, 360000],
    ["Random", "Bubble Sort", "Random", 800, 621576, 800, 640000, 640000],
    ["Random", "Bubble Sort", "Random", 1000, 978978, 1000, 1000000, 1000000],
    ["Random", "Bubble Sort", "Random", 1500, 2186957, 1500, 2250000, 2250000],
    ["Random", "Bubble Sort", "Random", 2000, 3833916, 2000, 4000000, 4000000],
    ["Random", "Bubble Sort", "Random", 3000, 8702900, 3000, 9000000, 9000000],
    ["Random", "Bubble Sort", "Random", 4000, 15551887, 4000, 16000000, 16000000],
    ["Random", "Bubble Sort", "Random", 6000, 35087847, 6000, 36000000, 36000000],
    ["Random", "Bubble Sort", "Random", 8000, 62863857, 8000, 64000000, 64000000],
  ],
  "bubble-sort-reversed": [
    ["Reversed", "Bubble Sort", "Reversed", 10, 110, 10, 100, 100],
    ["Reversed", "Bubble Sort", "Reversed", 15, 240, 15, 225, 225],
    ["Reversed", "Bubble Sort", "Reversed", 20, 420, 20, 400, 400],
    ["Reversed", "Bubble Sort", "Reversed", 30, 930, 30, 900, 900],
    ["Reversed", "Bubble Sort", "Reversed", 40, 1640, 40, 1600, 1600],
    ["Reversed", "Bubble Sort", "Reversed", 60, 3660, 60, 3600, 3600],
    ["Reversed", "Bubble Sort", "Reversed", 80, 6480, 80, 6400, 6400],
    ["Reversed", "Bubble Sort", "Reversed", 100, 10100, 100, 10000, 10000],
    ["Reversed", "Bubble Sort", "Reversed", 150, 22650, 150, 22500, 22500],
    ["Reversed", "Bubble Sort", "Reversed", 200, 40200, 200, 40000, 40000],
    ["Reversed", "Bubble Sort", "Reversed", 300, 90300, 300, 90000, 90000],
    ["Reversed", "Bubble Sort", "Reversed", 400, 160400, 400, 160000, 160000],
    ["Reversed", "Bubble Sort", "Reversed", 600, 360600, 600, 360000, 360000],
    ["Reversed", "Bubble Sort", "Reversed", 800, 640800, 800, 640000, 640000],
    ["Reversed", "Bubble Sort", "Reversed", 1000, 1001000, 1000, 1000000, 1000000],
    ["Reversed", "Bubble Sort", "Reversed", 1500, 2251500, 1500, 2250000, 2250000],
    ["Reversed", "Bubble Sort", "Reversed", 2000, 4002000, 2000, 4000000, 4000000],
    ["Reversed", "Bubble Sort", "Reversed", 3000, 9003000, 3000, 9000000, 9000000],
    ["Reversed", "Bubble Sort", "Reversed", 4000, 16004000, 4000, 16000000, 16000000],
    ["Reversed", "Bubble Sort", "Reversed", 6000, 36006000, 6000, 36000000, 36000000],
    ["Reversed", "Bubble Sort", "Reversed", 8000, 64008000, 8000, 64000000, 64000000],
  ],
  "selection-sort-all": [
    ["Random", "Selection Sort", "Random", 10, 55, 100, 100, 100],
    ["Sorted", "Selection Sort", "Sorted", 10, 55, 100, 100, 100],
    ["Half Sorted", "Selection Sort", "Half Sorted", 10, 55, 100, 100, 100],
    ["Sorted Half", "Selection Sort", "Sorted Half", 10, 55, 100, 100, 100],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 10, 55, 100, 100, 100],
    ["Reversed", "Selection Sort", "Reversed", 10, 55, 100, 100, 100],
    ["Same", "Selection Sort", "Same", 10, 55, 100, 100, 100],
    ["Zig Zag", "Selection Sort", "Zig Zag", 10, 55, 100, 100, 100],
    ["Sawtooth", "Selection Sort", "Sawtooth", 10, 55, 100, 100, 100],
    ["Square", "Selection Sort", "Square", 10, 55, 100, 100, 100],
    ["Zeroes", "Selection Sort", "Zeroes", 10, 55, 100, 100, 100],
    ["Thousands", "Selection Sort", "Thousands", 10, 55, 100, 100, 100],
    ["Millions", "Selection Sort", "Millions", 10, 55, 100, 100, 100],
    ["Blender", "Selection Sort", "Blender", 10, 55, 100, 100, 100],
    ["Random", "Selection Sort", "Random", 15, 120, 225, 225, 225],
    ["Sorted", "Selection Sort", "Sorted", 15, 120, 225, 225, 225],
    ["Half Sorted", "Selection Sort", "Half Sorted", 14, 105, 196, 196, 196],
    ["Sorted Half", "Selection Sort", "Sorted Half", 14, 105, 196, 196, 196],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 15, 120, 225, 225, 225],
    ["Reversed", "Selection Sort", "Reversed", 15, 120, 225, 225, 225],
    ["Same", "Selection Sort", "Same", 15, 120, 225, 225, 225],
    ["Zig Zag", "Selection Sort", "Zig Zag", 15, 120, 225, 225, 225],
    ["Sawtooth", "Selection Sort", "Sawtooth", 15, 120, 225, 225, 225],
    ["Square", "Selection Sort", "Square", 15, 120, 225, 225, 225],
    ["Zeroes", "Selection Sort", "Zeroes", 15, 120, 225, 225, 225],
    ["Thousands", "Selection Sort", "Thousands", 15, 120, 225, 225, 225],
    ["Millions", "Selection Sort", "Millions", 15, 120, 225, 225, 225],
    ["Blender", "Selection Sort", "Blender", 15, 120, 225, 225, 225],
    ["Random", "Selection Sort", "Random", 20, 210, 400, 400, 400],
    ["Sorted", "Selection Sort", "Sorted", 20, 210, 400, 400, 400],
    ["Half Sorted", "Selection Sort", "Half Sorted", 20, 210, 400, 400, 400],
    ["Sorted Half", "Selection Sort", "Sorted Half", 20, 210, 400, 400, 400],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 20, 210, 400, 400, 400],
    ["Reversed", "Selection Sort", "Reversed", 20, 210, 400, 400, 400],
    ["Same", "Selection Sort", "Same", 20, 210, 400, 400, 400],
    ["Zig Zag", "Selection Sort", "Zig Zag", 20, 210, 400, 400, 400],
    ["Sawtooth", "Selection Sort", "Sawtooth", 20, 210, 400, 400, 400],
    ["Square", "Selection Sort", "Square", 20, 210, 400, 400, 400],
    ["Zeroes", "Selection Sort", "Zeroes", 20, 210, 400, 400, 400],
    ["Thousands", "Selection Sort", "Thousands", 20, 210, 400, 400, 400],
    ["Millions", "Selection Sort", "Millions", 20, 210, 400, 400, 400],
    ["Blender", "Selection Sort", "Blender", 20, 210, 400, 400, 400],
    ["Random", "Selection Sort", "Random", 30, 465, 900, 900, 900],
    ["Sorted", "Selection Sort", "Sorted", 30, 465, 900, 900, 900],
    ["Half Sorted", "Selection Sort", "Half Sorted", 30, 465, 900, 900, 900],
    ["Sorted Half", "Selection Sort", "Sorted Half", 30, 465, 900, 900, 900],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 30, 465, 900, 900, 900],
    ["Reversed", "Selection Sort", "Reversed", 30, 465, 900, 900, 900],
    ["Same", "Selection Sort", "Same", 30, 465, 900, 900, 900],
    ["Zig Zag", "Selection Sort", "Zig Zag", 30, 465, 900, 900, 900],
    ["Sawtooth", "Selection Sort", "Sawtooth", 30, 465, 900, 900, 900],
    ["Square", "Selection Sort", "Square", 30, 465, 900, 900, 900],
    ["Zeroes", "Selection Sort", "Zeroes", 30, 465, 900, 900, 900],
    ["Thousands", "Selection Sort", "Thousands", 30, 465, 900, 900, 900],
    ["Millions", "Selection Sort", "Millions", 30, 465, 900, 900, 900],
    ["Blender", "Selection Sort", "Blender", 30, 465, 900, 900, 900],
    ["Random", "Selection Sort", "Random", 40, 820, 1600, 1600, 1600],
    ["Sorted", "Selection Sort", "Sorted", 40, 820, 1600, 1600, 1600],
    ["Half Sorted", "Selection Sort", "Half Sorted", 40, 820, 1600, 1600, 1600],
    ["Sorted Half", "Selection Sort", "Sorted Half", 40, 820, 1600, 1600, 1600],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 40, 820, 1600, 1600, 1600],
    ["Reversed", "Selection Sort", "Reversed", 40, 820, 1600, 1600, 1600],
    ["Same", "Selection Sort", "Same", 40, 820, 1600, 1600, 1600],
    ["Zig Zag", "Selection Sort", "Zig Zag", 40, 820, 1600, 1600, 1600],
    ["Sawtooth", "Selection Sort", "Sawtooth", 40, 820, 1600, 1600, 1600],
    ["Square", "Selection Sort", "Square", 40, 820, 1600, 1600, 1600],
    ["Zeroes", "Selection Sort", "Zeroes", 40, 820, 1600, 1600, 1600],
    ["Thousands", "Selection Sort", "Thousands", 40, 820, 1600, 1600, 1600],
    ["Millions", "Selection Sort", "Millions", 40, 820, 1600, 1600, 1600],
    ["Blender", "Selection Sort", "Blender", 40, 820, 1600, 1600, 1600],
    ["Random", "Selection Sort", "Random", 60, 1830, 3600, 3600, 3600],
    ["Sorted", "Selection Sort", "Sorted", 60, 1830, 3600, 3600, 3600],
    ["Half Sorted", "Selection Sort", "Half Sorted", 60, 1830, 3600, 3600, 3600],
    ["Sorted Half", "Selection Sort", "Sorted Half", 60, 1830, 3600, 3600, 3600],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 60, 1830, 3600, 3600, 3600],
    ["Reversed", "Selection Sort", "Reversed", 60, 1830, 3600, 3600, 3600],
    ["Same", "Selection Sort", "Same", 60, 1830, 3600, 3600, 3600],
    ["Zig Zag", "Selection Sort", "Zig Zag", 60, 1830, 3600, 3600, 3600],
    ["Sawtooth", "Selection Sort", "Sawtooth", 60, 1830, 3600, 3600, 3600],
    ["Square", "Selection Sort", "Square", 60, 1830, 3600, 3600, 3600],
    ["Zeroes", "Selection Sort", "Zeroes", 60, 1830, 3600, 3600, 3600],
    ["Thousands", "Selection Sort", "Thousands", 60, 1830, 3600, 3600, 3600],
    ["Millions", "Selection Sort", "Millions", 60, 1830, 3600, 3600, 3600],
    ["Blender", "Selection Sort", "Blender", 60, 1830, 3600, 3600, 3600],
    ["Random", "Selection Sort", "Random", 80, 3240, 6400, 6400, 6400],
    ["Sorted", "Selection Sort", "Sorted", 80, 3240, 6400, 6400, 6400],
    ["Half Sorted", "Selection Sort", "Half Sorted", 80, 3240, 6400, 6400, 6400],
    ["Sorted Half", "Selection Sort", "Sorted Half", 80, 3240, 6400, 6400, 6400],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 80, 3240, 6400, 6400, 6400],
    ["Reversed", "Selection Sort", "Reversed", 80, 3240, 6400, 6400, 6400],
    ["Same", "Selection Sort", "Same", 80, 3240, 6400, 6400, 6400],
    ["Zig Zag", "Selection Sort", "Zig Zag", 80, 3240, 6400, 6400, 6400],
    ["Sawtooth", "Selection Sort", "Sawtooth", 80, 3240, 6400, 6400, 6400],
    ["Square", "Selection Sort", "Square", 80, 3240, 6400, 6400, 6400],
    ["Zeroes", "Selection Sort", "Zeroes", 80, 3240, 6400, 6400, 6400],
    ["Thousands", "Selection Sort", "Thousands", 80, 3240, 6400, 6400, 6400],
    ["Millions", "Selection Sort", "Millions", 80, 3240, 6400, 6400, 6400],
    ["Blender", "Selection Sort", "Blender", 80, 3240, 6400, 6400, 6400],
    ["Random", "Selection Sort", "Random", 100, 5050, 10000, 10000, 10000],
    ["Sorted", "Selection Sort", "Sorted", 100, 5050, 10000, 10000, 10000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 100, 5050, 10000, 10000, 10000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 100, 5050, 10000, 10000, 10000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 100, 5050, 10000, 10000, 10000],
    ["Reversed", "Selection Sort", "Reversed", 100, 5050, 10000, 10000, 10000],
    ["Same", "Selection Sort", "Same", 100, 5050, 10000, 10000, 10000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 100, 5050, 10000, 10000, 10000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 100, 5050, 10000, 10000, 10000],
    ["Square", "Selection Sort", "Square", 100, 5050, 10000, 10000, 10000],
    ["Zeroes", "Selection Sort", "Zeroes", 100, 5050, 10000, 10000, 10000],
    ["Thousands", "Selection Sort", "Thousands", 100, 5050, 10000, 10000, 10000],
    ["Millions", "Selection Sort", "Millions", 100, 5050, 10000, 10000, 10000],
    ["Blender", "Selection Sort", "Blender", 100, 5050, 10000, 10000, 10000],
    ["Random", "Selection Sort", "Random", 150, 11325, 22500, 22500, 22500],
    ["Sorted", "Selection Sort", "Sorted", 150, 11325, 22500, 22500, 22500],
    ["Half Sorted", "Selection Sort", "Half Sorted", 150, 11325, 22500, 22500, 22500],
    ["Sorted Half", "Selection Sort", "Sorted Half", 150, 11325, 22500, 22500, 22500],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 150, 11325, 22500, 22500, 22500],
    ["Reversed", "Selection Sort", "Reversed", 150, 11325, 22500, 22500, 22500],
    ["Same", "Selection Sort", "Same", 150, 11325, 22500, 22500, 22500],
    ["Zig Zag", "Selection Sort", "Zig Zag", 150, 11325, 22500, 22500, 22500],
    ["Sawtooth", "Selection Sort", "Sawtooth", 150, 11325, 22500, 22500, 22500],
    ["Square", "Selection Sort", "Square", 150, 11325, 22500, 22500, 22500],
    ["Zeroes", "Selection Sort", "Zeroes", 150, 11325, 22500, 22500, 22500],
    ["Thousands", "Selection Sort", "Thousands", 150, 11325, 22500, 22500, 22500],
    ["Millions", "Selection Sort", "Millions", 150, 11325, 22500, 22500, 22500],
    ["Blender", "Selection Sort", "Blender", 150, 11325, 22500, 22500, 22500],
    ["Random", "Selection Sort", "Random", 200, 20100, 40000, 40000, 40000],
    ["Sorted", "Selection Sort", "Sorted", 200, 20100, 40000, 40000, 40000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 200, 20100, 40000, 40000, 40000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 200, 20100, 40000, 40000, 40000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 200, 20100, 40000, 40000, 40000],
    ["Reversed", "Selection Sort", "Reversed", 200, 20100, 40000, 40000, 40000],
    ["Same", "Selection Sort", "Same", 200, 20100, 40000, 40000, 40000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 200, 20100, 40000, 40000, 40000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 200, 20100, 40000, 40000, 40000],
    ["Square", "Selection Sort", "Square", 200, 20100, 40000, 40000, 40000],
    ["Zeroes", "Selection Sort", "Zeroes", 200, 20100, 40000, 40000, 40000],
    ["Thousands", "Selection Sort", "Thousands", 200, 20100, 40000, 40000, 40000],
    ["Millions", "Selection Sort", "Millions", 200, 20100, 40000, 40000, 40000],
    ["Blender", "Selection Sort", "Blender", 200, 20100, 40000, 40000, 40000],
    ["Random", "Selection Sort", "Random", 300, 45150, 90000, 90000, 90000],
    ["Sorted", "Selection Sort", "Sorted", 300, 45150, 90000, 90000, 90000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 300, 45150, 90000, 90000, 90000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 300, 45150, 90000, 90000, 90000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 300, 45150, 90000, 90000, 90000],
    ["Reversed", "Selection Sort", "Reversed", 300, 45150, 90000, 90000, 90000],
    ["Same", "Selection Sort", "Same", 300, 45150, 90000, 90000, 90000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 300, 45150, 90000, 90000, 90000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 300, 45150, 90000, 90000, 90000],
    ["Square", "Selection Sort", "Square", 300, 45150, 90000, 90000, 90000],
    ["Zeroes", "Selection Sort", "Zeroes", 300, 45150, 90000, 90000, 90000],
    ["Thousands", "Selection Sort", "Thousands", 300, 45150, 90000, 90000, 90000],
    ["Millions", "Selection Sort", "Millions", 300, 45150, 90000, 90000, 90000],
    ["Blender", "Selection Sort", "Blender", 300, 45150, 90000, 90000, 90000],
    ["Random", "Selection Sort", "Random", 400, 80200, 160000, 160000, 160000],
    ["Sorted", "Selection Sort", "Sorted", 400, 80200, 160000, 160000, 160000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 400, 80200, 160000, 160000, 160000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 400, 80200, 160000, 160000, 160000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 400, 80200, 160000, 160000, 160000],
    ["Reversed", "Selection Sort", "Reversed", 400, 80200, 160000, 160000, 160000],
    ["Same", "Selection Sort", "Same", 400, 80200, 160000, 160000, 160000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 400, 80200, 160000, 160000, 160000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 400, 80200, 160000, 160000, 160000],
    ["Square", "Selection Sort", "Square", 400, 80200, 160000, 160000, 160000],
    ["Zeroes", "Selection Sort", "Zeroes", 400, 80200, 160000, 160000, 160000],
    ["Thousands", "Selection Sort", "Thousands", 400, 80200, 160000, 160000, 160000],
    ["Millions", "Selection Sort", "Millions", 400, 80200, 160000, 160000, 160000],
    ["Blender", "Selection Sort", "Blender", 400, 80200, 160000, 160000, 160000],
    ["Random", "Selection Sort", "Random", 600, 180300, 360000, 360000, 360000],
    ["Sorted", "Selection Sort", "Sorted", 600, 180300, 360000, 360000, 360000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 600, 180300, 360000, 360000, 360000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 600, 180300, 360000, 360000, 360000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 600, 180300, 360000, 360000, 360000],
    ["Reversed", "Selection Sort", "Reversed", 600, 180300, 360000, 360000, 360000],
    ["Same", "Selection Sort", "Same", 600, 180300, 360000, 360000, 360000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 600, 180300, 360000, 360000, 360000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 600, 180300, 360000, 360000, 360000],
    ["Square", "Selection Sort", "Square", 600, 180300, 360000, 360000, 360000],
    ["Zeroes", "Selection Sort", "Zeroes", 600, 180300, 360000, 360000, 360000],
    ["Thousands", "Selection Sort", "Thousands", 600, 180300, 360000, 360000, 360000],
    ["Millions", "Selection Sort", "Millions", 600, 180300, 360000, 360000, 360000],
    ["Blender", "Selection Sort", "Blender", 600, 180300, 360000, 360000, 360000],
    ["Random", "Selection Sort", "Random", 800, 320400, 640000, 640000, 640000],
    ["Sorted", "Selection Sort", "Sorted", 800, 320400, 640000, 640000, 640000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 800, 320400, 640000, 640000, 640000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 800, 320400, 640000, 640000, 640000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 800, 320400, 640000, 640000, 640000],
    ["Reversed", "Selection Sort", "Reversed", 800, 320400, 640000, 640000, 640000],
    ["Same", "Selection Sort", "Same", 800, 320400, 640000, 640000, 640000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 800, 320400, 640000, 640000, 640000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 800, 320400, 640000, 640000, 640000],
    ["Square", "Selection Sort", "Square", 800, 320400, 640000, 640000, 640000],
    ["Zeroes", "Selection Sort", "Zeroes", 800, 320400, 640000, 640000, 640000],
    ["Thousands", "Selection Sort", "Thousands", 800, 320400, 640000, 640000, 640000],
    ["Millions", "Selection Sort", "Millions", 800, 320400, 640000, 640000, 640000],
    ["Blender", "Selection Sort", "Blender", 800, 320400, 640000, 640000, 640000],
    ["Random", "Selection Sort", "Random", 1000, 500500, 1000000, 1000000, 1000000],
    ["Sorted", "Selection Sort", "Sorted", 1000, 500500, 1000000, 1000000, 1000000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 1000, 500500, 1000000, 1000000, 1000000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 1000, 500500, 1000000, 1000000, 1000000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 1000, 500500, 1000000, 1000000, 1000000],
    ["Reversed", "Selection Sort", "Reversed", 1000, 500500, 1000000, 1000000, 1000000],
    ["Same", "Selection Sort", "Same", 1000, 500500, 1000000, 1000000, 1000000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 1000, 500500, 1000000, 1000000, 1000000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 1000, 500500, 1000000, 1000000, 1000000],
    ["Square", "Selection Sort", "Square", 1000, 500500, 1000000, 1000000, 1000000],
    ["Zeroes", "Selection Sort", "Zeroes", 1000, 500500, 1000000, 1000000, 1000000],
    ["Thousands", "Selection Sort", "Thousands", 1000, 500500, 1000000, 1000000, 1000000],
    ["Millions", "Selection Sort", "Millions", 1000, 500500, 1000000, 1000000, 1000000],
    ["Blender", "Selection Sort", "Blender", 1000, 500500, 1000000, 1000000, 1000000],
    ["Random", "Selection Sort", "Random", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Sorted", "Selection Sort", "Sorted", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Reversed", "Selection Sort", "Reversed", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Same", "Selection Sort", "Same", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Square", "Selection Sort", "Square", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Zeroes", "Selection Sort", "Zeroes", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Thousands", "Selection Sort", "Thousands", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Millions", "Selection Sort", "Millions", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Blender", "Selection Sort", "Blender", 1500, 1125750, 2250000, 2250000, 2250000],
    ["Random", "Selection Sort", "Random", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Sorted", "Selection Sort", "Sorted", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Reversed", "Selection Sort", "Reversed", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Same", "Selection Sort", "Same", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Square", "Selection Sort", "Square", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Zeroes", "Selection Sort", "Zeroes", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Thousands", "Selection Sort", "Thousands", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Millions", "Selection Sort", "Millions", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Blender", "Selection Sort", "Blender", 2000, 2001000, 4000000, 4000000, 4000000],
    ["Random", "Selection Sort", "Random", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Sorted", "Selection Sort", "Sorted", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Reversed", "Selection Sort", "Reversed", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Same", "Selection Sort", "Same", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Square", "Selection Sort", "Square", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Zeroes", "Selection Sort", "Zeroes", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Thousands", "Selection Sort", "Thousands", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Millions", "Selection Sort", "Millions", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Blender", "Selection Sort", "Blender", 3000, 4501500, 9000000, 9000000, 9000000],
    ["Random", "Selection Sort", "Random", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Sorted", "Selection Sort", "Sorted", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Reversed", "Selection Sort", "Reversed", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Same", "Selection Sort", "Same", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Square", "Selection Sort", "Square", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Zeroes", "Selection Sort", "Zeroes", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Thousands", "Selection Sort", "Thousands", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Millions", "Selection Sort", "Millions", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Blender", "Selection Sort", "Blender", 4000, 8002000, 16000000, 16000000, 16000000],
    ["Random", "Selection Sort", "Random", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Sorted", "Selection Sort", "Sorted", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Reversed", "Selection Sort", "Reversed", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Same", "Selection Sort", "Same", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Square", "Selection Sort", "Square", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Zeroes", "Selection Sort", "Zeroes", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Thousands", "Selection Sort", "Thousands", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Millions", "Selection Sort", "Millions", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Blender", "Selection Sort", "Blender", 6000, 18003000, 36000000, 36000000, 36000000],
    ["Random", "Selection Sort", "Random", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Sorted", "Selection Sort", "Sorted", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Half Sorted", "Selection Sort", "Half Sorted", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Sorted Half", "Selection Sort", "Sorted Half", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Semi Sorted", "Selection Sort", "Semi Sorted", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Reversed", "Selection Sort", "Reversed", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Same", "Selection Sort", "Same", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Zig Zag", "Selection Sort", "Zig Zag", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Sawtooth", "Selection Sort", "Sawtooth", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Square", "Selection Sort", "Square", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Zeroes", "Selection Sort", "Zeroes", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Thousands", "Selection Sort", "Thousands", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Millions", "Selection Sort", "Millions", 8000, 32004000, 64000000, 64000000, 64000000],
    ["Blender", "Selection Sort", "Blender", 8000, 32004000, 64000000, 64000000, 64000000],
  ],
  "insertion-sort-sorted": [
    ["Sorted", "Insertion Sort", "Sorted", 10, 9, 10, 100, 100],
    ["Sorted", "Insertion Sort", "Sorted", 15, 14, 15, 225, 225],
    ["Sorted", "Insertion Sort", "Sorted", 20, 19, 20, 400, 400],
    ["Sorted", "Insertion Sort", "Sorted", 30, 29, 30, 900, 900],
    ["Sorted", "Insertion Sort", "Sorted", 40, 39, 40, 1600, 1600],
    ["Sorted", "Insertion Sort", "Sorted", 60, 59, 60, 3600, 3600],
    ["Sorted", "Insertion Sort", "Sorted", 80, 79, 80, 6400, 6400],
    ["Sorted", "Insertion Sort", "Sorted", 100, 99, 100, 10000, 10000],
    ["Sorted", "Insertion Sort", "Sorted", 150, 149, 150, 22500, 22500],
    ["Sorted", "Insertion Sort", "Sorted", 200, 199, 200, 40000, 40000],
    ["Sorted", "Insertion Sort", "Sorted", 300, 299, 300, 90000, 90000],
    ["Sorted", "Insertion Sort", "Sorted", 400, 399, 400, 160000, 160000],
    ["Sorted", "Insertion Sort", "Sorted", 600, 599, 600, 360000, 360000],
    ["Sorted", "Insertion Sort", "Sorted", 800, 799, 800, 640000, 640000],
    ["Sorted", "Insertion Sort", "Sorted", 1000, 999, 1000, 1000000, 1000000],
    ["Sorted", "Insertion Sort", "Sorted", 1500, 1499, 1500, 2250000, 2250000],
    ["Sorted", "Insertion Sort", "Sorted", 2000, 1999, 2000, 4000000, 4000000],
    ["Sorted", "Insertion Sort", "Sorted", 3000, 2999, 3000, 9000000, 9000000],
    ["Sorted", "Insertion Sort", "Sorted", 4000, 3999, 4000, 16000000, 16000000],
    ["Sorted", "Insertion Sort", "Sorted", 6000, 5999, 6000, 36000000, 36000000],
    ["Sorted", "Insertion Sort", "Sorted", 8000, 7999, 8000, 64000000, 64000000],
  ],
  "insertion-sort-random": [
    ["Random", "Insertion Sort", "Random", 10, 34, 10, 100, 100],
    ["Random", "Insertion Sort", "Random", 15, 50, 15, 225, 225],
    ["Random", "Insertion Sort", "Random", 20, 85, 20, 400, 400],
    ["Random", "Insertion Sort", "Random", 30, 236, 30, 900, 900],
    ["Random", "Insertion Sort", "Random", 40, 381, 40, 1600, 1600],
    ["Random", "Insertion Sort", "Random", 60, 1005, 60, 3600, 3600],
    ["Random", "Insertion Sort", "Random", 80, 1733, 80, 6400, 6400],
    ["Random", "Insertion Sort", "Random", 100, 2285, 100, 10000, 10000],
    ["Random", "Insertion Sort", "Random", 150, 5721, 150, 22500, 22500],
    ["Random", "Insertion Sort", "Random", 200, 9802, 200, 40000, 40000],
    ["Random", "Insertion Sort", "Random", 300, 22881, 300, 90000, 90000],
    ["Random", "Insertion Sort", "Random", 400, 39313, 400, 160000, 160000],
    ["Random", "Insertion Sort", "Random", 600, 92239, 600, 360000, 360000],
    ["Random", "Insertion Sort", "Random", 800, 165636, 800, 640000, 640000],
    ["Random", "Insertion Sort", "Random", 1000, 246609, 1000, 1000000, 1000000],
    ["Random", "Insertion Sort", "Random", 1500, 566628, 1500, 2250000, 2250000],
    ["Random", "Insertion Sort", "Random", 2000, 981163, 2000, 4000000, 4000000],
    ["Random", "Insertion Sort", "Random", 3000, 2273922, 3000, 9000000, 9000000],
    ["Random", "Insertion Sort", "Random", 4000, 4062374, 4000, 16000000, 16000000],
    ["Random", "Insertion Sort", "Random", 6000, 8858923, 6000, 36000000, 36000000],
    ["Random", "Insertion Sort", "Random", 8000, 15953111, 8000, 64000000, 64000000],
  ],
  "insertion-sort-reversed": [
    ["Reversed", "Insertion Sort", "Reversed", 10, 54, 10, 100, 100],
    ["Reversed", "Insertion Sort", "Reversed", 15, 119, 15, 225, 225],
    ["Reversed", "Insertion Sort", "Reversed", 20, 209, 20, 400, 400],
    ["Reversed", "Insertion Sort", "Reversed", 30, 464, 30, 900, 900],
    ["Reversed", "Insertion Sort", "Reversed", 40, 819, 40, 1600, 1600],
    ["Reversed", "Insertion Sort", "Reversed", 60, 1829, 60, 3600, 3600],
    ["Reversed", "Insertion Sort", "Reversed", 80, 3239, 80, 6400, 6400],
    ["Reversed", "Insertion Sort", "Reversed", 100, 5049, 100, 10000, 10000],
    ["Reversed", "Insertion Sort", "Reversed", 150, 11324, 150, 22500, 22500],
    ["Reversed", "Insertion Sort", "Reversed", 200, 20099, 200, 40000, 40000],
    ["Reversed", "Insertion Sort", "Reversed", 300, 45149, 300, 90000, 90000],
    ["Reversed", "Insertion Sort", "Reversed", 400, 80199, 400, 160000, 160000],
    ["Reversed", "Insertion Sort", "Reversed", 600, 180299, 600, 360000, 360000],
    ["Reversed", "Insertion Sort", "Reversed", 800, 320399, 800, 640000, 640000],
    ["Reversed", "Insertion Sort", "Reversed", 1000, 500499, 1000, 1000000, 1000000],
    ["Reversed", "Insertion Sort", "Reversed", 1500, 1125749, 1500, 2250000, 2250000],
    ["Reversed", "Insertion Sort", "Reversed", 2000, 2000999, 2000, 4000000, 4000000],
    ["Reversed", "Insertion Sort", "Reversed", 3000, 4501499, 3000, 9000000, 9000000],
    ["Reversed", "Insertion Sort", "Reversed", 4000, 8001999, 4000, 16000000, 16000000],
    ["Reversed", "Insertion Sort", "Reversed", 6000, 18002999, 6000, 36000000, 36000000],
    ["Reversed", "Insertion Sort", "Reversed", 8000, 32003999, 8000, 64000000, 64000000],
  ],
  "counting-sort-various": [
    ["Random", "Counting Sort", "Random", 10, 48, 20, 20, 20],
    ["Sorted", "Counting Sort", "Sorted", 10, 50, 20, 20, 20],
    ["Reversed", "Counting Sort", "Reversed", 10, 50, 20, 20, 20],
    ["Random", "Counting Sort", "Random", 15, 75, 30, 30, 30],
    ["Sorted", "Counting Sort", "Sorted", 15, 75, 30, 30, 30],
    ["Reversed", "Counting Sort", "Reversed", 15, 75, 30, 30, 30],
    ["Random", "Counting Sort", "Random", 20, 100, 40, 40, 40],
    ["Sorted", "Counting Sort", "Sorted", 20, 100, 40, 40, 40],
    ["Reversed", "Counting Sort", "Reversed", 20, 100, 40, 40, 40],
    ["Random", "Counting Sort", "Random", 30, 150, 60, 60, 60],
    ["Sorted", "Counting Sort", "Sorted", 30, 150, 60, 60, 60],
    ["Reversed", "Counting Sort", "Reversed", 30, 150, 60, 60, 60],
    ["Random", "Counting Sort", "Random", 40, 200, 80, 80, 80],
    ["Sorted", "Counting Sort", "Sorted", 40, 200, 80, 80, 80],
    ["Reversed", "Counting Sort", "Reversed", 40, 200, 80, 80, 80],
    ["Random", "Counting Sort", "Random", 60, 292, 120, 120, 120],
    ["Sorted", "Counting Sort", "Sorted", 60, 300, 120, 120, 120],
    ["Reversed", "Counting Sort", "Reversed", 60, 300, 120, 120, 120],
    ["Random", "Counting Sort", "Random", 80, 400, 160, 160, 160],
    ["Sorted", "Counting Sort", "Sorted", 80, 400, 160, 160, 160],
    ["Reversed", "Counting Sort", "Reversed", 80, 400, 160, 160, 160],
    ["Random", "Counting Sort", "Random", 100, 498, 200, 200, 200],
    ["Sorted", "Counting Sort", "Sorted", 100, 500, 200, 200, 200],
    ["Reversed", "Counting Sort", "Reversed", 100, 500, 200, 200, 200],
    ["Random", "Counting Sort", "Random", 150, 750, 300, 300, 300],
    ["Sorted", "Counting Sort", "Sorted", 150, 750, 300, 300, 300],
    ["Reversed", "Counting Sort", "Reversed", 150, 750, 300, 300, 300],
    ["Random", "Counting Sort", "Random", 200, 996, 400, 400, 400],
    ["Sorted", "Counting Sort", "Sorted", 200, 1000, 400, 400, 400],
    ["Reversed", "Counting Sort", "Reversed", 200, 1000, 400, 400, 400],
    ["Random", "Counting Sort", "Random", 300, 1498, 600, 600, 600],
    ["Sorted", "Counting Sort", "Sorted", 300, 1500, 600, 600, 600],
    ["Reversed", "Counting Sort", "Reversed", 300, 1500, 600, 600, 600],
    ["Random", "Counting Sort", "Random", 400, 1998, 800, 800, 800],
    ["Sorted", "Counting Sort", "Sorted", 400, 2000, 800, 800, 800],
    ["Reversed", "Counting Sort", "Reversed", 400, 2000, 800, 800, 800],
    ["Random", "Counting Sort", "Random", 600, 3000, 1200, 1200, 1200],
    ["Sorted", "Counting Sort", "Sorted", 600, 3000, 1200, 1200, 1200],
    ["Reversed", "Counting Sort", "Reversed", 600, 3000, 1200, 1200, 1200],
    ["Random", "Counting Sort", "Random", 800, 4000, 1600, 1600, 1600],
    ["Sorted", "Counting Sort", "Sorted", 800, 4000, 1600, 1600, 1600],
    ["Reversed", "Counting Sort", "Reversed", 800, 4000, 1600, 1600, 1600],
    ["Random", "Counting Sort", "Random", 1000, 4998, 2000, 2000, 2000],
    ["Sorted", "Counting Sort", "Sorted", 1000, 5000, 2000, 2000, 2000],
    ["Reversed", "Counting Sort", "Reversed", 1000, 5000, 2000, 2000, 2000],
    ["Random", "Counting Sort", "Random", 1500, 7500, 3000, 3000, 3000],
    ["Sorted", "Counting Sort", "Sorted", 1500, 7500, 3000, 3000, 3000],
    ["Reversed", "Counting Sort", "Reversed", 1500, 7500, 3000, 3000, 3000],
    ["Random", "Counting Sort", "Random", 2000, 10000, 4000, 4000, 4000],
    ["Sorted", "Counting Sort", "Sorted", 2000, 10000, 4000, 4000, 4000],
    ["Reversed", "Counting Sort", "Reversed", 2000, 10000, 4000, 4000, 4000],
    ["Random", "Counting Sort", "Random", 3000, 15000, 6000, 6000, 6000],
    ["Sorted", "Counting Sort", "Sorted", 3000, 15000, 6000, 6000, 6000],
    ["Reversed", "Counting Sort", "Reversed", 3000, 15000, 6000, 6000, 6000],
    ["Random", "Counting Sort", "Random", 4000, 20000, 8000, 8000, 8000],
    ["Sorted", "Counting Sort", "Sorted", 4000, 20000, 8000, 8000, 8000],
    ["Reversed", "Counting Sort", "Reversed", 4000, 20000, 8000, 8000, 8000],
    ["Random", "Counting Sort", "Random", 6000, 30000, 12000, 12000, 12000],
    ["Sorted", "Counting Sort", "Sorted", 6000, 30000, 12000, 12000, 12000],
    ["Reversed", "Counting Sort", "Reversed", 6000, 30000, 12000, 12000, 12000],
    ["Random", "Counting Sort", "Random", 8000, 39994, 16000, 16000, 16000],
    ["Sorted", "Counting Sort", "Sorted", 8000, 40000, 16000, 16000, 16000],
    ["Reversed", "Counting Sort", "Reversed", 8000, 40000, 16000, 16000, 16000],
  ],
  "counting-sort-identical": [
    ["Zeroes", "Counting Sort", "Zeroes", 10, 32, 20, 20, 20],
    ["Same", "Counting Sort", "Same", 10, 32, 20, 20, 20],
    ["Zeroes", "Counting Sort", "Zeroes", 15, 47, 30, 30, 30],
    ["Same", "Counting Sort", "Same", 15, 47, 30, 30, 30],
    ["Zeroes", "Counting Sort", "Zeroes", 20, 62, 40, 40, 40],
    ["Same", "Counting Sort", "Same", 20, 62, 40, 40, 40],
    ["Zeroes", "Counting Sort", "Zeroes", 30, 92, 60, 60, 60],
    ["Same", "Counting Sort", "Same", 30, 92, 60, 60, 60],
    ["Zeroes", "Counting Sort", "Zeroes", 40, 122, 80, 80, 80],
    ["Same", "Counting Sort", "Same", 40, 122, 80, 80, 80],
    ["Zeroes", "Counting Sort", "Zeroes", 60, 182, 120, 120, 120],
    ["Same", "Counting Sort", "Same", 60, 182, 120, 120, 120],
    ["Zeroes", "Counting Sort", "Zeroes", 80, 242, 160, 160, 160],
    ["Same", "Counting Sort", "Same", 80, 242, 160, 160, 160],
    ["Zeroes", "Counting Sort", "Zeroes", 100, 302, 200, 200, 200],
    ["Same", "Counting Sort", "Same", 100, 302, 200, 200, 200],
    ["Zeroes", "Counting Sort", "Zeroes", 150, 452, 300, 300, 300],
    ["Same", "Counting Sort", "Same", 150, 452, 300, 300, 300],
    ["Zeroes", "Counting Sort", "Zeroes", 200, 602, 400, 400, 400],
    ["Same", "Counting Sort", "Same", 200, 602, 400, 400, 400],
    ["Zeroes", "Counting Sort", "Zeroes", 300, 902, 600, 600, 600],
    ["Same", "Counting Sort", "Same", 300, 902, 600, 600, 600],
    ["Zeroes", "Counting Sort", "Zeroes", 400, 1202, 800, 800, 800],
    ["Same", "Counting Sort", "Same", 400, 1202, 800, 800, 800],
    ["Zeroes", "Counting Sort", "Zeroes", 600, 1802, 1200, 1200, 1200],
    ["Same", "Counting Sort", "Same", 600, 1802, 1200, 1200, 1200],
    ["Zeroes", "Counting Sort", "Zeroes", 800, 2402, 1600, 1600, 1600],
    ["Same", "Counting Sort", "Same", 800, 2402, 1600, 1600, 1600],
    ["Zeroes", "Counting Sort", "Zeroes", 1000, 3002, 2000, 2000, 2000],
    ["Same", "Counting Sort", "Same", 1000, 3002, 2000, 2000, 2000],
    ["Zeroes", "Counting Sort", "Zeroes", 1500, 4502, 3000, 3000, 3000],
    ["Same", "Counting Sort", "Same", 1500, 4502, 3000, 3000, 3000],
    ["Zeroes", "Counting Sort", "Zeroes", 2000, 6002, 4000, 4000, 4000],
    ["Same", "Counting Sort", "Same", 2000, 6002, 4000, 4000, 4000],
    ["Zeroes", "Counting Sort", "Zeroes", 3000, 9002, 6000, 6000, 6000],
    ["Same", "Counting Sort", "Same", 3000, 9002, 6000, 6000, 6000],
    ["Zeroes", "Counting Sort", "Zeroes", 4000, 12002, 8000, 8000, 8000],
    ["Same", "Counting Sort", "Same", 4000, 12002, 8000, 8000, 8000],
    ["Zeroes", "Counting Sort", "Zeroes", 6000, 18002, 12000, 12000, 12000],
    ["Same", "Counting Sort", "Same", 6000, 18002, 12000, 12000, 12000],
    ["Zeroes", "Counting Sort", "Zeroes", 8000, 24002, 16000, 16000, 16000],
    ["Same", "Counting Sort", "Same", 8000, 24002, 16000, 16000, 16000],
  ],
  "bubble-sort-variance": [
    ["Thousands", "Counting Sort", "Thousands", 10, 1930, 20, 20, 20],
    ["Millions", "Counting Sort", "Millions", 10, 1845748, 20, 20, 20],
    ["Thousands", "Counting Sort", "Thousands", 15, 1311, 30, 30, 30],
    ["Millions", "Counting Sort", "Millions", 15, 1717057, 30, 30, 30],
    ["Thousands", "Counting Sort", "Thousands", 20, 1984, 40, 40, 40],
    ["Millions", "Counting Sort", "Millions", 20, 1968686, 40, 40, 40],
    ["Thousands", "Counting Sort", "Thousands", 30, 1968, 60, 60, 60],
    ["Millions", "Counting Sort", "Millions", 30, 1942366, 60, 60, 60],
    ["Thousands", "Counting Sort", "Thousands", 40, 2112, 80, 80, 80],
    ["Millions", "Counting Sort", "Millions", 40, 1979746, 80, 80, 80],
    ["Thousands", "Counting Sort", "Thousands", 60, 2088, 120, 120, 120],
    ["Millions", "Counting Sort", "Millions", 60, 1941688, 120, 120, 120],
    ["Thousands", "Counting Sort", "Thousands", 80, 2208, 160, 160, 160],
    ["Millions", "Counting Sort", "Millions", 80, 1958276, 160, 160, 160],
    ["Thousands", "Counting Sort", "Thousands", 100, 2264, 200, 200, 200],
    ["Millions", "Counting Sort", "Millions", 100, 1962642, 200, 200, 200],
    ["Thousands", "Counting Sort", "Thousands", 150, 2434, 300, 300, 300],
    ["Millions", "Counting Sort", "Millions", 150, 1987072, 300, 300, 300],
    ["Thousands", "Counting Sort", "Thousands", 200, 2594, 400, 400, 400],
    ["Millions", "Counting Sort", "Millions", 200, 1974248, 400, 400, 400],
    ["Thousands", "Counting Sort", "Thousands", 300, 2894, 600, 600, 600],
    ["Millions", "Counting Sort", "Millions", 300, 1983498, 600, 600, 600],
    ["Thousands", "Counting Sort", "Thousands", 400, 3200, 800, 800, 800],
    ["Millions", "Counting Sort", "Millions", 400, 1991110, 800, 800, 800],
    ["Thousands", "Counting Sort", "Thousands", 600, 3794, 1200, 1200, 1200],
    ["Millions", "Counting Sort", "Millions", 600, 1994658, 1200, 1200, 1200],
    ["Thousands", "Counting Sort", "Thousands", 800, 4400, 1600, 1600, 1600],
    ["Millions", "Counting Sort", "Millions", 800, 1997864, 1600, 1600, 1600],
    ["Thousands", "Counting Sort", "Thousands", 1000, 5000, 2000, 2000, 2000],
    ["Millions", "Counting Sort", "Millions", 1000, 1999278, 2000, 2000, 2000],
    ["Thousands", "Counting Sort", "Thousands", 1500, 6500, 3000, 3000, 3000],
    ["Millions", "Counting Sort", "Millions", 1500, 2003210, 3000, 3000, 3000],
    ["Thousands", "Counting Sort", "Thousands", 2000, 7998, 4000, 4000, 4000],
    ["Millions", "Counting Sort", "Millions", 2000, 2001004, 4000, 4000, 4000],
    ["Thousands", "Counting Sort", "Thousands", 3000, 11000, 6000, 6000, 6000],
    ["Millions", "Counting Sort", "Millions", 3000, 2006734, 6000, 6000, 6000],
    ["Thousands", "Counting Sort", "Thousands", 4000, 14000, 8000, 8000, 8000],
    ["Millions", "Counting Sort", "Millions", 4000, 2011704, 8000, 8000, 8000],
    ["Thousands", "Counting Sort", "Thousands", 6000, 20000, 12000, 12000, 12000],
    ["Millions", "Counting Sort", "Millions", 6000, 2017298, 12000, 12000, 12000],
    ["Thousands", "Counting Sort", "Thousands", 8000, 26000, 16000, 16000, 16000],
    ["Millions", "Counting Sort", "Millions", 8000, 2023652, 16000, 16000, 16000],
  ],
  "quick-sort-sorted": [
    ["Sorted", "Quick Sort", "Sorted", 10, 64, 33, 33, 100],
    ["Sorted", "Quick Sort", "Sorted", 15, 134, 59, 59, 225],
    ["Sorted", "Quick Sort", "Sorted", 20, 229, 86, 86, 400],
    ["Sorted", "Quick Sort", "Sorted", 30, 494, 147, 147, 900],
    ["Sorted", "Quick Sort", "Sorted", 40, 859, 213, 213, 1600],
    ["Sorted", "Quick Sort", "Sorted", 60, 1889, 354, 354, 3600],
    ["Sorted", "Quick Sort", "Sorted", 80, 3319, 506, 506, 6400],
    ["Sorted", "Quick Sort", "Sorted", 100, 5149, 664, 664, 10000],
    ["Sorted", "Quick Sort", "Sorted", 150, 11474, 1084, 1084, 22500],
    ["Sorted", "Quick Sort", "Sorted", 200, 20299, 1529, 1529, 40000],
    ["Sorted", "Quick Sort", "Sorted", 300, 45449, 2469, 2469, 90000],
    ["Sorted", "Quick Sort", "Sorted", 400, 80599, 3458, 3458, 160000],
    ["Sorted", "Quick Sort", "Sorted", 600, 180899, 5537, 5537, 360000],
    ["Sorted", "Quick Sort", "Sorted", 800, 321199, 7715, 7715, 640000],
    ["Sorted", "Quick Sort", "Sorted", 1000, 501499, 9966, 9966, 1000000],
    ["Sorted", "Quick Sort", "Sorted", 1500, 1127249, 15826, 15826, 2250000],
    ["Sorted", "Quick Sort", "Sorted", 2000, 2002999, 21932, 21932, 4000000],
    ["Sorted", "Quick Sort", "Sorted", 3000, 4504499, 34652, 34652, 9000000],
    ["Sorted", "Quick Sort", "Sorted", 4000, 8005999, 47863, 47863, 16000000],
    ["Sorted", "Quick Sort", "Sorted", 6000, 18008999, 75304, 75304, 36000000],
    ["Sorted", "Quick Sort", "Sorted", 8000, 30427800, 103726, 103726, 64000000],
  ],
  "quick-sort-blended": [
    ["Blender", "Quick Sort", "Blender", 839, 15456, 8149, 8149, 703921],
    ["Blender", "Quick Sort", "Blender", 881, 18537, 8619, 8619, 776161],
    ["Blender", "Quick Sort", "Blender", 1078, 23282, 10860, 10860, 1162084],
    ["Blender", "Quick Sort", "Blender", 1410, 27570, 14751, 14751, 1988100],
    ["Blender", "Quick Sort", "Blender", 1223, 31541, 12543, 12543, 1495729],
    ["Blender", "Quick Sort", "Blender", 956, 20733, 9465, 9465, 913936],
    ["Blender", "Quick Sort", "Blender", 1130, 21494, 11461, 11461, 1276900],
    ["Blender", "Quick Sort", "Blender", 1330, 44975, 13802, 13802, 1768900],
    ["Blender", "Quick Sort", "Blender", 1556, 41307, 16499, 16499, 2421136],
    ["Blender", "Quick Sort", "Blender", 1417, 32898, 14834, 14834, 2007889],
    ["Blender", "Quick Sort", "Blender", 1927, 81101, 21028, 21028, 3713329],
    ["Blender", "Quick Sort", "Blender", 2109, 62326, 23288, 23288, 4447881],
    ["Blender", "Quick Sort", "Blender", 2166, 76130, 24001, 24001, 4691556],
    ["Blender", "Quick Sort", "Blender", 1664, 40273, 17806, 17806, 2768896],
    ["Blender", "Quick Sort", "Blender", 2425, 114346, 27266, 27266, 5880625],
    ["Blender", "Quick Sort", "Blender", 2571, 93108, 29125, 29125, 6610041],
    ["Blender", "Quick Sort", "Blender", 2806, 66203, 32141, 32141, 7873636],
    ["Blender", "Quick Sort", "Blender", 1744, 34306, 18780, 18780, 3041536],
    ["Blender", "Quick Sort", "Blender", 2778, 101243, 31780, 31780, 7717284],
    ["Blender", "Quick Sort", "Blender", 2155, 92781, 23863, 23863, 4644025],
    ["Blender", "Quick Sort", "Blender", 2455, 103557, 27647, 27647, 6027025],
    ["Blender", "Quick Sort", "Blender", 3280, 155996, 38309, 38309, 10758400],
    ["Blender", "Quick Sort", "Blender", 2385, 110155, 26759, 26759, 5688225],
    ["Blender", "Quick Sort", "Blender", 2750, 111718, 31419, 31419, 7562500],
    ["Blender", "Quick Sort", "Blender", 3282, 188046, 38335, 38335, 10771524],
    ["Blender", "Quick Sort", "Blender", 3734, 290152, 44310, 44310, 13942756],
    ["Blender", "Quick Sort", "Blender", 3374, 158140, 39544, 39544, 11383876],
    ["Blender", "Quick Sort", "Blender", 4059, 222609, 48655, 48655, 16475481],
    ["Blender", "Quick Sort", "Blender", 2628, 98628, 29853, 29853, 6906384],
    ["Blender", "Quick Sort", "Blender", 3846, 253632, 45803, 45803, 14791716],
    ["Blender", "Quick Sort", "Blender", 4011, 250936, 48011, 48011, 16088121],
    ["Blender", "Quick Sort", "Blender", 4934, 341966, 60533, 60533, 24344356],
    ["Blender", "Quick Sort", "Blender", 4589, 352385, 55820, 55820, 21058921],
    ["Blender", "Quick Sort", "Blender", 4108, 414488, 49313, 49313, 16875664],
    ["Blender", "Quick Sort", "Blender", 3388, 144271, 39728, 39728, 11478544],
    ["Blender", "Quick Sort", "Blender", 5011, 579826, 61590, 61590, 25110121],
    ["Blender", "Quick Sort", "Blender", 5448, 280877, 67618, 67618, 29680704],
    ["Blender", "Quick Sort", "Blender", 6168, 366522, 77659, 77659, 38044224],
    ["Blender", "Quick Sort", "Blender", 4781, 139260, 58439, 58439, 22857961],
  ],
  "merge-sort-all": [
    ["Random", "Merge Sort", "Random", 10, 68, 33, 33, 33],
    ["Sorted", "Merge Sort", "Sorted", 10, 68, 33, 33, 33],
    ["Half Sorted", "Merge Sort", "Half Sorted", 10, 68, 33, 33, 33],
    ["Sorted Half", "Merge Sort", "Sorted Half", 10, 68, 33, 33, 33],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 10, 68, 33, 33, 33],
    ["Reversed", "Merge Sort", "Reversed", 10, 68, 33, 33, 33],
    ["Same", "Merge Sort", "Same", 10, 68, 33, 33, 33],
    ["Zig Zag", "Merge Sort", "Zig Zag", 10, 68, 33, 33, 33],
    ["Sawtooth", "Merge Sort", "Sawtooth", 10, 68, 33, 33, 33],
    ["Square", "Merge Sort", "Square", 10, 68, 33, 33, 33],
    ["Zeroes", "Merge Sort", "Zeroes", 10, 68, 33, 33, 33],
    ["Thousands", "Merge Sort", "Thousands", 10, 68, 33, 33, 33],
    ["Millions", "Merge Sort", "Millions", 10, 68, 33, 33, 33],
    ["Blender", "Merge Sort", "Blender", 10, 68, 33, 33, 33],
    ["Random", "Merge Sort", "Random", 15, 118, 59, 59, 59],
    ["Sorted", "Merge Sort", "Sorted", 15, 118, 59, 59, 59],
    ["Half Sorted", "Merge Sort", "Half Sorted", 14, 108, 53, 53, 53],
    ["Sorted Half", "Merge Sort", "Sorted Half", 14, 108, 53, 53, 53],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 15, 118, 59, 59, 59],
    ["Reversed", "Merge Sort", "Reversed", 15, 118, 59, 59, 59],
    ["Same", "Merge Sort", "Same", 15, 118, 59, 59, 59],
    ["Zig Zag", "Merge Sort", "Zig Zag", 15, 118, 59, 59, 59],
    ["Sawtooth", "Merge Sort", "Sawtooth", 15, 118, 59, 59, 59],
    ["Square", "Merge Sort", "Square", 15, 118, 59, 59, 59],
    ["Zeroes", "Merge Sort", "Zeroes", 15, 118, 59, 59, 59],
    ["Thousands", "Merge Sort", "Thousands", 15, 118, 59, 59, 59],
    ["Millions", "Merge Sort", "Millions", 15, 118, 59, 59, 59],
    ["Blender", "Merge Sort", "Blender", 15, 118, 59, 59, 59],
    ["Random", "Merge Sort", "Random", 20, 176, 86, 86, 86],
    ["Sorted", "Merge Sort", "Sorted", 20, 176, 86, 86, 86],
    ["Half Sorted", "Merge Sort", "Half Sorted", 20, 176, 86, 86, 86],
    ["Sorted Half", "Merge Sort", "Sorted Half", 20, 176, 86, 86, 86],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 20, 176, 86, 86, 86],
    ["Reversed", "Merge Sort", "Reversed", 20, 176, 86, 86, 86],
    ["Same", "Merge Sort", "Same", 20, 176, 86, 86, 86],
    ["Zig Zag", "Merge Sort", "Zig Zag", 20, 176, 86, 86, 86],
    ["Sawtooth", "Merge Sort", "Sawtooth", 20, 176, 86, 86, 86],
    ["Square", "Merge Sort", "Square", 20, 176, 86, 86, 86],
    ["Zeroes", "Merge Sort", "Zeroes", 20, 176, 86, 86, 86],
    ["Thousands", "Merge Sort", "Thousands", 20, 176, 86, 86, 86],
    ["Millions", "Merge Sort", "Millions", 20, 176, 86, 86, 86],
    ["Blender", "Merge Sort", "Blender", 20, 176, 86, 86, 86],
    ["Random", "Merge Sort", "Random", 30, 296, 147, 147, 147],
    ["Sorted", "Merge Sort", "Sorted", 30, 296, 147, 147, 147],
    ["Half Sorted", "Merge Sort", "Half Sorted", 30, 296, 147, 147, 147],
    ["Sorted Half", "Merge Sort", "Sorted Half", 30, 296, 147, 147, 147],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 30, 296, 147, 147, 147],
    ["Reversed", "Merge Sort", "Reversed", 30, 296, 147, 147, 147],
    ["Same", "Merge Sort", "Same", 30, 296, 147, 147, 147],
    ["Zig Zag", "Merge Sort", "Zig Zag", 30, 296, 147, 147, 147],
    ["Sawtooth", "Merge Sort", "Sawtooth", 30, 296, 147, 147, 147],
    ["Square", "Merge Sort", "Square", 30, 296, 147, 147, 147],
    ["Zeroes", "Merge Sort", "Zeroes", 30, 296, 147, 147, 147],
    ["Thousands", "Merge Sort", "Thousands", 30, 296, 147, 147, 147],
    ["Millions", "Merge Sort", "Millions", 30, 296, 147, 147, 147],
    ["Blender", "Merge Sort", "Blender", 30, 296, 147, 147, 147],
    ["Random", "Merge Sort", "Random", 40, 432, 213, 213, 213],
    ["Sorted", "Merge Sort", "Sorted", 40, 432, 213, 213, 213],
    ["Half Sorted", "Merge Sort", "Half Sorted", 40, 432, 213, 213, 213],
    ["Sorted Half", "Merge Sort", "Sorted Half", 40, 432, 213, 213, 213],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 40, 432, 213, 213, 213],
    ["Reversed", "Merge Sort", "Reversed", 40, 432, 213, 213, 213],
    ["Same", "Merge Sort", "Same", 40, 432, 213, 213, 213],
    ["Zig Zag", "Merge Sort", "Zig Zag", 40, 432, 213, 213, 213],
    ["Sawtooth", "Merge Sort", "Sawtooth", 40, 432, 213, 213, 213],
    ["Square", "Merge Sort", "Square", 40, 432, 213, 213, 213],
    ["Zeroes", "Merge Sort", "Zeroes", 40, 432, 213, 213, 213],
    ["Thousands", "Merge Sort", "Thousands", 40, 432, 213, 213, 213],
    ["Millions", "Merge Sort", "Millions", 40, 432, 213, 213, 213],
    ["Blender", "Merge Sort", "Blender", 40, 432, 213, 213, 213],
    ["Random", "Merge Sort", "Random", 60, 712, 354, 354, 354],
    ["Sorted", "Merge Sort", "Sorted", 60, 712, 354, 354, 354],
    ["Half Sorted", "Merge Sort", "Half Sorted", 60, 712, 354, 354, 354],
    ["Sorted Half", "Merge Sort", "Sorted Half", 60, 712, 354, 354, 354],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 60, 712, 354, 354, 354],
    ["Reversed", "Merge Sort", "Reversed", 60, 712, 354, 354, 354],
    ["Same", "Merge Sort", "Same", 60, 712, 354, 354, 354],
    ["Zig Zag", "Merge Sort", "Zig Zag", 60, 712, 354, 354, 354],
    ["Sawtooth", "Merge Sort", "Sawtooth", 60, 712, 354, 354, 354],
    ["Square", "Merge Sort", "Square", 60, 712, 354, 354, 354],
    ["Zeroes", "Merge Sort", "Zeroes", 60, 712, 354, 354, 354],
    ["Thousands", "Merge Sort", "Thousands", 60, 712, 354, 354, 354],
    ["Millions", "Merge Sort", "Millions", 60, 712, 354, 354, 354],
    ["Blender", "Merge Sort", "Blender", 60, 712, 354, 354, 354],
    ["Random", "Merge Sort", "Random", 80, 1024, 506, 506, 506],
    ["Sorted", "Merge Sort", "Sorted", 80, 1024, 506, 506, 506],
    ["Half Sorted", "Merge Sort", "Half Sorted", 80, 1024, 506, 506, 506],
    ["Sorted Half", "Merge Sort", "Sorted Half", 80, 1024, 506, 506, 506],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 80, 1024, 506, 506, 506],
    ["Reversed", "Merge Sort", "Reversed", 80, 1024, 506, 506, 506],
    ["Same", "Merge Sort", "Same", 80, 1024, 506, 506, 506],
    ["Zig Zag", "Merge Sort", "Zig Zag", 80, 1024, 506, 506, 506],
    ["Sawtooth", "Merge Sort", "Sawtooth", 80, 1024, 506, 506, 506],
    ["Square", "Merge Sort", "Square", 80, 1024, 506, 506, 506],
    ["Zeroes", "Merge Sort", "Zeroes", 80, 1024, 506, 506, 506],
    ["Thousands", "Merge Sort", "Thousands", 80, 1024, 506, 506, 506],
    ["Millions", "Merge Sort", "Millions", 80, 1024, 506, 506, 506],
    ["Blender", "Merge Sort", "Blender", 80, 1024, 506, 506, 506],
    ["Random", "Merge Sort", "Random", 100, 1344, 664, 664, 664],
    ["Sorted", "Merge Sort", "Sorted", 100, 1344, 664, 664, 664],
    ["Half Sorted", "Merge Sort", "Half Sorted", 100, 1344, 664, 664, 664],
    ["Sorted Half", "Merge Sort", "Sorted Half", 100, 1344, 664, 664, 664],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 100, 1344, 664, 664, 664],
    ["Reversed", "Merge Sort", "Reversed", 100, 1344, 664, 664, 664],
    ["Same", "Merge Sort", "Same", 100, 1344, 664, 664, 664],
    ["Zig Zag", "Merge Sort", "Zig Zag", 100, 1344, 664, 664, 664],
    ["Sawtooth", "Merge Sort", "Sawtooth", 100, 1344, 664, 664, 664],
    ["Square", "Merge Sort", "Square", 100, 1344, 664, 664, 664],
    ["Zeroes", "Merge Sort", "Zeroes", 100, 1344, 664, 664, 664],
    ["Thousands", "Merge Sort", "Thousands", 100, 1344, 664, 664, 664],
    ["Millions", "Merge Sort", "Millions", 100, 1344, 664, 664, 664],
    ["Blender", "Merge Sort", "Blender", 100, 1344, 664, 664, 664],
    ["Random", "Merge Sort", "Random", 150, 2188, 1084, 1084, 1084],
    ["Sorted", "Merge Sort", "Sorted", 150, 2188, 1084, 1084, 1084],
    ["Half Sorted", "Merge Sort", "Half Sorted", 150, 2188, 1084, 1084, 1084],
    ["Sorted Half", "Merge Sort", "Sorted Half", 150, 2188, 1084, 1084, 1084],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 150, 2188, 1084, 1084, 1084],
    ["Reversed", "Merge Sort", "Reversed", 150, 2188, 1084, 1084, 1084],
    ["Same", "Merge Sort", "Same", 150, 2188, 1084, 1084, 1084],
    ["Zig Zag", "Merge Sort", "Zig Zag", 150, 2188, 1084, 1084, 1084],
    ["Sawtooth", "Merge Sort", "Sawtooth", 150, 2188, 1084, 1084, 1084],
    ["Square", "Merge Sort", "Square", 150, 2188, 1084, 1084, 1084],
    ["Zeroes", "Merge Sort", "Zeroes", 150, 2188, 1084, 1084, 1084],
    ["Thousands", "Merge Sort", "Thousands", 150, 2188, 1084, 1084, 1084],
    ["Millions", "Merge Sort", "Millions", 150, 2188, 1084, 1084, 1084],
    ["Blender", "Merge Sort", "Blender", 150, 2188, 1084, 1084, 1084],
    ["Random", "Merge Sort", "Random", 200, 3088, 1529, 1529, 1529],
    ["Sorted", "Merge Sort", "Sorted", 200, 3088, 1529, 1529, 1529],
    ["Half Sorted", "Merge Sort", "Half Sorted", 200, 3088, 1529, 1529, 1529],
    ["Sorted Half", "Merge Sort", "Sorted Half", 200, 3088, 1529, 1529, 1529],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 200, 3088, 1529, 1529, 1529],
    ["Reversed", "Merge Sort", "Reversed", 200, 3088, 1529, 1529, 1529],
    ["Same", "Merge Sort", "Same", 200, 3088, 1529, 1529, 1529],
    ["Zig Zag", "Merge Sort", "Zig Zag", 200, 3088, 1529, 1529, 1529],
    ["Sawtooth", "Merge Sort", "Sawtooth", 200, 3088, 1529, 1529, 1529],
    ["Square", "Merge Sort", "Square", 200, 3088, 1529, 1529, 1529],
    ["Zeroes", "Merge Sort", "Zeroes", 200, 3088, 1529, 1529, 1529],
    ["Thousands", "Merge Sort", "Thousands", 200, 3088, 1529, 1529, 1529],
    ["Millions", "Merge Sort", "Millions", 200, 3088, 1529, 1529, 1529],
    ["Blender", "Merge Sort", "Blender", 200, 3088, 1529, 1529, 1529],
    ["Random", "Merge Sort", "Random", 300, 4976, 2469, 2469, 2469],
    ["Sorted", "Merge Sort", "Sorted", 300, 4976, 2469, 2469, 2469],
    ["Half Sorted", "Merge Sort", "Half Sorted", 300, 4976, 2469, 2469, 2469],
    ["Sorted Half", "Merge Sort", "Sorted Half", 300, 4976, 2469, 2469, 2469],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 300, 4976, 2469, 2469, 2469],
    ["Reversed", "Merge Sort", "Reversed", 300, 4976, 2469, 2469, 2469],
    ["Same", "Merge Sort", "Same", 300, 4976, 2469, 2469, 2469],
    ["Zig Zag", "Merge Sort", "Zig Zag", 300, 4976, 2469, 2469, 2469],
    ["Sawtooth", "Merge Sort", "Sawtooth", 300, 4976, 2469, 2469, 2469],
    ["Square", "Merge Sort", "Square", 300, 4976, 2469, 2469, 2469],
    ["Zeroes", "Merge Sort", "Zeroes", 300, 4976, 2469, 2469, 2469],
    ["Thousands", "Merge Sort", "Thousands", 300, 4976, 2469, 2469, 2469],
    ["Millions", "Merge Sort", "Millions", 300, 4976, 2469, 2469, 2469],
    ["Blender", "Merge Sort", "Blender", 300, 4976, 2469, 2469, 2469],
    ["Random", "Merge Sort", "Random", 400, 6976, 3458, 3458, 3458],
    ["Sorted", "Merge Sort", "Sorted", 400, 6976, 3458, 3458, 3458],
    ["Half Sorted", "Merge Sort", "Half Sorted", 400, 6976, 3458, 3458, 3458],
    ["Sorted Half", "Merge Sort", "Sorted Half", 400, 6976, 3458, 3458, 3458],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 400, 6976, 3458, 3458, 3458],
    ["Reversed", "Merge Sort", "Reversed", 400, 6976, 3458, 3458, 3458],
    ["Same", "Merge Sort", "Same", 400, 6976, 3458, 3458, 3458],
    ["Zig Zag", "Merge Sort", "Zig Zag", 400, 6976, 3458, 3458, 3458],
    ["Sawtooth", "Merge Sort", "Sawtooth", 400, 6976, 3458, 3458, 3458],
    ["Square", "Merge Sort", "Square", 400, 6976, 3458, 3458, 3458],
    ["Zeroes", "Merge Sort", "Zeroes", 400, 6976, 3458, 3458, 3458],
    ["Thousands", "Merge Sort", "Thousands", 400, 6976, 3458, 3458, 3458],
    ["Millions", "Merge Sort", "Millions", 400, 6976, 3458, 3458, 3458],
    ["Blender", "Merge Sort", "Blender", 400, 6976, 3458, 3458, 3458],
    ["Random", "Merge Sort", "Random", 600, 11152, 5537, 5537, 5537],
    ["Sorted", "Merge Sort", "Sorted", 600, 11152, 5537, 5537, 5537],
    ["Half Sorted", "Merge Sort", "Half Sorted", 600, 11152, 5537, 5537, 5537],
    ["Sorted Half", "Merge Sort", "Sorted Half", 600, 11152, 5537, 5537, 5537],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 600, 11152, 5537, 5537, 5537],
    ["Reversed", "Merge Sort", "Reversed", 600, 11152, 5537, 5537, 5537],
    ["Same", "Merge Sort", "Same", 600, 11152, 5537, 5537, 5537],
    ["Zig Zag", "Merge Sort", "Zig Zag", 600, 11152, 5537, 5537, 5537],
    ["Sawtooth", "Merge Sort", "Sawtooth", 600, 11152, 5537, 5537, 5537],
    ["Square", "Merge Sort", "Square", 600, 11152, 5537, 5537, 5537],
    ["Zeroes", "Merge Sort", "Zeroes", 600, 11152, 5537, 5537, 5537],
    ["Thousands", "Merge Sort", "Thousands", 600, 11152, 5537, 5537, 5537],
    ["Millions", "Merge Sort", "Millions", 600, 11152, 5537, 5537, 5537],
    ["Blender", "Merge Sort", "Blender", 600, 11152, 5537, 5537, 5537],
    ["Random", "Merge Sort", "Random", 800, 15552, 7715, 7715, 7715],
    ["Sorted", "Merge Sort", "Sorted", 800, 15552, 7715, 7715, 7715],
    ["Half Sorted", "Merge Sort", "Half Sorted", 800, 15552, 7715, 7715, 7715],
    ["Sorted Half", "Merge Sort", "Sorted Half", 800, 15552, 7715, 7715, 7715],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 800, 15552, 7715, 7715, 7715],
    ["Reversed", "Merge Sort", "Reversed", 800, 15552, 7715, 7715, 7715],
    ["Same", "Merge Sort", "Same", 800, 15552, 7715, 7715, 7715],
    ["Zig Zag", "Merge Sort", "Zig Zag", 800, 15552, 7715, 7715, 7715],
    ["Sawtooth", "Merge Sort", "Sawtooth", 800, 15552, 7715, 7715, 7715],
    ["Square", "Merge Sort", "Square", 800, 15552, 7715, 7715, 7715],
    ["Zeroes", "Merge Sort", "Zeroes", 800, 15552, 7715, 7715, 7715],
    ["Thousands", "Merge Sort", "Thousands", 800, 15552, 7715, 7715, 7715],
    ["Millions", "Merge Sort", "Millions", 800, 15552, 7715, 7715, 7715],
    ["Blender", "Merge Sort", "Blender", 800, 15552, 7715, 7715, 7715],
    ["Random", "Merge Sort", "Random", 1000, 19952, 9966, 9966, 9966],
    ["Sorted", "Merge Sort", "Sorted", 1000, 19952, 9966, 9966, 9966],
    ["Half Sorted", "Merge Sort", "Half Sorted", 1000, 19952, 9966, 9966, 9966],
    ["Sorted Half", "Merge Sort", "Sorted Half", 1000, 19952, 9966, 9966, 9966],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 1000, 19952, 9966, 9966, 9966],
    ["Reversed", "Merge Sort", "Reversed", 1000, 19952, 9966, 9966, 9966],
    ["Same", "Merge Sort", "Same", 1000, 19952, 9966, 9966, 9966],
    ["Zig Zag", "Merge Sort", "Zig Zag", 1000, 19952, 9966, 9966, 9966],
    ["Sawtooth", "Merge Sort", "Sawtooth", 1000, 19952, 9966, 9966, 9966],
    ["Square", "Merge Sort", "Square", 1000, 19952, 9966, 9966, 9966],
    ["Zeroes", "Merge Sort", "Zeroes", 1000, 19952, 9966, 9966, 9966],
    ["Thousands", "Merge Sort", "Thousands", 1000, 19952, 9966, 9966, 9966],
    ["Millions", "Merge Sort", "Millions", 1000, 19952, 9966, 9966, 9966],
    ["Blender", "Merge Sort", "Blender", 1000, 19952, 9966, 9966, 9966],
    ["Random", "Merge Sort", "Random", 1500, 31904, 15826, 15826, 15826],
    ["Sorted", "Merge Sort", "Sorted", 1500, 31904, 15826, 15826, 15826],
    ["Half Sorted", "Merge Sort", "Half Sorted", 1500, 31904, 15826, 15826, 15826],
    ["Sorted Half", "Merge Sort", "Sorted Half", 1500, 31904, 15826, 15826, 15826],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 1500, 31904, 15826, 15826, 15826],
    ["Reversed", "Merge Sort", "Reversed", 1500, 31904, 15826, 15826, 15826],
    ["Same", "Merge Sort", "Same", 1500, 31904, 15826, 15826, 15826],
    ["Zig Zag", "Merge Sort", "Zig Zag", 1500, 31904, 15826, 15826, 15826],
    ["Sawtooth", "Merge Sort", "Sawtooth", 1500, 31904, 15826, 15826, 15826],
    ["Square", "Merge Sort", "Square", 1500, 31904, 15826, 15826, 15826],
    ["Zeroes", "Merge Sort", "Zeroes", 1500, 31904, 15826, 15826, 15826],
    ["Thousands", "Merge Sort", "Thousands", 1500, 31904, 15826, 15826, 15826],
    ["Millions", "Merge Sort", "Millions", 1500, 31904, 15826, 15826, 15826],
    ["Blender", "Merge Sort", "Blender", 1500, 31904, 15826, 15826, 15826],
    ["Random", "Merge Sort", "Random", 2000, 43904, 21932, 21932, 21932],
    ["Sorted", "Merge Sort", "Sorted", 2000, 43904, 21932, 21932, 21932],
    ["Half Sorted", "Merge Sort", "Half Sorted", 2000, 43904, 21932, 21932, 21932],
    ["Sorted Half", "Merge Sort", "Sorted Half", 2000, 43904, 21932, 21932, 21932],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 2000, 43904, 21932, 21932, 21932],
    ["Reversed", "Merge Sort", "Reversed", 2000, 43904, 21932, 21932, 21932],
    ["Same", "Merge Sort", "Same", 2000, 43904, 21932, 21932, 21932],
    ["Zig Zag", "Merge Sort", "Zig Zag", 2000, 43904, 21932, 21932, 21932],
    ["Sawtooth", "Merge Sort", "Sawtooth", 2000, 43904, 21932, 21932, 21932],
    ["Square", "Merge Sort", "Square", 2000, 43904, 21932, 21932, 21932],
    ["Zeroes", "Merge Sort", "Zeroes", 2000, 43904, 21932, 21932, 21932],
    ["Thousands", "Merge Sort", "Thousands", 2000, 43904, 21932, 21932, 21932],
    ["Millions", "Merge Sort", "Millions", 2000, 43904, 21932, 21932, 21932],
    ["Blender", "Merge Sort", "Blender", 2000, 43904, 21932, 21932, 21932],
    ["Random", "Merge Sort", "Random", 3000, 69808, 34652, 34652, 34652],
    ["Sorted", "Merge Sort", "Sorted", 3000, 69808, 34652, 34652, 34652],
    ["Half Sorted", "Merge Sort", "Half Sorted", 3000, 69808, 34652, 34652, 34652],
    ["Sorted Half", "Merge Sort", "Sorted Half", 3000, 69808, 34652, 34652, 34652],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 3000, 69808, 34652, 34652, 34652],
    ["Reversed", "Merge Sort", "Reversed", 3000, 69808, 34652, 34652, 34652],
    ["Same", "Merge Sort", "Same", 3000, 69808, 34652, 34652, 34652],
    ["Zig Zag", "Merge Sort", "Zig Zag", 3000, 69808, 34652, 34652, 34652],
    ["Sawtooth", "Merge Sort", "Sawtooth", 3000, 69808, 34652, 34652, 34652],
    ["Square", "Merge Sort", "Square", 3000, 69808, 34652, 34652, 34652],
    ["Zeroes", "Merge Sort", "Zeroes", 3000, 69808, 34652, 34652, 34652],
    ["Thousands", "Merge Sort", "Thousands", 3000, 69808, 34652, 34652, 34652],
    ["Millions", "Merge Sort", "Millions", 3000, 69808, 34652, 34652, 34652],
    ["Blender", "Merge Sort", "Blender", 3000, 69808, 34652, 34652, 34652],
    ["Random", "Merge Sort", "Random", 4000, 95808, 47863, 47863, 47863],
    ["Sorted", "Merge Sort", "Sorted", 4000, 95808, 47863, 47863, 47863],
    ["Half Sorted", "Merge Sort", "Half Sorted", 4000, 95808, 47863, 47863, 47863],
    ["Sorted Half", "Merge Sort", "Sorted Half", 4000, 95808, 47863, 47863, 47863],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 4000, 95808, 47863, 47863, 47863],
    ["Reversed", "Merge Sort", "Reversed", 4000, 95808, 47863, 47863, 47863],
    ["Same", "Merge Sort", "Same", 4000, 95808, 47863, 47863, 47863],
    ["Zig Zag", "Merge Sort", "Zig Zag", 4000, 95808, 47863, 47863, 47863],
    ["Sawtooth", "Merge Sort", "Sawtooth", 4000, 95808, 47863, 47863, 47863],
    ["Square", "Merge Sort", "Square", 4000, 95808, 47863, 47863, 47863],
    ["Zeroes", "Merge Sort", "Zeroes", 4000, 95808, 47863, 47863, 47863],
    ["Thousands", "Merge Sort", "Thousands", 4000, 95808, 47863, 47863, 47863],
    ["Millions", "Merge Sort", "Millions", 4000, 95808, 47863, 47863, 47863],
    ["Blender", "Merge Sort", "Blender", 4000, 95808, 47863, 47863, 47863],
    ["Random", "Merge Sort", "Random", 6000, 151616, 75304, 75304, 75304],
    ["Sorted", "Merge Sort", "Sorted", 6000, 151616, 75304, 75304, 75304],
    ["Half Sorted", "Merge Sort", "Half Sorted", 6000, 151616, 75304, 75304, 75304],
    ["Sorted Half", "Merge Sort", "Sorted Half", 6000, 151616, 75304, 75304, 75304],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 6000, 151616, 75304, 75304, 75304],
    ["Reversed", "Merge Sort", "Reversed", 6000, 151616, 75304, 75304, 75304],
    ["Same", "Merge Sort", "Same", 6000, 151616, 75304, 75304, 75304],
    ["Zig Zag", "Merge Sort", "Zig Zag", 6000, 151616, 75304, 75304, 75304],
    ["Sawtooth", "Merge Sort", "Sawtooth", 6000, 151616, 75304, 75304, 75304],
    ["Square", "Merge Sort", "Square", 6000, 151616, 75304, 75304, 75304],
    ["Zeroes", "Merge Sort", "Zeroes", 6000, 151616, 75304, 75304, 75304],
    ["Thousands", "Merge Sort", "Thousands", 6000, 151616, 75304, 75304, 75304],
    ["Millions", "Merge Sort", "Millions", 6000, 151616, 75304, 75304, 75304],
    ["Blender", "Merge Sort", "Blender", 6000, 151616, 75304, 75304, 75304],
    ["Random", "Merge Sort", "Random", 8000, 207616, 103726, 103726, 103726],
    ["Sorted", "Merge Sort", "Sorted", 8000, 207616, 103726, 103726, 103726],
    ["Half Sorted", "Merge Sort", "Half Sorted", 8000, 207616, 103726, 103726, 103726],
    ["Sorted Half", "Merge Sort", "Sorted Half", 8000, 207616, 103726, 103726, 103726],
    ["Semi Sorted", "Merge Sort", "Semi Sorted", 8000, 207616, 103726, 103726, 103726],
    ["Reversed", "Merge Sort", "Reversed", 8000, 207616, 103726, 103726, 103726],
    ["Same", "Merge Sort", "Same", 8000, 207616, 103726, 103726, 103726],
    ["Zig Zag", "Merge Sort", "Zig Zag", 8000, 207616, 103726, 103726, 103726],
    ["Sawtooth", "Merge Sort", "Sawtooth", 8000, 207616, 103726, 103726, 103726],
    ["Square", "Merge Sort", "Square", 8000, 207616, 103726, 103726, 103726],
    ["Zeroes", "Merge Sort", "Zeroes", 8000, 207616, 103726, 103726, 103726],
    ["Thousands", "Merge Sort", "Thousands", 8000, 207616, 103726, 103726, 103726],
    ["Millions", "Merge Sort", "Millions", 8000, 207616, 103726, 103726, 103726],
    ["Blender", "Merge Sort", "Blender", 8000, 207616, 103726, 103726, 103726],
  ],
  "heap-sort-various": [
    ["Random", "Heap Sort", "Random", 10, 38, 33, 33, 33],
    ["Sorted", "Heap Sort", "Sorted", 10, 42, 33, 33, 33],
    ["Reversed", "Heap Sort", "Reversed", 10, 33, 33, 33, 33],
    ["Millions", "Heap Sort", "Millions", 10, 38, 33, 33, 33],
    ["Zig Zag", "Heap Sort", "Zig Zag", 10, 40, 33, 33, 33],
    ["Sawtooth", "Heap Sort", "Sawtooth", 10, 40, 33, 33, 33],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 10, 42, 33, 33, 33],
    ["Random", "Heap Sort", "Random", 15, 59, 59, 59, 59],
    ["Sorted", "Heap Sort", "Sorted", 15, 68, 59, 59, 59],
    ["Reversed", "Heap Sort", "Reversed", 15, 55, 59, 59, 59],
    ["Millions", "Heap Sort", "Millions", 15, 65, 59, 59, 59],
    ["Zig Zag", "Heap Sort", "Zig Zag", 15, 61, 59, 59, 59],
    ["Sawtooth", "Heap Sort", "Sawtooth", 15, 62, 59, 59, 59],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 15, 68, 59, 59, 59],
    ["Random", "Heap Sort", "Random", 20, 96, 86, 86, 86],
    ["Sorted", "Heap Sort", "Sorted", 20, 102, 86, 86, 86],
    ["Reversed", "Heap Sort", "Reversed", 20, 84, 86, 86, 86],
    ["Millions", "Heap Sort", "Millions", 20, 92, 86, 86, 86],
    ["Zig Zag", "Heap Sort", "Zig Zag", 20, 93, 86, 86, 86],
    ["Sawtooth", "Heap Sort", "Sawtooth", 20, 96, 86, 86, 86],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 20, 98, 86, 86, 86],
    ["Random", "Heap Sort", "Random", 30, 153, 147, 147, 147],
    ["Sorted", "Heap Sort", "Sorted", 30, 168, 147, 147, 147],
    ["Reversed", "Heap Sort", "Reversed", 30, 139, 147, 147, 147],
    ["Millions", "Heap Sort", "Millions", 30, 151, 147, 147, 147],
    ["Zig Zag", "Heap Sort", "Zig Zag", 30, 157, 147, 147, 147],
    ["Sawtooth", "Heap Sort", "Sawtooth", 30, 160, 147, 147, 147],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 30, 157, 147, 147, 147],
    ["Random", "Heap Sort", "Random", 40, 218, 213, 213, 213],
    ["Sorted", "Heap Sort", "Sorted", 40, 244, 213, 213, 213],
    ["Reversed", "Heap Sort", "Reversed", 40, 196, 213, 213, 213],
    ["Millions", "Heap Sort", "Millions", 40, 226, 213, 213, 213],
    ["Zig Zag", "Heap Sort", "Zig Zag", 40, 228, 213, 213, 213],
    ["Sawtooth", "Heap Sort", "Sawtooth", 40, 227, 213, 213, 213],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 40, 236, 213, 213, 213],
    ["Random", "Heap Sort", "Random", 60, 364, 354, 354, 354],
    ["Sorted", "Heap Sort", "Sorted", 60, 398, 354, 354, 354],
    ["Reversed", "Heap Sort", "Reversed", 60, 326, 354, 354, 354],
    ["Millions", "Heap Sort", "Millions", 60, 361, 354, 354, 354],
    ["Zig Zag", "Heap Sort", "Zig Zag", 60, 371, 354, 354, 354],
    ["Sawtooth", "Heap Sort", "Sawtooth", 60, 362, 354, 354, 354],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 60, 380, 354, 354, 354],
    ["Random", "Heap Sort", "Random", 80, 520, 506, 506, 506],
    ["Sorted", "Heap Sort", "Sorted", 80, 568, 506, 506, 506],
    ["Reversed", "Heap Sort", "Reversed", 80, 468, 506, 506, 506],
    ["Millions", "Heap Sort", "Millions", 80, 530, 506, 506, 506],
    ["Zig Zag", "Heap Sort", "Zig Zag", 80, 532, 506, 506, 506],
    ["Sawtooth", "Heap Sort", "Sawtooth", 80, 534, 506, 506, 506],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 80, 542, 506, 506, 506],
    ["Random", "Heap Sort", "Random", 100, 669, 664, 664, 664],
    ["Sorted", "Heap Sort", "Sorted", 100, 742, 664, 664, 664],
    ["Reversed", "Heap Sort", "Reversed", 100, 618, 664, 664, 664],
    ["Millions", "Heap Sort", "Millions", 100, 668, 664, 664, 664],
    ["Zig Zag", "Heap Sort", "Zig Zag", 100, 695, 664, 664, 664],
    ["Sawtooth", "Heap Sort", "Sawtooth", 100, 646, 664, 664, 664],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 100, 715, 664, 664, 664],
    ["Random", "Heap Sort", "Random", 150, 1104, 1084, 1084, 1084],
    ["Sorted", "Heap Sort", "Sorted", 150, 1206, 1084, 1084, 1084],
    ["Reversed", "Heap Sort", "Reversed", 150, 1001, 1084, 1084, 1084],
    ["Millions", "Heap Sort", "Millions", 150, 1110, 1084, 1084, 1084],
    ["Zig Zag", "Heap Sort", "Zig Zag", 150, 1121, 1084, 1084, 1084],
    ["Sawtooth", "Heap Sort", "Sawtooth", 150, 1100, 1084, 1084, 1084],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 150, 1136, 1084, 1084, 1084],
    ["Random", "Heap Sort", "Random", 200, 1567, 1529, 1529, 1529],
    ["Sorted", "Heap Sort", "Sorted", 200, 1690, 1529, 1529, 1529],
    ["Reversed", "Heap Sort", "Reversed", 200, 1408, 1529, 1529, 1529],
    ["Millions", "Heap Sort", "Millions", 200, 1562, 1529, 1529, 1529],
    ["Zig Zag", "Heap Sort", "Zig Zag", 200, 1576, 1529, 1529, 1529],
    ["Sawtooth", "Heap Sort", "Sawtooth", 200, 1565, 1529, 1529, 1529],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 200, 1609, 1529, 1529, 1529],
    ["Random", "Heap Sort", "Random", 300, 2515, 2469, 2469, 2469],
    ["Sorted", "Heap Sort", "Sorted", 300, 2712, 2469, 2469, 2469],
    ["Reversed", "Heap Sort", "Reversed", 300, 2296, 2469, 2469, 2469],
    ["Millions", "Heap Sort", "Millions", 300, 2530, 2469, 2469, 2469],
    ["Zig Zag", "Heap Sort", "Zig Zag", 300, 2533, 2469, 2469, 2469],
    ["Sawtooth", "Heap Sort", "Sawtooth", 300, 2518, 2469, 2469, 2469],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 300, 2587, 2469, 2469, 2469],
    ["Random", "Heap Sort", "Random", 400, 3512, 3458, 3458, 3458],
    ["Sorted", "Heap Sort", "Sorted", 400, 3774, 3458, 3458, 3458],
    ["Reversed", "Heap Sort", "Reversed", 400, 3224, 3458, 3458, 3458],
    ["Millions", "Heap Sort", "Millions", 400, 3504, 3458, 3458, 3458],
    ["Zig Zag", "Heap Sort", "Zig Zag", 400, 3558, 3458, 3458, 3458],
    ["Sawtooth", "Heap Sort", "Sawtooth", 400, 3402, 3458, 3458, 3458],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 400, 3645, 3458, 3458, 3458],
    ["Random", "Heap Sort", "Random", 600, 5610, 5537, 5537, 5537],
    ["Sorted", "Heap Sort", "Sorted", 600, 6006, 5537, 5537, 5537],
    ["Reversed", "Heap Sort", "Reversed", 600, 5174, 5537, 5537, 5537],
    ["Millions", "Heap Sort", "Millions", 600, 5599, 5537, 5537, 5537],
    ["Zig Zag", "Heap Sort", "Zig Zag", 600, 5660, 5537, 5537, 5537],
    ["Sawtooth", "Heap Sort", "Sawtooth", 600, 5582, 5537, 5537, 5537],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 600, 5777, 5537, 5537, 5537],
    ["Random", "Heap Sort", "Random", 800, 7810, 7715, 7715, 7715],
    ["Sorted", "Heap Sort", "Sorted", 800, 8356, 7715, 7715, 7715],
    ["Reversed", "Heap Sort", "Reversed", 800, 7244, 7715, 7715, 7715],
    ["Millions", "Heap Sort", "Millions", 800, 7825, 7715, 7715, 7715],
    ["Zig Zag", "Heap Sort", "Zig Zag", 800, 7896, 7715, 7715, 7715],
    ["Sawtooth", "Heap Sort", "Sawtooth", 800, 7742, 7715, 7715, 7715],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 800, 8130, 7715, 7715, 7715],
    ["Random", "Heap Sort", "Random", 1000, 10055, 9966, 9966, 9966],
    ["Sorted", "Heap Sort", "Sorted", 1000, 10710, 9966, 9966, 9966],
    ["Reversed", "Heap Sort", "Reversed", 1000, 9318, 9966, 9966, 9966],
    ["Millions", "Heap Sort", "Millions", 1000, 10103, 9966, 9966, 9966],
    ["Zig Zag", "Heap Sort", "Zig Zag", 1000, 10180, 9966, 9966, 9966],
    ["Sawtooth", "Heap Sort", "Sawtooth", 1000, 10016, 9966, 9966, 9966],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 1000, 10312, 9966, 9966, 9966],
    ["Random", "Heap Sort", "Random", 1500, 15993, 15826, 15826, 15826],
    ["Sorted", "Heap Sort", "Sorted", 1500, 17050, 15826, 15826, 15826],
    ["Reversed", "Heap Sort", "Reversed", 1500, 15024, 15826, 15826, 15826],
    ["Millions", "Heap Sort", "Millions", 1500, 16030, 15826, 15826, 15826],
    ["Zig Zag", "Heap Sort", "Zig Zag", 1500, 16171, 15826, 15826, 15826],
    ["Sawtooth", "Heap Sort", "Sawtooth", 1500, 15921, 15826, 15826, 15826],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 1500, 16365, 15826, 15826, 15826],
    ["Random", "Heap Sort", "Random", 2000, 22175, 21932, 21932, 21932],
    ["Sorted", "Heap Sort", "Sorted", 2000, 23302, 21932, 21932, 21932],
    ["Reversed", "Heap Sort", "Reversed", 2000, 20710, 21932, 21932, 21932],
    ["Millions", "Heap Sort", "Millions", 2000, 22182, 21932, 21932, 21932],
    ["Zig Zag", "Heap Sort", "Zig Zag", 2000, 22366, 21932, 21932, 21932],
    ["Sawtooth", "Heap Sort", "Sawtooth", 2000, 22023, 21932, 21932, 21932],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 2000, 22669, 21932, 21932, 21932],
    ["Random", "Heap Sort", "Random", 3000, 35055, 34652, 34652, 34652],
    ["Sorted", "Heap Sort", "Sorted", 3000, 37254, 34652, 34652, 34652],
    ["Reversed", "Heap Sort", "Reversed", 3000, 32752, 34652, 34652, 34652],
    ["Millions", "Heap Sort", "Millions", 3000, 35167, 34652, 34652, 34652],
    ["Zig Zag", "Heap Sort", "Zig Zag", 3000, 35302, 34652, 34652, 34652],
    ["Sawtooth", "Heap Sort", "Sawtooth", 3000, 34697, 34652, 34652, 34652],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 3000, 36072, 34652, 34652, 34652],
    ["Random", "Heap Sort", "Random", 4000, 48387, 47863, 47863, 47863],
    ["Sorted", "Heap Sort", "Sorted", 4000, 51144, 47863, 47863, 47863],
    ["Reversed", "Heap Sort", "Reversed", 4000, 45438, 47863, 47863, 47863],
    ["Millions", "Heap Sort", "Millions", 4000, 48317, 47863, 47863, 47863],
    ["Zig Zag", "Heap Sort", "Zig Zag", 4000, 48658, 47863, 47863, 47863],
    ["Sawtooth", "Heap Sort", "Sawtooth", 4000, 47791, 47863, 47863, 47863],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 4000, 49459, 47863, 47863, 47863],
    ["Random", "Heap Sort", "Random", 6000, 76147, 75304, 75304, 75304],
    ["Sorted", "Heap Sort", "Sorted", 6000, 80830, 75304, 75304, 75304],
    ["Reversed", "Heap Sort", "Reversed", 6000, 71600, 75304, 75304, 75304],
    ["Millions", "Heap Sort", "Millions", 6000, 76214, 75304, 75304, 75304],
    ["Zig Zag", "Heap Sort", "Zig Zag", 6000, 76752, 75304, 75304, 75304],
    ["Sawtooth", "Heap Sort", "Sawtooth", 6000, 75325, 75304, 75304, 75304],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 6000, 76168, 75304, 75304, 75304],
    ["Random", "Heap Sort", "Random", 8000, 104694, 103726, 103726, 103726],
    ["Sorted", "Heap Sort", "Sorted", 8000, 110640, 103726, 103726, 103726],
    ["Reversed", "Heap Sort", "Reversed", 8000, 98778, 103726, 103726, 103726],
    ["Millions", "Heap Sort", "Millions", 8000, 104675, 103726, 103726, 103726],
    ["Zig Zag", "Heap Sort", "Zig Zag", 8000, 105516, 103726, 103726, 103726],
    ["Sawtooth", "Heap Sort", "Sawtooth", 8000, 103498, 103726, 103726, 103726],
    ["Semi Sorted", "Heap Sort", "Semi Sorted", 8000, 105383, 103726, 103726, 103726],
  ],
  "heap-sort-identical": [
    ["Same", "Heap Sort", "Same", 10, 21, 33, 33, 33],
    ["Zeroes", "Heap Sort", "Zeroes", 10, 21, 33, 33, 33],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 10, 27, 33, 33, 33],
    ["Same", "Heap Sort", "Same", 15, 30, 59, 59, 59],
    ["Zeroes", "Heap Sort", "Zeroes", 15, 30, 59, 59, 59],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 14, 39, 53, 53, 53],
    ["Same", "Heap Sort", "Same", 20, 41, 86, 86, 86],
    ["Zeroes", "Heap Sort", "Zeroes", 20, 41, 86, 86, 86],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 20, 60, 86, 86, 86],
    ["Same", "Heap Sort", "Same", 30, 61, 147, 147, 147],
    ["Zeroes", "Heap Sort", "Zeroes", 30, 61, 147, 147, 147],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 30, 95, 147, 147, 147],
    ["Same", "Heap Sort", "Same", 40, 81, 213, 213, 213],
    ["Zeroes", "Heap Sort", "Zeroes", 40, 81, 213, 213, 213],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 40, 135, 213, 213, 213],
    ["Same", "Heap Sort", "Same", 60, 121, 354, 354, 354],
    ["Zeroes", "Heap Sort", "Zeroes", 60, 121, 354, 354, 354],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 60, 215, 354, 354, 354],
    ["Same", "Heap Sort", "Same", 80, 161, 506, 506, 506],
    ["Zeroes", "Heap Sort", "Zeroes", 80, 161, 506, 506, 506],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 80, 304, 506, 506, 506],
    ["Same", "Heap Sort", "Same", 100, 201, 664, 664, 664],
    ["Zeroes", "Heap Sort", "Zeroes", 100, 201, 664, 664, 664],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 100, 394, 664, 664, 664],
    ["Same", "Heap Sort", "Same", 150, 301, 1084, 1084, 1084],
    ["Zeroes", "Heap Sort", "Zeroes", 150, 301, 1084, 1084, 1084],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 150, 631, 1084, 1084, 1084],
    ["Same", "Heap Sort", "Same", 200, 401, 1529, 1529, 1529],
    ["Zeroes", "Heap Sort", "Zeroes", 200, 401, 1529, 1529, 1529],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 200, 881, 1529, 1529, 1529],
    ["Same", "Heap Sort", "Same", 300, 601, 2469, 2469, 2469],
    ["Zeroes", "Heap Sort", "Zeroes", 300, 601, 2469, 2469, 2469],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 300, 1404, 2469, 2469, 2469],
    ["Same", "Heap Sort", "Same", 400, 801, 3458, 3458, 3458],
    ["Zeroes", "Heap Sort", "Zeroes", 400, 801, 3458, 3458, 3458],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 400, 1954, 3458, 3458, 3458],
    ["Same", "Heap Sort", "Same", 600, 1201, 5537, 5537, 5537],
    ["Zeroes", "Heap Sort", "Zeroes", 600, 1201, 5537, 5537, 5537],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 600, 3099, 5537, 5537, 5537],
    ["Same", "Heap Sort", "Same", 800, 1601, 7715, 7715, 7715],
    ["Zeroes", "Heap Sort", "Zeroes", 800, 1601, 7715, 7715, 7715],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 800, 4299, 7715, 7715, 7715],
    ["Same", "Heap Sort", "Same", 1000, 2001, 9966, 9966, 9966],
    ["Zeroes", "Heap Sort", "Zeroes", 1000, 2001, 9966, 9966, 9966],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 1000, 5499, 9966, 9966, 9966],
    ["Same", "Heap Sort", "Same", 1500, 3001, 15826, 15826, 15826],
    ["Zeroes", "Heap Sort", "Zeroes", 1500, 3001, 15826, 15826, 15826],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 1500, 8738, 15826, 15826, 15826],
    ["Same", "Heap Sort", "Same", 2000, 4001, 21932, 21932, 21932],
    ["Zeroes", "Heap Sort", "Zeroes", 2000, 4001, 21932, 21932, 21932],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 2000, 11988, 21932, 21932, 21932],
    ["Same", "Heap Sort", "Same", 3000, 6001, 34652, 34652, 34652],
    ["Zeroes", "Heap Sort", "Zeroes", 3000, 6001, 34652, 34652, 34652],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 3000, 18965, 34652, 34652, 34652],
    ["Same", "Heap Sort", "Same", 4000, 8001, 47863, 47863, 47863],
    ["Zeroes", "Heap Sort", "Zeroes", 4000, 8001, 47863, 47863, 47863],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 4000, 25965, 47863, 47863, 47863],
    ["Same", "Heap Sort", "Same", 6000, 12001, 75304, 75304, 75304],
    ["Zeroes", "Heap Sort", "Zeroes", 6000, 12001, 75304, 75304, 75304],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 6000, 40918, 75304, 75304, 75304],
    ["Same", "Heap Sort", "Same", 8000, 16001, 103726, 103726, 103726],
    ["Zeroes", "Heap Sort", "Zeroes", 8000, 16001, 103726, 103726, 103726],
    ["Same - Zeroes", "Heap Sort", "Same - Zeroes", 8000, 55918, 103726, 103726, 103726],
  ],
  "tim-sort-identical": [
    ["Zeroes", "Tim Sort", "Zeroes", 10, 8, 10, 33, 33],
    ["Same", "Tim Sort", "Same", 10, 8, 10, 33, 33],
    ["Zeroes", "Tim Sort", "Zeroes", 15, 13, 15, 59, 59],
    ["Same", "Tim Sort", "Same", 15, 13, 15, 59, 59],
    ["Zeroes", "Tim Sort", "Zeroes", 20, 18, 20, 86, 86],
    ["Same", "Tim Sort", "Same", 20, 18, 20, 86, 86],
    ["Zeroes", "Tim Sort", "Zeroes", 30, 28, 30, 147, 147],
    ["Same", "Tim Sort", "Same", 30, 28, 30, 147, 147],
    ["Zeroes", "Tim Sort", "Zeroes", 40, 40, 40, 213, 213],
    ["Same", "Tim Sort", "Same", 40, 40, 40, 213, 213],
    ["Zeroes", "Tim Sort", "Zeroes", 60, 60, 60, 354, 354],
    ["Same", "Tim Sort", "Same", 60, 60, 60, 354, 354],
    ["Zeroes", "Tim Sort", "Zeroes", 80, 81, 80, 506, 506],
    ["Same", "Tim Sort", "Same", 80, 81, 80, 506, 506],
    ["Zeroes", "Tim Sort", "Zeroes", 100, 101, 100, 664, 664],
    ["Same", "Tim Sort", "Same", 100, 101, 100, 664, 664],
    ["Zeroes", "Tim Sort", "Zeroes", 150, 152, 150, 1084, 1084],
    ["Same", "Tim Sort", "Same", 150, 152, 150, 1084, 1084],
    ["Zeroes", "Tim Sort", "Zeroes", 200, 202, 200, 1529, 1529],
    ["Same", "Tim Sort", "Same", 200, 202, 200, 1529, 1529],
    ["Zeroes", "Tim Sort", "Zeroes", 300, 303, 300, 2469, 2469],
    ["Same", "Tim Sort", "Same", 300, 303, 300, 2469, 2469],
    ["Zeroes", "Tim Sort", "Zeroes", 400, 403, 400, 3458, 3458],
    ["Same", "Tim Sort", "Same", 400, 403, 400, 3458, 3458],
    ["Zeroes", "Tim Sort", "Zeroes", 600, 604, 600, 5537, 5537],
    ["Same", "Tim Sort", "Same", 600, 604, 600, 5537, 5537],
    ["Zeroes", "Tim Sort", "Zeroes", 800, 804, 800, 7715, 7715],
    ["Same", "Tim Sort", "Same", 800, 804, 800, 7715, 7715],
    ["Zeroes", "Tim Sort", "Zeroes", 1000, 1004, 1000, 9966, 9966],
    ["Same", "Tim Sort", "Same", 1000, 1004, 1000, 9966, 9966],
    ["Zeroes", "Tim Sort", "Zeroes", 1500, 1505, 1500, 15826, 15826],
    ["Same", "Tim Sort", "Same", 1500, 1505, 1500, 15826, 15826],
    ["Zeroes", "Tim Sort", "Zeroes", 2000, 2005, 2000, 21932, 21932],
    ["Same", "Tim Sort", "Same", 2000, 2005, 2000, 21932, 21932],
    ["Zeroes", "Tim Sort", "Zeroes", 3000, 3006, 3000, 34652, 34652],
    ["Same", "Tim Sort", "Same", 3000, 3006, 3000, 34652, 34652],
    ["Zeroes", "Tim Sort", "Zeroes", 4000, 4006, 4000, 47863, 47863],
    ["Same", "Tim Sort", "Same", 4000, 4006, 4000, 47863, 47863],
    ["Zeroes", "Tim Sort", "Zeroes", 6000, 6007, 6000, 75304, 75304],
    ["Same", "Tim Sort", "Same", 6000, 6007, 6000, 75304, 75304],
    ["Zeroes", "Tim Sort", "Zeroes", 8000, 8007, 8000, 103726, 103726],
    ["Same", "Tim Sort", "Same", 8000, 8007, 8000, 103726, 103726],
  ],
  "tim-sort-sorted": [
    ["Sorted", "Tim Sort", "Sorted", 10, 8, 10, 33, 33],
    ["Reversed", "Tim Sort", "Reversed", 10, 13, 10, 33, 33],
    ["Square", "Tim Sort", "Square", 10, 27, 10, 33, 33],
    ["Sorted", "Tim Sort", "Sorted", 15, 72, 15, 59, 59],
    ["Reversed", "Tim Sort", "Reversed", 15, 97, 15, 59, 59],
    ["Square", "Tim Sort", "Square", 15, 66, 15, 59, 59],
    ["Sorted", "Tim Sort", "Sorted", 20, 140, 20, 86, 86],
    ["Reversed", "Tim Sort", "Reversed", 20, 124, 20, 86, 86],
    ["Square", "Tim Sort", "Square", 20, 114, 20, 86, 86],
    ["Sorted", "Tim Sort", "Sorted", 30, 267, 30, 147, 147],
    ["Reversed", "Tim Sort", "Reversed", 30, 172, 30, 147, 147],
    ["Square", "Tim Sort", "Square", 30, 221, 30, 147, 147],
    ["Sorted", "Tim Sort", "Sorted", 40, 202, 40, 213, 213],
    ["Reversed", "Tim Sort", "Reversed", 40, 103, 40, 213, 213],
    ["Square", "Tim Sort", "Square", 40, 243, 40, 213, 213],
    ["Sorted", "Tim Sort", "Sorted", 60, 349, 60, 354, 354],
    ["Reversed", "Tim Sort", "Reversed", 60, 156, 60, 354, 354],
    ["Square", "Tim Sort", "Square", 60, 517, 60, 354, 354],
    ["Sorted", "Tim Sort", "Sorted", 80, 292, 80, 506, 506],
    ["Reversed", "Tim Sort", "Reversed", 80, 208, 80, 506, 506],
    ["Square", "Tim Sort", "Square", 80, 626, 80, 506, 506],
    ["Sorted", "Tim Sort", "Sorted", 100, 380, 100, 664, 664],
    ["Reversed", "Tim Sort", "Reversed", 100, 260, 100, 664, 664],
    ["Square", "Tim Sort", "Square", 100, 834, 100, 664, 664],
    ["Sorted", "Tim Sort", "Sorted", 150, 598, 150, 1084, 1084],
    ["Reversed", "Tim Sort", "Reversed", 150, 466, 150, 1084, 1084],
    ["Square", "Tim Sort", "Square", 150, 1204, 150, 1084, 1084],
    ["Sorted", "Tim Sort", "Sorted", 200, 816, 200, 1529, 1529],
    ["Reversed", "Tim Sort", "Reversed", 200, 546, 200, 1529, 1529],
    ["Square", "Tim Sort", "Square", 200, 1894, 200, 1529, 1529],
    ["Sorted", "Tim Sort", "Sorted", 300, 1012, 300, 2469, 2469],
    ["Reversed", "Tim Sort", "Reversed", 300, 820, 300, 2469, 2469],
    ["Square", "Tim Sort", "Square", 300, 2585, 300, 2469, 2469],
    ["Sorted", "Tim Sort", "Sorted", 400, 1317, 400, 3458, 3458],
    ["Reversed", "Tim Sort", "Reversed", 400, 1092, 400, 3458, 3458],
    ["Square", "Tim Sort", "Square", 400, 3785, 400, 3458, 3458],
    ["Sorted", "Tim Sort", "Sorted", 600, 1763, 600, 5537, 5537],
    ["Reversed", "Tim Sort", "Reversed", 600, 1636, 600, 5537, 5537],
    ["Square", "Tim Sort", "Square", 600, 3202, 600, 5537, 5537],
    ["Sorted", "Tim Sort", "Sorted", 800, 2318, 800, 7715, 7715],
    ["Reversed", "Tim Sort", "Reversed", 800, 2178, 800, 7715, 7715],
    ["Square", "Tim Sort", "Square", 800, 4471, 800, 7715, 7715],
    ["Sorted", "Tim Sort", "Sorted", 1000, 2876, 1000, 9966, 9966],
    ["Reversed", "Tim Sort", "Reversed", 1000, 2714, 1000, 9966, 9966],
    ["Square", "Tim Sort", "Square", 1000, 9058, 1000, 9966, 9966],
    ["Sorted", "Tim Sort", "Sorted", 1500, 5510, 1500, 15826, 15826],
    ["Reversed", "Tim Sort", "Reversed", 1500, 4634, 1500, 15826, 15826],
    ["Square", "Tim Sort", "Square", 1500, 8446, 1500, 15826, 15826],
    ["Sorted", "Tim Sort", "Sorted", 2000, 7370, 2000, 21932, 21932],
    ["Reversed", "Tim Sort", "Reversed", 2000, 5458, 2000, 21932, 21932],
    ["Square", "Tim Sort", "Square", 2000, 12252, 2000, 21932, 21932],
    ["Sorted", "Tim Sort", "Sorted", 3000, 9791, 3000, 34652, 34652],
    ["Reversed", "Tim Sort", "Reversed", 3000, 8181, 3000, 34652, 34652],
    ["Square", "Tim Sort", "Square", 3000, 17492, 3000, 34652, 34652],
    ["Sorted", "Tim Sort", "Sorted", 4000, 12375, 4000, 47863, 47863],
    ["Reversed", "Tim Sort", "Reversed", 4000, 10903, 4000, 47863, 47863],
    ["Square", "Tim Sort", "Square", 4000, 25506, 4000, 47863, 47863],
    ["Sorted", "Tim Sort", "Sorted", 6000, 17298, 6000, 75304, 75304],
    ["Reversed", "Tim Sort", "Reversed", 6000, 16346, 6000, 75304, 75304],
    ["Square", "Tim Sort", "Square", 6000, 37562, 6000, 75304, 75304],
    ["Sorted", "Tim Sort", "Sorted", 8000, 22384, 8000, 103726, 103726],
    ["Reversed", "Tim Sort", "Reversed", 8000, 21788, 8000, 103726, 103726],
    ["Square", "Tim Sort", "Square", 8000, 50683, 8000, 103726, 103726],
  ],
  "tim-sort-semi-sorted": [
    ["Sorted Half", "Tim Sort", "Sorted Half", 10, 29, 10, 33, 33],
    ["Half Sorted", "Tim Sort", "Half Sorted", 10, 30, 10, 33, 33],
    ["Zig Zag", "Tim Sort", "Zig Zag", 10, 43, 10, 33, 33],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 10, 8, 10, 33, 33],
    ["Sawtooth", "Tim Sort", "Sawtooth", 10, 36, 10, 33, 33],
    ["Sorted Half", "Tim Sort", "Sorted Half", 14, 61, 14, 53, 53],
    ["Half Sorted", "Tim Sort", "Half Sorted", 14, 50, 14, 53, 53],
    ["Zig Zag", "Tim Sort", "Zig Zag", 15, 88, 15, 59, 59],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 15, 64, 15, 59, 59],
    ["Sawtooth", "Tim Sort", "Sawtooth", 15, 75, 15, 59, 59],
    ["Sorted Half", "Tim Sort", "Sorted Half", 20, 141, 20, 86, 86],
    ["Half Sorted", "Tim Sort", "Half Sorted", 20, 129, 20, 86, 86],
    ["Zig Zag", "Tim Sort", "Zig Zag", 20, 168, 20, 86, 86],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 20, 150, 20, 86, 86],
    ["Sawtooth", "Tim Sort", "Sawtooth", 20, 147, 20, 86, 86],
    ["Sorted Half", "Tim Sort", "Sorted Half", 30, 311, 30, 147, 147],
    ["Half Sorted", "Tim Sort", "Half Sorted", 30, 303, 30, 147, 147],
    ["Zig Zag", "Tim Sort", "Zig Zag", 30, 336, 30, 147, 147],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 30, 303, 30, 147, 147],
    ["Sawtooth", "Tim Sort", "Sawtooth", 30, 301, 30, 147, 147],
    ["Sorted Half", "Tim Sort", "Sorted Half", 40, 360, 40, 213, 213],
    ["Half Sorted", "Tim Sort", "Half Sorted", 40, 356, 40, 213, 213],
    ["Zig Zag", "Tim Sort", "Zig Zag", 40, 381, 40, 213, 213],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 40, 326, 40, 213, 213],
    ["Sawtooth", "Tim Sort", "Sawtooth", 40, 361, 40, 213, 213],
    ["Sorted Half", "Tim Sort", "Sorted Half", 60, 630, 60, 354, 354],
    ["Half Sorted", "Tim Sort", "Half Sorted", 60, 714, 60, 354, 354],
    ["Zig Zag", "Tim Sort", "Zig Zag", 60, 768, 60, 354, 354],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 60, 737, 60, 354, 354],
    ["Sawtooth", "Tim Sort", "Sawtooth", 60, 676, 60, 354, 354],
    ["Sorted Half", "Tim Sort", "Sorted Half", 80, 673, 80, 506, 506],
    ["Half Sorted", "Tim Sort", "Half Sorted", 80, 700, 80, 506, 506],
    ["Zig Zag", "Tim Sort", "Zig Zag", 80, 870, 80, 506, 506],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 80, 768, 80, 506, 506],
    ["Sawtooth", "Tim Sort", "Sawtooth", 80, 796, 80, 506, 506],
    ["Sorted Half", "Tim Sort", "Sorted Half", 100, 932, 100, 664, 664],
    ["Half Sorted", "Tim Sort", "Half Sorted", 100, 1000, 100, 664, 664],
    ["Zig Zag", "Tim Sort", "Zig Zag", 100, 1254, 100, 664, 664],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 100, 1182, 100, 664, 664],
    ["Sawtooth", "Tim Sort", "Sawtooth", 100, 1093, 100, 664, 664],
    ["Sorted Half", "Tim Sort", "Sorted Half", 150, 1294, 150, 1084, 1084],
    ["Half Sorted", "Tim Sort", "Half Sorted", 150, 1255, 150, 1084, 1084],
    ["Zig Zag", "Tim Sort", "Zig Zag", 150, 1843, 150, 1084, 1084],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 150, 1682, 150, 1084, 1084],
    ["Sawtooth", "Tim Sort", "Sawtooth", 150, 1729, 150, 1084, 1084],
    ["Sorted Half", "Tim Sort", "Sorted Half", 200, 1930, 200, 1529, 1529],
    ["Half Sorted", "Tim Sort", "Half Sorted", 200, 1887, 200, 1529, 1529],
    ["Zig Zag", "Tim Sort", "Zig Zag", 200, 2891, 200, 1529, 1529],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 200, 2404, 200, 1529, 1529],
    ["Sawtooth", "Tim Sort", "Sawtooth", 200, 2709, 200, 1529, 1529],
    ["Sorted Half", "Tim Sort", "Sorted Half", 300, 3010, 300, 2469, 2469],
    ["Half Sorted", "Tim Sort", "Half Sorted", 300, 2806, 300, 2469, 2469],
    ["Zig Zag", "Tim Sort", "Zig Zag", 300, 4106, 300, 2469, 2469],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 300, 3917, 300, 2469, 2469],
    ["Sawtooth", "Tim Sort", "Sawtooth", 300, 3941, 300, 2469, 2469],
    ["Sorted Half", "Tim Sort", "Sorted Half", 400, 4272, 400, 3458, 3458],
    ["Half Sorted", "Tim Sort", "Half Sorted", 400, 4224, 400, 3458, 3458],
    ["Zig Zag", "Tim Sort", "Zig Zag", 400, 6177, 400, 3458, 3458],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 400, 5597, 400, 3458, 3458],
    ["Sawtooth", "Tim Sort", "Sawtooth", 400, 5861, 400, 3458, 3458],
    ["Sorted Half", "Tim Sort", "Sorted Half", 600, 6096, 600, 5537, 5537],
    ["Half Sorted", "Tim Sort", "Half Sorted", 600, 5950, 600, 5537, 5537],
    ["Zig Zag", "Tim Sort", "Zig Zag", 600, 8728, 600, 5537, 5537],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 600, 8447, 600, 5537, 5537],
    ["Sawtooth", "Tim Sort", "Sawtooth", 600, 8000, 600, 5537, 5537],
    ["Sorted Half", "Tim Sort", "Sorted Half", 800, 8790, 800, 7715, 7715],
    ["Half Sorted", "Tim Sort", "Half Sorted", 800, 8755, 800, 7715, 7715],
    ["Zig Zag", "Tim Sort", "Zig Zag", 800, 13137, 800, 7715, 7715],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 800, 12375, 800, 7715, 7715],
    ["Sawtooth", "Tim Sort", "Sawtooth", 800, 12039, 800, 7715, 7715],
    ["Sorted Half", "Tim Sort", "Sorted Half", 1000, 11992, 1000, 9966, 9966],
    ["Half Sorted", "Tim Sort", "Half Sorted", 1000, 12012, 1000, 9966, 9966],
    ["Zig Zag", "Tim Sort", "Zig Zag", 1000, 18658, 1000, 9966, 9966],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 1000, 17217, 1000, 9966, 9966],
    ["Sawtooth", "Tim Sort", "Sawtooth", 1000, 16108, 1000, 9966, 9966],
    ["Sorted Half", "Tim Sort", "Sorted Half", 1500, 17036, 1500, 15826, 15826],
    ["Half Sorted", "Tim Sort", "Half Sorted", 1500, 17100, 1500, 15826, 15826],
    ["Zig Zag", "Tim Sort", "Zig Zag", 1500, 26717, 1500, 15826, 15826],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 1500, 25160, 1500, 15826, 15826],
    ["Sawtooth", "Tim Sort", "Sawtooth", 1500, 22158, 1500, 15826, 15826],
    ["Sorted Half", "Tim Sort", "Sorted Half", 2000, 25304, 2000, 21932, 21932],
    ["Half Sorted", "Tim Sort", "Half Sorted", 2000, 25092, 2000, 21932, 21932],
    ["Zig Zag", "Tim Sort", "Zig Zag", 2000, 40203, 2000, 21932, 21932],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 2000, 37698, 2000, 21932, 21932],
    ["Sawtooth", "Tim Sort", "Sawtooth", 2000, 33396, 2000, 21932, 21932],
    ["Sorted Half", "Tim Sort", "Sorted Half", 3000, 36580, 3000, 34652, 34652],
    ["Half Sorted", "Tim Sort", "Half Sorted", 3000, 37349, 3000, 34652, 34652],
    ["Zig Zag", "Tim Sort", "Zig Zag", 3000, 56677, 3000, 34652, 34652],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 3000, 55464, 3000, 34652, 34652],
    ["Sawtooth", "Tim Sort", "Sawtooth", 3000, 42757, 3000, 34652, 34652],
    ["Sorted Half", "Tim Sort", "Sorted Half", 4000, 53480, 4000, 47863, 47863],
    ["Half Sorted", "Tim Sort", "Half Sorted", 4000, 54813, 4000, 47863, 47863],
    ["Zig Zag", "Tim Sort", "Zig Zag", 4000, 84911, 4000, 47863, 47863],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 4000, 81307, 4000, 47863, 47863],
    ["Sawtooth", "Tim Sort", "Sawtooth", 4000, 66342, 4000, 47863, 47863],
    ["Sorted Half", "Tim Sort", "Sorted Half", 6000, 76389, 6000, 75304, 75304],
    ["Half Sorted", "Tim Sort", "Half Sorted", 6000, 78345, 6000, 75304, 75304],
    ["Zig Zag", "Tim Sort", "Zig Zag", 6000, 117368, 6000, 75304, 75304],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 6000, 120171, 6000, 75304, 75304],
    ["Sawtooth", "Tim Sort", "Sawtooth", 6000, 81244, 6000, 75304, 75304],
    ["Sorted Half", "Tim Sort", "Sorted Half", 8000, 111182, 8000, 103726, 103726],
    ["Half Sorted", "Tim Sort", "Half Sorted", 8000, 113964, 8000, 103726, 103726],
    ["Zig Zag", "Tim Sort", "Zig Zag", 8000, 175178, 8000, 103726, 103726],
    ["Semi Sorted", "Tim Sort", "Semi Sorted", 8000, 177659, 8000, 103726, 103726],
    ["Sawtooth", "Tim Sort", "Sawtooth", 8000, 116802, 8000, 103726, 103726],
  ],
  "tim-sort-blended": [
    ["Blender 0", "Tim Sort", "Blender 0", 10, 46, 10, 33, 33],
    ["Blender 1", "Tim Sort", "Blender 1", 10, 43, 10, 33, 33],
    ["Blender 2", "Tim Sort", "Blender 2", 10, 46, 10, 33, 33],
    ["Blender 0", "Tim Sort", "Blender 0", 15, 89, 15, 59, 59],
    ["Blender 1", "Tim Sort", "Blender 1", 15, 89, 15, 59, 59],
    ["Blender 2", "Tim Sort", "Blender 2", 15, 89, 15, 59, 59],
    ["Blender 0", "Tim Sort", "Blender 0", 20, 125, 20, 86, 86],
    ["Blender 1", "Tim Sort", "Blender 1", 20, 179, 20, 86, 86],
    ["Blender 2", "Tim Sort", "Blender 2", 20, 132, 20, 86, 86],
    ["Blender 0", "Tim Sort", "Blender 0", 30, 379, 30, 147, 147],
    ["Blender 1", "Tim Sort", "Blender 1", 30, 390, 30, 147, 147],
    ["Blender 2", "Tim Sort", "Blender 2", 30, 292, 30, 147, 147],
    ["Blender 0", "Tim Sort", "Blender 0", 40, 406, 40, 213, 213],
    ["Blender 1", "Tim Sort", "Blender 1", 40, 413, 40, 213, 213],
    ["Blender 2", "Tim Sort", "Blender 2", 40, 382, 40, 213, 213],
    ["Blender 0", "Tim Sort", "Blender 0", 60, 826, 60, 354, 354],
    ["Blender 1", "Tim Sort", "Blender 1", 60, 693, 60, 354, 354],
    ["Blender 2", "Tim Sort", "Blender 2", 60, 816, 60, 354, 354],
    ["Blender 0", "Tim Sort", "Blender 0", 80, 942, 80, 506, 506],
    ["Blender 1", "Tim Sort", "Blender 1", 80, 836, 80, 506, 506],
    ["Blender 2", "Tim Sort", "Blender 2", 80, 869, 80, 506, 506],
    ["Blender 0", "Tim Sort", "Blender 0", 100, 1218, 100, 664, 664],
    ["Blender 1", "Tim Sort", "Blender 1", 100, 1260, 100, 664, 664],
    ["Blender 2", "Tim Sort", "Blender 2", 100, 1221, 100, 664, 664],
    ["Blender 0", "Tim Sort", "Blender 0", 150, 1731, 150, 1084, 1084],
    ["Blender 1", "Tim Sort", "Blender 1", 150, 1787, 150, 1084, 1084],
    ["Blender 2", "Tim Sort", "Blender 2", 150, 1654, 150, 1084, 1084],
    ["Blender 0", "Tim Sort", "Blender 0", 200, 2667, 200, 1529, 1529],
    ["Blender 1", "Tim Sort", "Blender 1", 200, 2782, 200, 1529, 1529],
    ["Blender 2", "Tim Sort", "Blender 2", 200, 2643, 200, 1529, 1529],
    ["Blender 0", "Tim Sort", "Blender 0", 300, 3758, 300, 2469, 2469],
    ["Blender 1", "Tim Sort", "Blender 1", 300, 3977, 300, 2469, 2469],
    ["Blender 2", "Tim Sort", "Blender 2", 300, 3621, 300, 2469, 2469],
    ["Blender 0", "Tim Sort", "Blender 0", 400, 6190, 400, 3458, 3458],
    ["Blender 1", "Tim Sort", "Blender 1", 400, 5914, 400, 3458, 3458],
    ["Blender 2", "Tim Sort", "Blender 2", 400, 5865, 400, 3458, 3458],
    ["Blender 0", "Tim Sort", "Blender 0", 600, 7662, 600, 5537, 5537],
    ["Blender 1", "Tim Sort", "Blender 1", 600, 8120, 600, 5537, 5537],
    ["Blender 2", "Tim Sort", "Blender 2", 600, 7532, 600, 5537, 5537],
    ["Blender 0", "Tim Sort", "Blender 0", 800, 10925, 800, 7715, 7715],
    ["Blender 1", "Tim Sort", "Blender 1", 800, 12058, 800, 7715, 7715],
    ["Blender 2", "Tim Sort", "Blender 2", 800, 10833, 800, 7715, 7715],
    ["Blender 0", "Tim Sort", "Blender 0", 1000, 16082, 1000, 9966, 9966],
    ["Blender 1", "Tim Sort", "Blender 1", 1000, 17400, 1000, 9966, 9966],
    ["Blender 2", "Tim Sort", "Blender 2", 1000, 17175, 1000, 9966, 9966],
    ["Blender 0", "Tim Sort", "Blender 0", 1500, 24255, 1500, 15826, 15826],
    ["Blender 1", "Tim Sort", "Blender 1", 1500, 21285, 1500, 15826, 15826],
    ["Blender 2", "Tim Sort", "Blender 2", 1500, 22918, 1500, 15826, 15826],
    ["Blender 0", "Tim Sort", "Blender 0", 2000, 33890, 2000, 21932, 21932],
    ["Blender 1", "Tim Sort", "Blender 1", 2000, 32832, 2000, 21932, 21932],
    ["Blender 2", "Tim Sort", "Blender 2", 2000, 33864, 2000, 21932, 21932],
    ["Blender 0", "Tim Sort", "Blender 0", 3000, 48235, 3000, 34652, 34652],
    ["Blender 1", "Tim Sort", "Blender 1", 3000, 47911, 3000, 34652, 34652],
    ["Blender 2", "Tim Sort", "Blender 2", 3000, 49938, 3000, 34652, 34652],
    ["Blender 0", "Tim Sort", "Blender 0", 4000, 76772, 4000, 47863, 47863],
    ["Blender 1", "Tim Sort", "Blender 1", 4000, 80414, 4000, 47863, 47863],
    ["Blender 2", "Tim Sort", "Blender 2", 4000, 69428, 4000, 47863, 47863],
    ["Blender 0", "Tim Sort", "Blender 0", 6000, 103474, 6000, 75304, 75304],
    ["Blender 1", "Tim Sort", "Blender 1", 6000, 100048, 6000, 75304, 75304],
    ["Blender 2", "Tim Sort", "Blender 2", 6000, 108933, 6000, 75304, 75304],
    ["Blender 0", "Tim Sort", "Blender 0", 8000, 154241, 8000, 103726, 103726],
    ["Blender 1", "Tim Sort", "Blender 1", 8000, 165882, 8000, 103726, 103726],
    ["Blender 2", "Tim Sort", "Blender 2", 8000, 144767, 8000, 103726, 103726],
  ],
  "tim-sort-random": [
    ["Random", "Tim Sort", "Random", 10, 41, 10, 33, 33],
    ["Random", "Tim Sort", "Random", 15, 98, 15, 59, 59],
    ["Random", "Tim Sort", "Random", 20, 168, 20, 86, 86],
    ["Random", "Tim Sort", "Random", 30, 330, 30, 147, 147],
    ["Random", "Tim Sort", "Random", 40, 356, 40, 213, 213],
    ["Random", "Tim Sort", "Random", 60, 741, 60, 354, 354],
    ["Random", "Tim Sort", "Random", 80, 885, 80, 506, 506],
    ["Random", "Tim Sort", "Random", 100, 1274, 100, 664, 664],
    ["Random", "Tim Sort", "Random", 150, 1808, 150, 1084, 1084],
    ["Random", "Tim Sort", "Random", 200, 2937, 200, 1529, 1529],
    ["Random", "Tim Sort", "Random", 300, 4111, 300, 2469, 2469],
    ["Random", "Tim Sort", "Random", 400, 6331, 400, 3458, 3458],
    ["Random", "Tim Sort", "Random", 600, 9217, 600, 5537, 5537],
    ["Random", "Tim Sort", "Random", 800, 13683, 800, 7715, 7715],
    ["Random", "Tim Sort", "Random", 1000, 19580, 1000, 9966, 9966],
    ["Random", "Tim Sort", "Random", 1500, 27493, 1500, 15826, 15826],
    ["Random", "Tim Sort", "Random", 2000, 41750, 2000, 21932, 21932],
    ["Random", "Tim Sort", "Random", 3000, 59308, 3000, 34652, 34652],
    ["Random", "Tim Sort", "Random", 4000, 89475, 4000, 47863, 47863],
    ["Random", "Tim Sort", "Random", 6000, 128149, 6000, 75304, 75304],
    ["Random", "Tim Sort", "Random", 8000, 190930, 8000, 103726, 103726],
  ],
}

export function preanalyzed(name: string): Analysis[] {
  return data[name] && data[name].map(deserialize)
}
